--[[
	签到活动
	唐实聪
	2016.6.3
]]
local MonthSignLayer = class("MonthSignLayer", function()
    local layer = cc.Sprite:create()
    layer:setContentSize( 720, 463 )
	return layer
end)

function MonthSignLayer:create( params )
	return self.new( params )
end

function MonthSignLayer:ctor( params )	
	local _params = params or {}
    self._activityType = _params.activityType
	self._timeInfo = _params.activeInfo or {}
	self._size = self:getContentSize()
	self._rewardList = {}
	for k,v in pairs(_params.table or {}) do
		local _reward = {}
		local _list = string.split( v.items, "," )
		for j, u in ipairs( _list ) do
			local _data = string.split( u, "|" )
			_reward[#_reward + 1] = {
				itemType = _data[1],
				itemId = _data[2],
				itemNum = _data[3],
			}
		end
		table.insert(self._rewardList, {
			id = tonumber(k),
			reward = _reward
		})
	end

	self:buildStatus( _params.status )
	self:initUI()
end

function MonthSignLayer:buildStatus( status )
	self._status = status or {}
	PlayerData:setSecondDaySignStatus(status)
	zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_REDORBUBBLE_CHANGE, data = {tag = "activityShow"}})
	table.sort(self._rewardList, function( a, b )
		local _idA = tonumber(a.id) or 0
		local _idB = tonumber(b.id) or 0
		local _statusA = tonumber(self._status[tostring(_idA)]) or 0
		local _statusB = tonumber(self._status[tostring(_idB)]) or 0
		if _statusA ~= _statusB then
			return ( _statusA + 1 )%3 > ( _statusB + 1 )%3
		end
		return _idA < _idB
	end)
end

function MonthSignLayer:initUI()
	-- tableview
	local tableview = zc.ZCTableView:create( cc.size( self._size.width, 403 ) )
    tableview:setBounceable( true )
    tableview:setInertia( true )
    tableview:setDirection( cc.SCROLLVIEW_DIRECTION_VERTICAL )
    tableview:setPosition( cc.p( 0, 60 ) )
    tableview:setDelegate()
    tableview:setVerticalFillOrder( cc.TABLEVIEW_FILL_TOPDOWN )
    local cellWidth = self._size.width
    local cellHeight = 160
    local function cellSizeForTable(table, index) 
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex(table, index)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            self:buildCell( cell, cellWidth, cellHeight )
        end
        self:updateCell( cell, index + 1 )

        return cell
    end
    local function numberOfCellsInTableView()
    	return #self._rewardList
    end
    tableview:registerScriptHandler( cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX )
    tableview:registerScriptHandler( tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX )
    tableview:registerScriptHandler( numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW )
    self:addChild(tableview)
    self._tableview = tableview
    tableview:reloadData()

    -- bottom
    local _file = "res/Main/role/trinket/sellDi.png"
    local _rect = cc.rect(20, 10, 549, 71)
    if self._activityType == 1 then 
        _file = "res/Main/activity/newDi.png"
        _rect = cc.rect(25, 10, 30, 30)
    end
    local bottom = ccui.Scale9Sprite:create(_rect, _file)
    if self._activityType == 1 then 
        bottom:setContentSize(710, 62)
        bottom:setAnchorPoint(0.5, 0)
        bottom:setPosition(self._size.width*0.5, 3)
    else
        bottom:setContentSize(728, 68)
        bottom:setAnchorPoint(0, 0)
        bottom:setPosition(-2, -5)
    end
    self:addChild( bottom )
    local bottomSize = bottom:getContentSize()
    -- 活动时间
    local timeBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
    timeBg:setContentSize( 315, 30 )
    timeBg:setPosition( 192, bottomSize.height/2 )
    bottom:addChild( timeBg )
    local _openTime = os.date("%m" .. zc.lang("date_month") .. "%d" .. zc.lang("date_day_1"), self._timeInfo.openTime )
    local _endTime = os.date("%m" .. zc.lang("date_month") .. "%d" .. zc.lang("date_day_1"), self._timeInfo.endTime )
    local timeLabel = zc.createLabel({
    	text = zc.lang("activity_time") .. ":    ".._openTime.."  ~  ".._endTime,
        fontSize = 20,
        color = display.COLOR_TAB_NORMAL_COLOR1,
        color2 = display.COLOR_TAB_NORMAL_COLOR2,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 37, bottomSize.height/2 ),
	})
    bottom:addChild( timeLabel )
end

function MonthSignLayer:buildCell( cell, cellWidth, cellHeight )
	-- bg
	local bg = cc.Sprite:create() 
	bg:setContentSize( 666, 156 )
	bg:setPosition( cellWidth/2, cellHeight/2 - 5 )
	cell:addChild( bg )
	cell._bg = bg
	local bgSize = bg:getContentSize()
	-- title
	local title = cc.Sprite:create( "res/Main/pub/name_bg.png" )
	title:setAnchorPoint( 0, 1 )
	title:setPosition( -2, bgSize.height + 3 )
	bg:addChild( title )
	-- titleLabel
	local titleLabel = zc.createLabel({
        fontSize = 20,
        stroke = true,
        pixel = 1.5,
        color = display.COLOR_BLACK,
        color2 = display.COLOR_TAB_NORMAL_COLOR1,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 15, title:getContentSize().height/2 ),
	})
	title:addChild( titleLabel )
	cell._titleLabel = titleLabel
	-- icon
	local icons = cc.Sprite:create()
	icons:setContentSize( 440, cellHeight )
	icons:setAnchorPoint( 0, 0.5 )
	icons:setPosition( 20, cellHeight/2 )
	bg:addChild( icons )
	cell._icons = icons
	-- btn
	local btn = zc.createCommomBtn({
		tag = 1,
		text = zc.lang("fetch_2"),
		pos = cc.p( 553, bgSize.height/2 ),
		endCallback = function()
			local _idx = cell._index
			zc.sendHttp({
				apiName = "MonthSign_getGift",
				params = {id = _idx},
				successCallback = function(backData)
                    zc.createShowReward(backData)
                    backData.noAnimation = true
	                PlayerData:updataData( backData )
                    -- 获得兵器武将
                    local _weaponInfo = nil
                    local _heroId = nil
                    for i, v in ipairs( backData.clientItems ) do
                        local _mainType = tonumber( v.type ) or 0
                        if _mainType == 3 then
                            _weaponInfo = PlayerData:getWeaponInfoInfoById( v.model.id )
                            break
                        elseif _mainType == zc.mainTypeSoldier then
                            _heroId = v.itemId
                            break
                        end
                    end
                    if _heroId then
                        zc.createPubShowGetHero({id=_heroId})
                    elseif _weaponInfo then
                        zc.createPop("makeGodWeapon", {info = _weaponInfo, onlyShow = true})
                    end

					self:buildStatus( backData.status )
					self._tableview:reloadDataAndScrollToCurrentCell()
				end,
			})
		end
	})
	bg:addChild( btn )
	cell._btn = btn
	-- 已领取
	local fetched = cc.Sprite:create( "res/Main/activity/monthSign/fetched.png" )
	fetched:setPosition( btn:getPosition() )
	bg:addChild( fetched )
	cell._fetched = fetched
end

function MonthSignLayer:updateCell( cell, index )
	local _data = self._rewardList[index]
	local _status = self._status[tostring(_data.id)]
	-----------
	cell._index = _data.id
	cell._bg:setTexture( "res/Main/activity/cell_bg_"..( index % 3 )..".png" )
	cell._titleLabel:setText(zc.lang("signIn") .. string.format(zc.lang("di_x_day"), _data.id))

	self:refreshReward( cell._icons, _data.reward )
	if _status == 0 then
		-- 不可领取
		cell._btn:setEnable( false )
		cell._btn:setVisible( true )
		cell._btn:setText(zc.lang("un_achieve"))
		cell._fetched:setVisible( false )
	elseif _status == 1 then
		-- 可领取
		cell._btn:setEnable( true )
		cell._btn:setVisible( true )
		cell._btn:setText(zc.lang("fetch_2"))
		cell._fetched:setVisible( false )
	elseif _status == 2 then
		-- 已领取
		cell._btn:setVisible( false )
		cell._fetched:setVisible( true )
	end
end

function MonthSignLayer:refreshReward( icons, data )
	if not data then
		return
	end
	icons:removeAllChildren()
	for i, v in ipairs( data ) do
		if v and v.itemType then
	    	local itemType = tonumber(v.itemType)
	        local _showItem = true
	        local _showHero = nil
	        if tonumber(itemType) == zc.mainTypeSoldier then
	            _showItem = false
	            _showHero = function() zc.createHeroInfoPop({heroId = v["itemId"], showHero = true}) end
	        end
	        local _icon = zc.createIconByData({
	            data = {
		            mainType = v.itemType,
		            type = v.itemId, 
		            id = v.itemId,
		            num = v.itemNum,
	            }, 
	            showDetailPop = _showItem,
	        	listener = _showHero,
		        pos = cc.p( 88*( i - 0.5 ), icons:getContentSize().height/2 ),
	        })
			_icon:setScale( 0.65 )
			icons:addChild( _icon )
		end
	end
end

return MonthSignLayer