--[[
	演武榜
	唐实聪
	2016.3.17
]]
local ArenaMainLayer = class("ArenaMainLayer", function ()
    return ZCDialog:create()
end)

function ArenaMainLayer:create( params )
	return ArenaMainLayer.new( params )
end

function ArenaMainLayer:ctor( data )
	self:initData( data )
	self:initUI()

	self:refreshMyInfo()
	self:refreshChallengeTimes()
	self:refreshChangeConsume()
	for i = 1, #self._player do
		self:refreshPlayer( i, true )
	end
end

function ArenaMainLayer:onEnter()
	if self._refresh then
		self:refresh()
		self._refresh = false
	end
	--功能引导
	self:guideFunc_1()
end

function ArenaMainLayer:onCleanup()
    self:removeAllChildren()
	local _forms = PlayerData:getNowFormationFiles()
	local _remove = {}
	local adm = ccs.ArmatureDataManager:getInstance()
    for i, v in ipairs( self._record ) do
        -- 如果已上阵，不删除
        local _path, _resId = zc.battle.getHeroArmaturePathByResId( v[1], v[2] )
        if not _forms[_resId] and not _remove[_path] then
            adm:removeArmatureFileInfo( _path )
            _remove[_path] = true
        end
    end
	helper.collectMemory()
end

function ArenaMainLayer:initData( data )
	-- dump( data, "ArenaMainLayer" )
	-- 当前已购买的挑战次数
	self._challengeNumBuy   = tonumber( data.challengeNumBuy )
	-- 当前已挑战次数
	self._challengeNum      = tonumber( data.challengeNum )
	-- 挑战次数上限
	self._challengeNumLimit = tonumber( data.challengeNumLimit )
	-- 挑战玩家信息列表
	self._users             = data.users
	table.sort(self._users, function( a, b )
		return tonumber( a.index ) < tonumber( b.index )
	end)
	-- 挑战成功获取积分比例
	self._winScoreRate 		= tonumber( data.conf.addScoresRate )
	-- 时间信息
	self._timeInfo          = data.timeInfo
	-- 所属分组
	self._type              = data.type or 1
	-- 排名
	self._rank              = data.rank or 0
	-- 积分
	self._score             = data.scores or 0
	-- 刷新银币
	self._flushMoney        = data.flushMoney
	-- 防守阵容英雄
	self._defenceHero 		= data.defensive or {}
	-- 防守阵容阵法
	self._defenceMatrix 	= tonumber( data.defensiveFormationId ) or 0

	if inArray( self._type, { 1, 2, 3 } ) then
		-- 预选赛
		self._isFinal = false
	elseif inArray( self._type, { 11, 12 } ) then
		-- 决赛
		self._isFinal = true
	end
	
	-- 英雄缩放比例
	self._scale             = 1.2
	
	-- 动画时间
	self._enterActionTime 	= 1

	local _size = self:getContentSize()
	self._width = _size.width
	self._height = _size.height

	self._record = {}
end
function ArenaMainLayer:initUI()
	-- 背景
	local background = cc.Sprite:create( "res/Main/arena/background.jpg" )
	background:setPosition( self._width/2, self._height/2 )
	self:addChild( background )
	zc.setBgSize( background )
	-- 云
    zc.getYunDongTai(nil,2,function(armUp, armDown)
        if not tolua.isnull(self) then
            self:addChild( armUp, 3 )
            self:addChild( armDown, 3 )
        end
    end)


	-- 标题层
	local _ltLayerWidth = self._width
	local _ltLayerHeight = self._height
	local _ltLayer = cc.Sprite:create()
	_ltLayer:setContentSize( _ltLayerWidth, _ltLayerHeight )
	_ltLayer:setAnchorPoint( 0, 1 )
	_ltLayer:setPosition( -50, self._height )
	self:addChild( _ltLayer, 3 )
	self._ltLayer = _ltLayer
	-- 标题
	local _titlePath = "res/Main/arena/title_1.png"
	if self._isFinal then
		-- 决赛
		_titlePath = "res/Main/arena/title_2.png"
	end
	local _title = cc.Sprite:create( _titlePath )
	_title:setAnchorPoint( 0, 1 )
	_title:setPosition( 0, _ltLayerHeight + 50 )
	_title:setCascadeOpacityEnabled( true )
	_ltLayer:addChild( _title )
	_ltLayer._title = _title
	-- 组
	local _groupText = zc.createLabel({
		text = "["..LANGUAGE_ARENA["arena_type_"..self._type].."]",
		fontSize = 26,
		color = display.COLOR_YELLOW,
        outLine = true,
        pixel = 2,
	})
	_groupText:setAnchorPoint( 0, 0.5 )
	_groupText:setPosition( 390, _ltLayerHeight - 48 )
	_ltLayer:addChild( _groupText )
	_ltLayer._groupText = _groupText
	-- 我的排名
	local _myRankText = zc.createLabel({
        text = LANGUAGE_ARENA.rank_mine,
        fontSize = 22,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0, 0 ),
        pos = cc.p( 10, _ltLayerHeight - 151 ),
	})
	_ltLayer:addChild( _myRankText )
	-- 我的排名背景
	local _myRankNumBg = cc.Sprite:create( "res/Main/arena/purpleBg.png" )
	_myRankNumBg:setAnchorPoint( 0, 1 )
	_myRankNumBg:setPosition( -30, _myRankText:getPositionY() )
	_ltLayer:addChild( _myRankNumBg )
	-- 我的排名数字
	local _myRankNum = zc.createFntLabel({
		fnt = "res/Main/arena/fnt/arena_num.fnt",
        ap = cc.p( 0, 1 ),
        pos = cc.p( _myRankText:getPositionX(), _myRankText:getPositionY() - 13 ),
        addwidth = -8,
	})
	_ltLayer:addChild( _myRankNum )
	_ltLayer._myRankNum = _myRankNum
	-- 我的积分
	local _myScoreText = zc.createLabel({
        text = LANGUAGE_ARENA.score_mine,
        fontSize = 22,
        stroke = true,
        pixel = 2,
        ap = cc.p( 0, 1 ),
        pos = cc.p( _myRankText:getPositionX(), _myRankNum:getPositionY() - 30 ),
	})
	_ltLayer:addChild( _myScoreText )
	-- 我的积分数字
	local _myScoreNum = zc.createLabel({
        fontSize = 23,
        color = display.COLOR_GREEN,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0, 1 ),
        pos = cc.p( _myScoreText:getPositionX() + 7, _myScoreText:getPositionY() - _myScoreText:getContentSize().height ),
	})
	_ltLayer:addChild( _myScoreNum )
	_ltLayer._myScoreNum = _myScoreNum
	if self._isFinal then
		_myScoreText:setVisible( false )
		_myScoreNum:setVisible( false )
	end
	-- 动作
	_ltLayer:runAction(
		cc.Spawn:create(
			cc.EaseExponentialOut:create( cc.MoveTo:create( self._enterActionTime, cc.p( 0, self._height ) ) ),
			cc.FadeIn:create( self._enterActionTime )
		)
	)


	-- 按钮容器
	local _rtLayerWidth = 80
	local _rtLayerHeight = self._height
	local _rtLayer = cc.Sprite:create()
	_rtLayer:setContentSize( _rtLayerWidth, _rtLayerHeight )
	_rtLayer:setAnchorPoint( 1, 0 )
	_rtLayer:setPosition( self._width + 80, 0 )
	self:addChild( _rtLayer, 3 )
	self._rtLayer = _rtLayer
	-- 返回按钮
	local _back = zc.createTopBackButton({
		listener = function ()
			LayerManager.removeLayout()
		end
	})
	_back:setPosition( _rtLayerWidth, _rtLayerHeight )
	_rtLayer:addChild( _back )
	-- 名人榜背景
	local _hallFameBg = cc.Sprite:create( "res/Main/arena/btnBg.png" )
	local _hallFameBgSize = _hallFameBg:getContentSize()
	local _hallFameBgWidth = _hallFameBgSize.width
	local _hallFameBgHeight = _hallFameBgSize.height
	_hallFameBg:setPosition( _rtLayerWidth - 42, _back:getPositionY() - _back:getContentSize().height/2 - _hallFameBgHeight )
	_rtLayer:addChild( _hallFameBg )
	-- 名人榜
	local _hallFameBtn = zc.createPushButton({
		normalFile = "res/Main/arena/hallFame_up.png",
		selectedFile = "res/Main/arena/hallFame_down.png",
		needSwallow = false,
		endCallback = function()
			zc.createArenaHallFamePop()
		end,
	})
	_hallFameBtn:setPosition( _hallFameBgWidth/2, _hallFameBgHeight/2 )
	_hallFameBg:addChild( _hallFameBtn )
	-- 名人榜文字
	local _hallFameLabel = zc.createLabel({
        text = LANGUAGE_ARENA.btn_hallFame,
        fontSize = 20,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0.5, 0 ),
        pos = cc.p( _hallFameBtn:getContentSize().width/2, 0 ),
	})
	_hallFameBtn:addChild( _hallFameLabel )
	-- 排行榜背景
	local _rankBg = cc.Sprite:create( "res/Main/arena/btnBg.png" )
	local _rankBgSize = _rankBg:getContentSize()
	local _rankBgWidth = _rankBgSize.width
	local _rankBgHeight = _rankBgSize.height
	_rankBg:setPosition( _hallFameBg:getPositionX(), _hallFameBg:getPositionY() - _hallFameBgHeight/2 - _rankBgHeight/2 - 5 )
	_rtLayer:addChild( _rankBg )
	-- 排行榜
	local _rankBtn = zc.createPushButton({
		normalFile = "res/Main/arena/rank_up.png",
		selectedFile = "res/Main/arena/rank_down.png",
		needSwallow = false,
		endCallback = function()
			--功能引导
			local _currentGID = getGuideManager():getCurrentGID()
			if _currentGID == GuideKey.KEY_12205 then
				zc.setGuideFuncNextStep(_currentGID)
			end
			zc.createArenaRankPop({type = self._type, rank = self._rank, score = self._score})
		end,
	})
	_rankBtn:setPosition( _rankBg:getContentSize().width/2, _rankBg:getContentSize().height/2 )
	_rankBg:addChild( _rankBtn )
	self._rankBtn = _rankBtn
	-- 排行榜文字
	local _rankLabel = zc.createLabel({
        text = LANGUAGE_ARENA.btn_rank,
        fontSize = 20,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0.5, 0 ),
        pos = cc.p( _rankBtn:getContentSize().width/2, 0 ),
	})
	_rankBtn:addChild( _rankLabel )
	_rtLayer:runAction(
		cc.Spawn:create(
			cc.EaseExponentialOut:create( cc.MoveTo:create( self._enterActionTime, cc.p( self._width, 0 ) ) ),
			cc.FadeIn:create( self._enterActionTime )
		)
	)


	-- 玩家
	self._player = {}
	for i = 1, 5 do
		self:buildPlayer( i )
	end


	-- 底部一排的y
	local _bottomY = 38

	-- 底部左侧
	local _lbLayerWidth = self._width
	local _lbLayerHeight = 76
	local _lbLayer = cc.Sprite:create()
	_lbLayer:setContentSize( _lbLayerWidth, _lbLayerHeight )
	_lbLayer:setAnchorPoint( 0, 0 )
	_lbLayer:setPosition( -50, 0 )
	self:addChild( _lbLayer, 3 )
	self._lbLayer = _lbLayer
	-- 剩余次数背景
	local _challengeBg = cc.Sprite:create( "res/Main/copy/leftTimesBg.png" )
	_challengeBg:setPosition( 105, _bottomY )
	_lbLayer:addChild( _challengeBg )
	-- 剩余次数文字
	local _challengeText = zc.createLabel({
        text = LANGUAGE_ARENA.times_left,
        fontSize = 20,
        stroke = true,
        pixel = 2,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 35, _challengeBg:getContentSize().height/2 ),
	})
	_challengeBg:addChild( _challengeText )
	-- 剩余次数数字
	local _challengeLabel = zc.createLabel({
        fontSize = 23,
        color = display.COLOR_GREEN,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( _challengeText:getPositionX() + _challengeText:getContentSize().width + 10, _challengeText:getPositionY() ),
	})
	_challengeBg:addChild( _challengeLabel )
	_lbLayer._challengeLabel = _challengeLabel
	-- 购买次数按钮
	local _challengeBuyBtn = zc.createPushButton({
		normalFile = "res/Main/common/button/common_btn2_u.png",
		selectedFile = "res/Main/common/button/common_btn2_d.png",
		disableFile = "res/Main/common/button/common_btn2_g.png",
		needSwallow = false,
		endCallback = function()
			self:clickBuyTimesBtnCallback()
		end,
		pos = cc.p( 220, _bottomY )
	})
	_lbLayer:addChild( _challengeBuyBtn )
	-- 动作
	_lbLayer:runAction(
		cc.Spawn:create(
			cc.EaseExponentialOut:create( cc.MoveTo:create( self._enterActionTime, cc.p( 0, 0 ) ) ),
			cc.FadeIn:create( self._enterActionTime )
		)
	)


	-- 底部中间
	local _mbLayerWidth = self._width
	local _mbLayerHeight = 76
	local _mbLayer = cc.Sprite:create()
	_mbLayer:setContentSize( _mbLayerWidth, _mbLayerHeight )
	_mbLayer:setAnchorPoint( 0, 0 )
	_mbLayer:setPosition( 0, -76 )
	self:addChild( _mbLayer, 3 )
	self._mbLayer = _mbLayer
	-- 数字
	local _moneyNum = zc.createLabel({
        text = self._flushMoney,
        fontSize = 24,
        outLine = true,
        pixel = 2,
        ap = cc.p( 1, 0.5 ),
        pos = cc.p( _mbLayerWidth/2 - 20, _bottomY ),
	})
	_mbLayer:addChild( _moneyNum )
	_mbLayer._moneyNum = _moneyNum
	-- 银币
	local _moneyIcon = cc.Sprite:create( "res/Main/common/common_icon_money.png" )
	_moneyIcon:setPosition( _moneyNum:getPositionX() - _moneyNum:getContentSize().width - _moneyIcon:getContentSize().width/2, _bottomY )
	_mbLayer:addChild( _moneyIcon )
	_mbLayer._moneyIcon = _moneyIcon
	-- 换一批
	local _changeBtn = zc.createCommomBtn({
        tag = 1,
        text = LANGUAGE_ARENA.btn_change,
        pos = cc.p( _mbLayerWidth/2 + 60, _bottomY ),
        endCallback = function ()
        	self:clickChangeBtnCallback()
        end,
    })
	_mbLayer:addChild( _changeBtn )
	-- 动作
	_mbLayer:runAction(
		cc.Spawn:create(
			cc.EaseExponentialOut:create( cc.MoveTo:create( self._enterActionTime, cc.p( 0, 0 ) ) ),
			cc.FadeIn:create( self._enterActionTime )
		)
	)

	-- 底部右侧
	local _rbLayerWidth = self._width
	local _rbLayerHeight = 80
	local _rbLayer = cc.Sprite:create()
	_rbLayer:setContentSize( _rbLayerWidth, _rbLayerHeight )
	_rbLayer:setAnchorPoint( 0, 0 )
	_rbLayer:setPosition( self._width + 300, 0 )
	self:addChild( _rbLayer, 3 )
	self._rbLayer = _rbLayer
	-- 防守阵容背景
	-- local _defenceBg = cc.Sprite:create( "res/Main/arena/btnBg.png" )
	-- _defenceBg:setPosition( _rbLayerWidth - _defenceBg:getContentSize().width/2 - 20, _bottomY + 10 )
	-- _rbLayer:addChild( _defenceBg )
	-- 防守阵容
	-- local _defenceBtn = zc.createPushButton({
	-- 	normalFile = "res/Main/arena/defence_up.png",
	-- 	selectedFile = "res/Main/arena/defence_down.png",
	-- 	needSwallow = false,
	-- 	endCallback = function()
	-- 		local params = {
	-- 		type = MatrixType.ArenaDefence, 
	-- 		data = {hero = self._defenceHero, matrix = self._defenceMatrix}, 
	-- 		callback = function( backData )
	-- 			-- dump(backData, "报错那：")
	-- 			PlayerData:setFormationsInfo(backData.formations)
	-- 			-- dump(PlayerData:getFormationsInfo(), "是什么呢：")
	-- 			-- self._defenceHero = backData.defensive or {}
	-- 			-- self._defenceMatrix = tonumber( backData.defensiveFormationId ) or 0
	-- 		end}
	-- 		zc.createBuzhenLayer(params)
	-- 	end,
	-- })
	-- _defenceBtn:setPosition( _defenceBg:getContentSize().width/2, _defenceBg:getContentSize().height/2 )
	-- _defenceBg:addChild( _defenceBtn )
	-- 防守阵容文字
	-- local _defenceLabel = zc.createLabel({
 --        text = LANGUAGE_ARENA.btn_defence,
 --        fontSize = 20,
 --        outLine = true,
 --        pixel = 2,
 --        ap = cc.p( 0.5, 0 ),
 --        pos = cc.p( _defenceBtn:getContentSize().width/2, 0 ),
	-- })
	-- _defenceBtn:addChild( _defenceLabel )
	-- 战报背景
	local _recordBg = cc.Sprite:create( "res/Main/arena/btnBg.png" )
	_recordBg:setPosition( _rbLayerWidth - 20 - _recordBg:getContentSize().width/2, _bottomY + 10 )
	_rbLayer:addChild( _recordBg )
	-- 战报
	local _recordBtn = zc.createPushButton({
		normalFile = "res/Main/arena/record_up.png",
		selectedFile = "res/Main/arena/record_down.png",
		needSwallow = false,
		endCallback = function()
			zc.createArenaRecordPop()
		end,
	})
	_recordBtn:setPosition( _recordBg:getContentSize().width/2, _recordBg:getContentSize().height/2 )
	_recordBg:addChild( _recordBtn )
	-- 战报文字
	local _recordLabel = zc.createLabel({
        text = LANGUAGE_ARENA.btn_record,
        fontSize = 20,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0.5, 0 ),
        pos = cc.p( _recordBtn:getContentSize().width/2, 0 ),
	})
	_recordBtn:addChild( _recordLabel )
	-- 商店背景
	local _shopBg = cc.Sprite:create( "res/Main/arena/btnBg.png" )
	_shopBg:setPosition( _recordBg:getPositionX() - _recordBg:getContentSize().width/2 - _shopBg:getContentSize().width/2 - 5, _bottomY + 10 )
	_rbLayer:addChild( _shopBg )
	-- 商店
	local _shopBtn = zc.createPushButton({
		normalFile = "res/Main/arena/shop_up.png",
		selectedFile = "res/Main/arena/shop_down.png",
		needSwallow = false,
		endCallback = function()
			zc.createArenaShopLayer()
		end,
	})
	_shopBtn:setPosition( _shopBg:getContentSize().width/2, _shopBg:getContentSize().height/2 )
	_shopBg:addChild( _shopBtn )
	_shopBtn = _shopBtn
	-- 商店文字
	local _shopLabel = zc.createLabel({
        text = LANGUAGE_ARENA.btn_shop,
        fontSize = 20,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0.5, 0 ),
        pos = cc.p( _shopBtn:getContentSize().width/2, 0 ),
	})
	_shopBtn:addChild( _shopLabel )
	_rbLayer:runAction(
		cc.Spawn:create(
			cc.EaseExponentialOut:create( cc.MoveTo:create( self._enterActionTime, cc.p( 0, 0 ) ) ),
			cc.FadeIn:create( self._enterActionTime )
		)
	)
end
-- 创建选手
function ArenaMainLayer:buildPlayer( index )
	-- 容器
	local _playerWidth = 406
	local _playerHeight = 604
	local _player = cc.Sprite:create()
	_player:setContentSize( _playerWidth, _playerHeight )
	_player:setPosition( self._width + 220, self._height/2 )
	self:addChild( _player, 2 )
	self._player[index] = _player
	local _posX = _playerWidth/2 + 10
	local _posY = 240
	-- 背景
	local _playerBg = cc.Sprite:create( "res/Main/arena/playerBg.png" )
	_playerBg:setPosition( _playerWidth/2, _playerHeight/2 )
	_player:addChild( _playerBg )
	_player._playerBg = playerBg
	-- 背景
	local _genderBg = cc.Sprite:create()
	_genderBg:setPosition( _playerWidth/2, _playerHeight/2 )
	_player:addChild( _genderBg )
	_player._genderBg = _genderBg
	-- 光圈
	local _genderLight = cc.Sprite:create()
	_genderLight:setPosition( _posX - 20, _posY + 5 )
	_player:addChild( _genderLight )
	_player._genderLight = _genderLight
	-- 英雄
	local _killed = cc.Sprite:create( "res/Main/arena/killed.png" )
	_killed:setAnchorPoint( 0.5, 0 )
	_killed:setPosition( _genderLight:getPosition() )
	_player:addChild( _killed, 2 )
    _player._killed = _killed
    local _heroBtn = zc.createPushButton({
    	touchSize = cc.size( 100, 150 ),
    	anchor = cc.p( 0.5, 0 ),
    	pos = cc.p( _genderLight:getPositionX(), _genderLight:getPositionY() ),
		endCallback = function()
			local _currentGID = getGuideManager():getCurrentGID()
			if _currentGID == GuideKey.KEY_12203 then
				zc.setGuideFuncNextStep(_currentGID)
			end
			if self._challengeNumLimit + self._challengeNumBuy - self._challengeNum <= 0 then
				zc.tipInfo( LANGUAGE_ARENA.times_not_enough )
			elseif self._users and self._users[index] and self._users[index].getScores and tonumber( self._users[index].getScores ) > 0 then
				zc.tipInfo( LANGUAGE_ARENA.challenged_player )
			else
				self:challenge( index )
			end
			--功能引导
			
		end
	})
	_player:addChild( _heroBtn )
	_heroBtn:setContentSize( cc.size( 100, 150 ) )
	if index == 3 then
		self._guide_heroBtn = _heroBtn
	end

	-- 姓名
	local _playerName = zc.createLabel({
        fontSize = 18,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0, 0.5 ),
	})
	_player:addChild( _playerName )
	_player._playerName = _playerName
	-- 积分
	local _playerScore = zc.createLabel({
        fontSize = 18,
        ap = cc.p( 1, 0.5 ),
        color = cc.c3b( 255, 221, 35 ),
	})
	_player:addChild( _playerScore, 1 )
	_player._playerScore = _playerScore
	-- 分隔线1
	local _splitLine1 = cc.Sprite:create( "res/Main/arena/playerSplit.png" )
	_splitLine1:setPosition( _posX - 50, _posY - 55 )
	_player:addChild( _splitLine1 )
	-- 排名
	local _rankBg = cc.Sprite:create( "res/Main/arena/redBg.png" )
	_rankBg:setPosition( _posX - 55, _posY - 75 )
	_player:addChild( _rankBg )
	_player._rankBg = _rankBg
	local _rankNum = zc.createFntLabel({
		fnt = "res/Main/arena/fnt/arena_num.fnt",
        ap = cc.p( 0.5, 0.5 ),
        pos = cc.p( _rankBg:getContentSize().width/2, _rankBg:getContentSize().height/2 - 5 ),
        addwidth = -8,
	})
	_rankBg:addChild( _rankNum )
	_player._rankNum = _rankNum
	-- 查看
	local _lookupBtn = zc.createPushButton({
		normalFile = "res/Main/arena/lookup.png",
		pos = cc.p( _posX + 20, _posY - 75 ),
		touchSize = cc.size( 50, 50 ),
		endCallback = function()
			if self._users[index].isNpc == 1 then
				zc.tipInfo(zc.lang("tips_unable_look_player"))
			else
				zc.createArenaLookupPop({userData = self._users[index], winScore = math.floor( tonumber( self._users[index].scores or 0 )*self._winScoreRate )})
			end
		end
	})
	_player:addChild( _lookupBtn )
	_player._lookupBtn = _lookupBtn
	-- 分隔线2
	local _splitLine2 = cc.Sprite:create( "res/Main/arena/playerSplit.png" )
	_splitLine2:setPosition( _posX - 60, _posY - 95 )
	_player:addChild( _splitLine2 )
	-- 战斗力图片
    local _fightingSp = cc.Sprite:create( "res/Main/role/common/fight_bg.png" )
    _fightingSp:setPosition( _posX - 45, _posY - 123 )
    _fightingSp:setScale( 0.7 )
	_player:addChild( _fightingSp )
    _player._fightingSp = _fightingSp
	-- 战斗力数字
	local _fightingNum = zc.createFntLabel({
        fnt = "res/Main/common/fnt/fight_change.fnt",
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 90, _fightingSp:getContentSize().height/2 ),
        addwidth = -14,
    })
    _fightingSp:addChild( _fightingNum )
    _fightingNum:setScale( 0.7 )
    _player._fightingNum = _fightingNum
	-- 分隔线3
	local _splitLine3 = cc.Sprite:create( "res/Main/arena/playerSplit.png" )
	_splitLine3:setPosition( _posX - 70, _posY - 151 )
	_player:addChild( _splitLine3 )

	return _player
end
-- 刷新选手
function ArenaMainLayer:refreshPlayer( index, animationFlag )
	local _player = self._player[index]
	if not _player then
		return
	end
	local _data = self._users[index]
	if not _data then
		_player:setVisible( false )
		return
	end
	if animationFlag then
		self:initAnimation( _player, index )
	end
	-- dump(_data,"refreshPlayer")
	local _posX = _player:getContentSize().width/2 + 10
	local _posY = 240
	local _modelId = ""
	local _sex = tonumber( _data.seigniorId ) or 1
	if _sex == 1 then
		-- 男
		-- 光圈
		_player._genderLight:setTexture( "res/Main/arena/maleLight.png" )
		-- 英雄
		_modelId = gameData.getDataFromCacheWithPrimaryKey("hero", 101).modelId
	else
		-- 女
		-- 光圈
		_player._genderLight:setTexture( "res/Main/arena/femaleLight.png" )
		-- 英雄
		_modelId = gameData.getDataFromCacheWithPrimaryKey("hero", 102).modelId
	end
	local _genderBgPath = zc.battle.checkModel( _modelId, _data.weaponId )
	_player._genderBg:setTexture( "res/Main/arena/".._genderBgPath..".png" )
	-- 英雄
	if _player._hero then
		_player._hero:removeFromParent()
		_player._hero = nil
	end
	local _armature = zc.battle.createHeroArmatureWithResId( _modelId, _data.weaponId )
	zc.soldierChangeWeapon( _armature, _data.weaponId )
	self._record[#self._record + 1] = {
		_modelId, _data.weaponId
	}
    _armature:setPosition( _player._genderLight:getPosition() )
    _armature:setScale( self._scale )
    _armature:getAnimation():play("idle")
	_player:addChild( _armature )
    _player._hero = _armature
	-- 姓名积分
	_player._playerName:setText( _data.userName )
	_player._playerScore:setText( "("..( _data.scores or 0 )..LANGUAGE_ARENA.score..")" )
	-- 排名
	_player._rankNum:setString( self:getRankString( _data.rank or 0 ) )
	-- 设置位置
	if not self._isFinal then
		-- 预选赛
		_player._playerName:setPosition( _posX - 40 - _player._playerName:getContentSize().width/2, _posY - 40 )
		_player._playerScore:setPosition( _player._rankBg:getPositionX() + _player._playerScore:getContentSize().width/2, _player._rankBg:getPositionY() - 5 )
		_player._rankNum:setVisible( false )
		_player._playerScore:setVisible( true )
	else
		-- 决赛
		local _totalWidth = _player._playerName:getContentSize().width + _player._playerScore:getContentSize().width
		_player._playerName:setPosition( _posX - 40 - _totalWidth/2, _posY - 40 )
		_player._playerScore:setPosition( _posX - 40 + _totalWidth/2, _posY - 40 )
		_player._rankNum:setVisible( true )
		_player._playerScore:setVisible( false )
	end
	-- 查看
	-- 战斗力
	_player._fightingNum:setString( _data.fighting )

	-- 置灰
    if self._users[index].getScores and tonumber( self._users[index].getScores ) > 0 then
    	zc.setGray( _player._playerBg )
    	zc.setGray( _player._genderBg )
    	zc.setGray( _player._genderLight )
    	zc.setArmatureGray( _armature )
    	_player._killed:setVisible( true )
    	_player._playerScore:setColor( display.COLOR_GRAY )
    	zc.setGray( _player._rankBg )
    	zc.setGray( _player._fightingSp )
    	zc.setGray( _player._fightingNum )
    else
    	zc.setGray( _player._playerBg, false )
    	zc.setGray( _player._genderBg, false )
    	zc.setGray( _player._genderLight, false )
    	zc.setArmatureGray( _armature, false )
    	_player._killed:setVisible( false )
    	_player._playerScore:setColor( cc.c3b( 255, 221, 35 ) )
    	zc.setGray( _player._rankBg, false )
    	zc.setGray( _player._fightingSp, false )
    	zc.setGray( _player._fightingNum, false )
    end
end
-- 挑战
function ArenaMainLayer:challenge( index )
	self._refresh = true
	-- dump( self._users[index], "self._users[index]" )
	local enemyIndex = self._users[index].index
	local enemys = self._users[index].heros
	local enemysData = {}
	if self._users[index].isNpc and tonumber( self._users[index].isNpc ) == 1 then
		-- robot
		for i, v in ipairs( enemys ) do
			local _weaponId = v.weaponId
			local _heroId = tonumber( v.heroId ) or 0
			if _heroId == 101 or _heroId == 102 then
				_weaponId = self._users[index].weaponId
			end
			enemysData[tonumber( v.location )] = {
				id = v.id,
				modelId = gameData.getDataFromCacheWithPrimaryKey("arena_robot_npc", v.heroId).modelId,
				weaponId = _weaponId,
			}
		end
	else
		-- 玩家
		for i, v in ipairs( enemys ) do
			local _weaponId = v.weaponId
			local _heroId = tonumber( v.heroId ) or 0
			if _heroId == 101 or _heroId == 102 then
				_weaponId = self._users[index].weaponId
			end
			enemysData[tonumber( v.location )] = {
				id = v.id,
				modelId = gameData.getDataFromCacheWithPrimaryKey("hero", v.heroId).modelId,
				weaponId = _weaponId,
			}
		end
	end
	-- 分享数据
	local shareData = {
		attackerInfo = {
			userId = PlayerData.info.userId,
			userName = PlayerData.info.userName,
		},
		defenderInfo = {
			userId = self._users[index].userId,
			userName = self._users[index].userName,
		},
		type = self._type,
	}
	local _dataInfo = {
		battleType= BattleType.ARENA,
		pvpIndex = enemyIndex,
		enemys = {["enemys"] = enemysData},
		shareData = shareData,
		fightScene = 6,
	}
	zc.createMatrixBattle( _dataInfo )
end

-- 选手的动画效果
function ArenaMainLayer:initAnimation( player, index )
	local oriPos = cc.p( self._width + 220, self._height/2 )
	player:setPosition( oriPos )
	local dstPos = cc.p( self._width/2 + ( index - 3 )*220 + 30, self._height/2 )
	local delayTime = 0.1
	local moveTime = 0.8
	player:runAction(
		cc.Sequence:create(
			cc.DelayTime:create( index*delayTime ),
			cc.EaseExponentialOut:create( cc.MoveTo:create( moveTime, dstPos ) )
		)
	)
end
function ArenaMainLayer:getRankString( rankNum )
	local rankStr = ""
	if tonumber( rankNum ) < 100 then
		rankStr = "NO."..tostring( rankNum )
	else
		rankStr = tostring( rankNum )
	end
	return rankStr
end

-- 刷新我的信息
function ArenaMainLayer:refreshMyInfo()
	local _ltLayer = self._ltLayer
	_ltLayer._myRankNum:setString( self:getRankString( self._rank ) )
	_ltLayer._myScoreNum:setText( self._score )
end
-- 刷新挑战剩余次数
function ArenaMainLayer:refreshChallengeTimes()
	local _lbLayer = self._lbLayer
	_lbLayer._challengeLabel:setText( ( self._challengeNumLimit + self._challengeNumBuy - self._challengeNum ).."/"..( self._challengeNumLimit ) )
end
-- 刷新换一批消耗
function ArenaMainLayer:refreshChangeConsume()
	local _mbLayer = self._mbLayer
	_mbLayer._moneyNum:setText( self._flushMoney )
end
-- 点击购买挑战次数回调
function ArenaMainLayer:clickBuyTimesBtnCallback()
	-- 购买挑战次数花费
	local _consumeData = gameData.getDataFromCache("consume_price", {type=20, subtype=( tonumber( self._challengeNumBuy ) + 1 )})
	self._comsumePrice = _consumeData.price or 0
	-- 购买弹窗
    zc.createBuyPop({
		tip = LANGUAGE_ARENA_BUY( self._comsumePrice ),
		special = {
			{
				begin = 5,
				length = 4,
				color = display.COLOR_YELLOW,
			}
		},
		consume = {
			[1] = {
				type = 1,
				num = self._comsumePrice,
			}
		},
		clickHide = true,
		callback = function( data )
			if self._challengeNumBuy >= self._challengeNum then
				-- 挑战次数已达到上限
				zc.tipInfo( LANGUAGE_ARENA.times_to_limit )
			else
        		zc.sendHttp({
        			apiName = "Arena_challengeBuy",
        			successCallback = function(data)
		    			-- 刷新资源
		                PlayerData:updataData( data )
	        			zc.tipInfo( zc.lang("tips_buy_success") )
	        			self._challengeNumBuy = data.challengeNumBuy
	        			self:refreshChallengeTimes()
				    end
			    })
        	end
		end
	})
end
-- 点击换一批回调
function ArenaMainLayer:clickChangeBtnCallback()
	zc.sendHttp({
		apiName = "Arena_flush",
		successCallback = function (data)
            -- 刷新资源
            PlayerData:updataData( data )
            -- 刷新时间信息
            self._timeInfo = data.timeInfo or {}
			-- 刷新玩家信息
			self._users = data.users
			table.sort(self._users, function( a, b )
				return a.index < b.index
			end)
			for i = 1, #self._users do
				self:refreshPlayer( i, true )
			end
        end,
    })
end

-- 战斗返回刷新界面
function ArenaMainLayer:refresh()
	zc.sendHttp({
		apiName = "Arena_main", 
		successCallback = function(data)
		
	        self:initData( data )

	        self:refreshMyInfo()
	        self:refreshChallengeTimes()
	        self:refreshChangeConsume()

	        for i = 1, #self._player do
				self:refreshPlayer( i )
			end
	    end
    })
end

--新手引导--功能引导
function ArenaMainLayer:guideFunc_1()    
   	local _currentGID = getGuideManager():getCurrentGID()
    if inArray(_currentGID, GuideGroup.DUI_HUA) then
        local function _callBack()
        	getGuideManager():addMaskLayer()
        	performWithDelay(self, function()
        		getGuideManager():removeMaskLayer()
	            getGuideManager():setObjTarget(self._guide_heroBtn)
	            getGuideManager():startGuide()
		    end, 0.3)
        end
        getGuideManager():setHeroFuncCallBack(_callBack)
        getGuideManager():startGuide()
    elseif _currentGID == GuideKey.KEY_12205 then
    	getGuideManager():addMaskLayer()
    	performWithDelay(self, function()
    		getGuideManager():removeMaskLayer()
	        getGuideManager():setObjTarget(self._rankBtn)
	        getGuideManager():startGuide()
	    end, 0.01)
    end
end

return ArenaMainLayer