--[[
    战斗布阵界面
    唐实聪
    2016.5.9
]]
local BattleUIMatrixLayer = class("BattleUIMatrixLayer", function()
    return ZCDialog:create()
end)

function BattleUIMatrixLayer:create( params )
    return self.new( params )
end

function BattleUIMatrixLayer:ctor( params )
    zc.createArmature({parentPath = "res/image/battle/dazhao_ef/", fileName = "dazhao_ef" , async = true})
    zc.createArmature({parentPath = "res/image/battle/ef_hit_jizhong/", fileName = "ef_hit_jizhong" , async = true})
    self._isFirstFight = tonumber(params.isFirstFight) == 1
    self._guideData = params.guideData
    self:initData( params )
    self:initUI()
    self:buildUpHeroData()
    self:refreshFormationInfo()
    self:refreshEnemy()

    print("BattleUIMatrixLayer .... ")
    -- dump(params)

end

function BattleUIMatrixLayer:onEnter()
    ---新手引导
    local _getGuideIndex = self:getGuideIndex()
    if self._guideData and _getGuideIndex > 6 then
        getGuideManager():setCurrentGID(GuideKey.KEY_KAIZHAN)
    end

    self:addGuide(self._fight)
end

function BattleUIMatrixLayer:initData( params )
    -- dump( params, "params!!" )
    -- 阵法点位置
    self._formationPoint = {}
    -- 起始点坐标
    local beginX = 50
    local beginY = 50
    -- 间隔
    local spaceX = 25
    local spaceY = 25
    -- 初始化坐标
    for x = 0, 2 do
        for y = 0, 2 do
            self._formationPoint[#self._formationPoint + 1] = cc.p( beginX - x*spaceX, beginY - y*spaceY )
        end
    end
    -- 阵法线转换矩阵
    self._formationLineTrans = {
        { 1, 1 }, { 1, 2 }, { 1, 3 },
        { 2, 1 }, { 2, 2 }, { 2, 3 },
        { 3, 1 }, { 3, 2 }, { 3, 3 },
    }

    -- 武将位置
    self._heroPos = {}
    for i = 1, 9 do
        self._heroPos[#self._heroPos + 1] = zc.battle.getInitialMatrixPoint({side = BATTLE_SIDE.LEFT, position = i})
    end
    -- 敌人位置
    self._enemyPos = {}
    for i = 1, 9 do
        self._enemyPos[#self._enemyPos + 1] = zc.battle.getInitialMatrixPoint({side = BATTLE_SIDE.RIGHT, position = i})
    end

    -- 武将脚下
    self._heroBottom = {}
    -- 敌人脚下
    self._enemyBottom = {}

    -- 武将
    self._hero = {}
    -- 武将按钮
    self._heroBtn = {}
    -- 武将检测
    self._detect = {}
    -- 敌人
    self._enemy = {}


    -- 所有阵法数据
    local playerLevel = tonumber( PlayerData.info.level )
    local playerVip   = tonumber( PlayerData.info.vip )
    local newForm = {}
    local levelOne = nil
    local vipOne = nil
    local __tmpInfo = clone(PlayerData:getFormationsInfo())
    for i, v in pairs(__tmpInfo) do
        if tonumber(v.needLevel) <= playerLevel and tonumber(v.needVip) <= playerVip then
            v["open"] = 1
            newForm[#newForm+1] = v
        else
            if tonumber(v.needVip) > playerVip then
                if vipOne then
                    if tonumber(vipOne.needVip) > tonumber(v.needVip) then
                        vipOne = v
                    end
                else
                    vipOne = v 
                end
            elseif tonumber(v.needLevel) > playerLevel then
                if levelOne then
                    if tonumber(levelOne.needLevel) > tonumber(v.needLevel) then
                        levelOne = v
                    end
                else
                    levelOne = v 
                end
            end
        end
    end
    newForm[#newForm+1] = levelOne
    newForm[#newForm+1] = vipOne
    table.sort(newForm, function(a,b)
        if tonumber(a.needVip) ~= tonumber(b.needVip) then
            return tonumber(a.needVip) < tonumber(b.needVip)
        else
            return tonumber(a.needLevel) < tonumber(b.needLevel)
        end
    end)
    self._formationList = newForm


    -- 找到状态为1的阵型，若没有状态为一的，即初始化时候阵容上没有武将，默认为5，表示初始化的阵型
    self._formationData = {}
    self._formationBack = ""
    self._formationIndex = 1
    for i, v in ipairs( self._formationList ) do
        if ( tonumber( v.status ) or 0 ) == 1 then
            self._formationData = v
            self._formationBack = self:getFormationString()
            self._formationIndex = i
        end
    end
    -- 武将列表
    self._heroList = {}
    -- 查询用的武将数据
    for i, v in ipairs( PlayerData:getGeneralInfo() ) do
        if tonumber(v.tactics.tactics) ~= 1 then
            if ( tonumber( v.status ) or 0 ) == 1 then
                -- 可以上阵
                local data = clone( v )
                -- data.upMatrix = upHero[tostring( data.id )]
                if tonumber( PlayerData.info.heroId ) == tonumber( data.id ) then
                    -- 是主角
                    data.isMainRole = true
                else
                    -- 不是主角
                    data.isMainRole = false
                end
                self._heroList[#self._heroList + 1] = data
            end
        end
    end
    -- 排序，只排一次
    table.sort(self._heroList, function( a, b )
        if a.isMainRole or b.isMainRole then
            -- 主角最前面
            return a.isMainRole
        else
            -- id排序
            return tonumber( a.fighting ) > tonumber( b.fighting )
        end
    end)
    -- dump(self._heroList,"战斗武将列表")
    -- 敌人数据
    self._enemyList = {}
    self._enemyData = {}
    for k, v in pairs( params.enemys or {} ) do
        if k == "enemys" then
            self._enemyList = v
        else
            self._enemyData[k] = v
        end
    end
    -- dump( self._enemyData, "self._enemyData" )
    --战斗场景
    self._fightScene = params.fightScene or 1
    -- 战斗类型
    self._battleType = params.battleType
    self._battleTypeData = {}
    -- 各类型自己需要的数据
    if self._battleType == BattleType.DrillField then
        -- 校场
        self._battleTypeData.drillPositionId = params.positionId
    elseif self._battleType == BattleType.PVE then
        -- 副本
        self._battleTypeData.tollgateId = params.tollgateId
    elseif self._battleType == BattleType.ARENA then
        -- 演武榜
        self._battleTypeData.pvpIndex = params.pvpIndex
        self._battleTypeData.shareData = params.shareData
    elseif self._battleType == BattleType.GodFight then
        -- 武神榜
        self._battleTypeData.toUserId = params.toUserId
        self._battleTypeData.mRank = params.mRank
    elseif self._battleType == BattleType.GrabFight then
        -- 抢粮榜
        self._battleTypeData.toUserId = params.toUserId
        self._battleTypeData.sType = params.sType
    elseif self._battleType == BattleType.WuShen_Tower then
        -- 武神塔
    elseif self._battleType == BattleType.QunXiong then
        -- 群雄
        self._battleTypeData.mainId = params.mainId
    elseif self._battleType == BattleType.TollgateHero then
        -- 名将副本
        self._battleTypeData.tollgateId = params.tollgateId
        self._battleTypeData.chapterId = params.chapterId
    elseif self._battleType == BattleType.WorldBoss then
        -- 世界boss
        self._battleTypeData.tollgateId = params.tollgateId
        self._battleTypeData.isForce = params.isForce
    elseif self._battleType == BattleType.WeaponCopy then
        self._battleTypeData.tollgateId = params.tollgateId
        self._battleTypeData.isBoss = params.isBoss
    elseif self._battleType == BattleType.Wushenshilian then 
        self._battleTypeData.tollgateId = params.tollgateId
    elseif self._battleType == BattleType.LimitChallenge then 
        self._battleTypeData.tollgateId = params.tollgateId
        self._battleTypeData.type = params.tollgateType
        if self._battleTypeData.type == 4 then
            self._limitChallenge4Flag = true
        end
    elseif self._battleType ==  BattleType.HeroCopy then
        --英雄试炼
        self._battleTypeData.tollgateId = params.tollgateId
    end

    -- 上阵英雄数量限制
    self._upHeroNumLimit = 5

    -- 武将移动状态
    self._moveIndex = 0


    -- 记录创建的武将
    self._recordHero = {}
    self._recordEnemy = {}

    self._size = self:getContentSize()

    self._exist = true
end
-- 构造上阵的武将数据
function BattleUIMatrixLayer:buildUpHeroData()
    -- 上阵武将数据
    local upHero = {}
    for i = 1, 9 do
        local id = tonumber( self._formationData["location"..i] ) or 0
        if id ~= 0 then
            upHero[tostring( id )] = true
        end
    end
    if table.nums( upHero ) == 0 then
        -- 没有人上阵，此时默认主角上阵
        local openMap = PlayerData:getFormationsOpenMap( self._formationData.formationId, self._formationData.level )
        local openArray = string.split( openMap, "|" )
        local position = 9
        for i, v in ipairs( openArray ) do
            position = position > tonumber( v ) and tonumber( v ) or position
        end
        self._formationData["location"..position] = PlayerData.info.heroId
        upHero[tostring( PlayerData.info.heroId )] = true
    end
    for i, v in ipairs( self._heroList ) do
        v.upMatrix = upHero[tostring( v.id )]
    end
    
    -- dump( self._heroList, "self._heroList" )
    self._heroTableview:reloadDataAndScrollToCurrentCell()
    self:refreshHero()
end

function BattleUIMatrixLayer:initUI()
    -- 背景
    local bg = zc.getFightScene( self._fightScene )
    bg:setPosition( self._size.width/2, 0 )
    self:addChild( bg )
    self._bgNode = bg

    -- 阴影层
    local shadow = cc.LayerColor:create( cc.c4b(0, 0, 0, 150) )
    shadow:setPosition( 0, 0 )
    shadow:setVisible( false )
    self:addChild( shadow )
    self._shadow = shadow

    -- 返回
    local back = zc.createTopBackButton({
        pos = cc.p( self._size.width - 60, self._size.height - 40 ), 
        listener = function()
            -- 当前的阵法数据
            local formationNew = self:getFormationString()
                -- 保存阵法后删除
                if self._limitChallenge4Flag then
                    LayerManager.removeLayout(nil, nil, false)
                    helper.releaseSpineCaches()
                else
                    zc.sendHttp({
                        apiName = "Formation_saveFormation",
                        params = {enableId = self._formationData.formationId, info = formationNew,type = 1},
                        successCallback = function(data)
                            if data.formations then
                                PlayerData:setFormationsInfo(data.formations)
                            end
                            self:removeArmature()
                            if self._battleType == BattleType.DrillField or
                                self._battleType == BattleType.PVE then
                                LayerManager.removeLayout(nil, nil, true)
                                -- 移除背景spine
                                helper.releaseSpineCaches()
                            else
                                print("-----22222")
                                if PlayerData.HeroCopyDetailPop then
                                    PlayerData.HeroCopyDetailPop = false
                                    LayerManager.removeLayout(nil, nil, true)
                                else
                                    LayerManager.removeLayout(nil, nil, false)
                                end
                                -- 移除背景spine
                                helper.releaseSpineCaches()
                            end
                        end,
                    })
                end
            -- end
        end,
    })
    self:addChild( back, 900 )

    -- 开战
    local fightArmature = zc.createSpine({
        fileName = "kaizhan",
        loop = true,
        actName = "idle",
    })
    fightArmature:setPosition( self._size.width - 140, 80 )
    self:addChild( fightArmature, 900 )
    local fight = ccui.Widget:create()
    fight:setContentSize( 280, 160 )
    fight:setPosition( fightArmature:getPosition() )
    fight:onTouch(
        function(event)
            if event.name == "began" then
                fightArmature:runAction(
                    cc.ScaleTo:create( 0.1, 0.9 )
                )
            elseif event.name == "moved" then
            elseif event.name == "ended" then
                musicManager.playEffect(zc.effect.btnFight)
                fightArmature:runAction(
                    cc.ScaleTo:create( 0.1, 1 )
                )
                self:startFight()
            elseif event.name == "cancelled" then
                fightArmature:runAction(
                    cc.ScaleTo:create( 0.1, 1 )
                )
            end
        end
    )
    fight:setTouchEnabled( true )
    self:addChild( fight, 900 )
    self._fight = fight

    -- 将领选择
    self:initHeroList()
    -- 阵法选择
    self:initFormationList()
    -- 触摸屏蔽层
    local _swallowWidth = self._size.width - 478 - 490
    local _swallowHeight = 125
    local _swallow = zc.createPushButton({
        touchSize = cc.size( _swallowWidth, _swallowHeight ),
        pos = cc.p( _swallowWidth/2 + 478, self._size.height - _swallowHeight/2 ),
        needSwallow = true,
    })
    self:addChild( _swallow, 700 )
    -- 脚下
    self:initBottom()

    -- 阵法信息
    local formationInfoBg = cc.Sprite:create( "res/Main/matrixBattle/formation_bg.png" )
    formationInfoBg:setPosition( self._size.width/2, 45 )
    self:addChild( formationInfoBg, 700 )
    --
    local upBtn = zc.createPushButton({
        normalFile = "res/Main/matrixBattle/up_arrow.png",
        touchScale = 0.9,
        pos = cc.p( 221, formationInfoBg:getContentSize().height/2 - 15),
        needSwallow = false,
        needEnableWhenMoving = true,
        endCallback = function()
            local _callback = function()
                -- self:refreshFormationInfo()
                self:changeMatrix(self._formationIndex)
            end
            zc.createSourcePop("BuzhenSelect", "src/layer/buzhen/BuzhenSelectPop.lua", {
                curMatrix = self._formationData, 
                useMatrixId = self._formationData["formationId"],
                endcallback = _callback,
            })
        end
    })
    formationInfoBg:addChild(upBtn)
    -- 等级
    local formationLevel = zc.createLabel({
        fontSize = 25,
        ap = cc.p(0.5, 0.5),
        pos = cc.p( 35, 28 ),
        stroke = true,
        pixel = 1,
        color = display.COLOR_WHITE,
    })
    formationInfoBg:addChild( formationLevel )
    self._formationLevel = formationLevel
    -- 名字
    local formationName = zc.createLabel({
        fontSize = 25,
        ap = cc.p(0.5, 0.5),
        pos = cc.p( 135, 28 ),
        stroke = true,
        pixel = 1,
        color = display.COLOR_WHITE,
    })
    formationInfoBg:addChild( formationName )
    self._formationName = formationName
    -- 总加成
    local formationTotalAddition = zc.createLabel({
        fontSize = 20,
        ap = cc.p(0.5, 0.5),
        pos = cc.p( 125, 70 ),
        outLine = true,
        pixel = 2,
        color = display.COLOR_GREEN,
    })
    formationInfoBg:addChild( formationTotalAddition )
    self._formationTotalAddition = formationTotalAddition
    -- 站位加成
    local formationSpecialAddition = zc.createLabel({
        fontSize = 20,
        ap = cc.p(0.5, 0.5),
        pos = cc.p( 125, 95 ),
        stroke = true,
        pixel = 2,
        color = display.COLOR_GREEN,
    })
    formationInfoBg:addChild( formationSpecialAddition )
    self._formationSpecialAddition = formationSpecialAddition

    --过关条件
    if self._battleType == BattleType.LimitChallenge then
        local _tollgateDb = gameData.getDataFromCacheWithPrimaryKey("specialTollgate", self._battleTypeData.tollgateId)
        self:createPassConditon(_tollgateDb)
    end
end

--过关条件
function BattleUIMatrixLayer:createPassConditon(_data)
    local _type = tonumber(_data["victoryType"]) or 1
    --过关条件
    local passConditonBg = cc.Sprite:create("res/Main/wushenTower/pass_condition_bg.png")
    passConditonBg:setAnchorPoint(0.5, 0.5)
    passConditonBg:setPosition(zc.width/2, zc.height-140)
    self:addChild(passConditonBg, 1)
    local passSp = cc.Sprite:create("res/Main/wushenTower/pass_condition.png")
    passSp:setAnchorPoint(0.5, 0.5)
    passSp:setPosition(passConditonBg:getContentSize().width/2, passConditonBg:getContentSize().height/2-32)
    passConditonBg:addChild(passSp, 1)
    local conditionStr = ""
    if _type == 2 or _type == 3 then
        conditionStr = _data.parameter
    end
    local conditionLab1 = zc.createLabel({
        text = conditionStr,
        outLine = true,
        fontSize = 24,
        ap = cc.p(0.5, 0.5),
        pos = cc.p(passConditonBg:getContentSize().width/2, passConditonBg:getContentSize().height/2),
        color = display.COLOR_RED,
    })
    passConditonBg:addChild(conditionLab1)
    local conditionLab2 = zc.createLabel({
        text = LANGUAGE_condition_uimatrix(_type),
        outLine = true,
        fontSize = 20,
        ap = cc.p(0, 0.5),
        pos = cc.p(passConditonBg:getContentSize().width/2, conditionLab1:getPositionY()),
        -- color = display.COLOR_RED,
    })
    passConditonBg:addChild(conditionLab2)
    conditionLab1:setPositionX(passConditonBg:getContentSize().width/2-conditionLab2:getContentSize().width/2)
    conditionLab2:setPositionX(conditionLab1:getPositionX()+conditionLab1:getContentSize().width/2)

end
-- 将领选择
function BattleUIMatrixLayer:initHeroList()
    -- bg
    local bg = cc.Sprite:create( "res/Main/matrixBattle/top_bg.png" )
    bg:setAnchorPoint( 0, 1 )
    bg:setPosition( -85, self._size.height + 40 )
    self:addChild( bg, 700 )
    -- tableview
    local tableviewSize = cc.size( 478, 125 )
    local tableview = zc.ZCTableView:create( tableviewSize )
    tableview:setDirection(cc.SCROLLVIEW_DIRECTION_HORIZONTAL)
    tableview:setVerticalFillOrder(cc.TABLEVIEW_FILL_TOPDOWN)
    tableview:setPosition( cc.p( 0, self._size.height - tableviewSize.height ) )
    tableview:setDelegate()
    self:addChild( tableview, 700 )
    local cellWidth = 95
    local cellHeight = tableviewSize.height
    local function cellSizeForTable(table, index)
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex(table, index)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
        end
        cell:removeAllChildren()

        local data = self._heroList[index + 1]
        if data then
            local _playerFlag = true
            if self._limitChallenge4Flag then
                local _playerHeroId = tonumber(PlayerData:getBaseInfoByTag("heroId")) or 0
                if tonumber(data.id) ~= _playerHeroId then
                    _playerFlag = false
                end
            end
            -- 英雄
            local hero = zc.createIconByData({
                data = {
                    mainType = zc.mainTypeSoldier,
                    type = data.heroId, 
                    id = data.heroId,
                    level = zc.getLevelTipShow()..data.level,
                    reincarnate = data.reincarnate,
                },
                showLevel = true,
                quality = data.quality,
                showGray = not _playerFlag,
                pos = cc.p( cellWidth/2, cellHeight/2 + 3 ), 
                listener = function()
                    if table:getContentOffset().x + cell:getPositionX() > tableviewSize.width then
                        return true
                    end
                    if not _playerFlag then
                        zc.tipInfo(zc.lang("tips_only_mainrole_online"))
                    elseif not data.upMatrix then
                        --新手引导
                        local _flag = false
                        local _currentGID = getGuideManager():getCurrentGID()
                        if _currentGID == GuideKey.KEY_2003 then
                            if tonumber(data.heroId) == GuideHeroId.KEY_2003 and (#self._heroList == 2) then 
                                _flag = true
                            end
                        end
                        if self._moveIndex == 0 then
                            self:heroUpMatrix( index + 1 )
                        end
                        if _flag then
                            getGuideManager():setNextGID(_currentGID)
                            self:addGuide()
                        end
                        
                    end
                end
            })
            hero:setScale( 0.71 )
            cell:addChild( hero )
            -- 是否上阵
            if data.upMatrix and _playerFlag then
                -- 已上阵
                local upMatrix = cc.Sprite:create( "res/Main/matrixBattle/hero_up_matrix.png" )
                upMatrix:setPosition( cellWidth/2, 10 )
                cell:addChild( upMatrix )
            end
            --新手引导
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_2003 then
                if tonumber(data.heroId) == GuideHeroId.KEY_2003 and (#self._heroList == 2) then 
                    if data.upMatrix then
                        getGuideManager():setCurrentGID(GuideKey.KEY_2004) --跳过布阵
                    else
                        self._guideBtnHero = hero
                        self._scale = 0.71
                        self._guideBtnHero:setSwallowTouches(true)
                    end
                end
            end
        end

        return cell
    end
    local function numberOfCellsInTableView()
        return  #self._heroList
    end
    tableview:registerScriptHandler(cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX)
    tableview:registerScriptHandler(tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX)
    tableview:registerScriptHandler(numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW)  
    -- tableview:reloadData()
    self._heroTableview = tableview
    -- 下阵检测
    local heroDetect = cc.Sprite:create()
    heroDetect:setContentSize( tableviewSize.width + 30, tableviewSize.height + 70 )
    heroDetect:setAnchorPoint( 0, 1 )
    heroDetect:setPosition( 0, self._size.height )
    self:addChild( heroDetect, 700 )
    self._heroDetect = heroDetect
end
-- 武将上阵函数，点击武将头像
function BattleUIMatrixLayer:heroUpMatrix( index )
    local heroData = self._heroList[index]
    -- 判断是否有相同英雄上阵
    local upHeroId = tonumber( heroData.heroId ) or 0
    for i, v in ipairs( self._heroList ) do
        if v.upMatrix then
            -- 上阵武将
            if tonumber( v.heroId ) == upHeroId then
                zc.tipInfo(zc.lang("tips_same_general_online"))
                return
            end
        end
    end
    -- 判断有没有位置
    local uppedNum = table.nums( self._hero )
    -- 开启阵位
    local openMap = PlayerData:getFormationsOpenMap( self._formationData.formationId, self._formationData.level )
    local array = string.split( openMap, "|" )
    local openedNum = #array
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID ~= GuideKey.KEY_2003 then
        if uppedNum >= openedNum then
            zc.tipInfo(zc.lang("tips_no_position_online"))
            return
        end
    end
    -- 上限数量
    local limitNum = self._upHeroNumLimit
    if uppedNum >= limitNum then
        zc.tipInfo(zc.lang("tips_online_num_limit"))
        return
    end
    -- 可以上阵
    local location = 0
    for i, v in ipairs( array ) do
        if tonumber( self._formationData["location"..v] ) == 0 then
            -- 空的，上
            location = tonumber( v )
            self._formationData["location"..v] = heroData.id
            heroData.upMatrix = true
            break
        end
    end
    -- 创建新英雄
    self:buildUpHeroData()
end

-- 阵法选择
function BattleUIMatrixLayer:initFormationList()
    -- bg
    local bg = cc.Sprite:create( "res/Main/matrixBattle/top_bg.png" )
    bg:setAnchorPoint( 1, 1 )
    bg:setPosition( self._size.width + 47, self._size.height + 40 )
    self:addChild( bg, 700 )
    -- tableview
    local tableviewSize = cc.size( 460, 125 )
    local tableview = zc.ZCTableView:create( tableviewSize )
    tableview:setDirection(cc.SCROLLVIEW_DIRECTION_HORIZONTAL)
    tableview:setVerticalFillOrder(cc.TABLEVIEW_FILL_TOPDOWN)
    tableview:setPosition( cc.p( self._size.width - 490, self._size.height - tableviewSize.height ) )
    tableview:setDelegate()
    self:addChild( tableview, 700 )
    local cellWidth = 90
    local cellHeight = tableviewSize.height
    local function cellSizeForTable(table, index)
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex(table, index)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
        end
        cell:removeAllChildren()

        local data = self._formationList[index + 1]
        if data then
      --    local iconBg = cc.Sprite:create( "res/Main/role/starup/iconBg.png" )
         --    iconBg:setPosition( cellWidth/2, cellHeight/2 )
         --    iconBg:setScale( 0.8 )
            -- cell:addChild( iconBg )
            local matrix = cc.Sprite:create()
            matrix:setContentSize( 80, 80 )
            matrix:setPosition( cellWidth/2, cellHeight/2 )
            cell:addChild( matrix )
            -- self:createFormation( matrix, data.map, data.open )
            self:createFormation( matrix, data )
            if tonumber( data.status ) == 1 then
                -- 已启用
                local formationOpen = cc.Sprite:create( "res/Main/matrixBattle/formation_open.png" )
                formationOpen:setPosition( cellWidth/2, 10 )
                -- cell:addChild( formationOpen )
            end
            -- 点击
            local btn = zc.createPushButton({
                touchSize = matrix:getBoundingBox(),
                pos = cc.p( cellWidth/2, cellHeight/2 ),
                needSwallow = false,
                needEnableWhenMoving = true,
                endCallback = function()
                    if table:getContentOffset().x + cell:getPositionX() + cellWidth < 0 then
                        return true
                    end
                    if self._moveIndex == 0 then
                        self:clickMatrix( index + 1 )
                    end
                end
            })
            cell:addChild( btn )
        end

        return cell
    end
    local function numberOfCellsInTableView()
        return  #self._formationList
    end

    tableview:registerScriptHandler(cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX)
    tableview:registerScriptHandler(tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX)
    tableview:registerScriptHandler(numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    tableview:reloadData()
    tableview:scrollToCell( self._formationIndex - 1 )
    self._formationTableview = tableview
end
-- 手绘阵法
function BattleUIMatrixLayer:createFormation( matrix, info )

    local itemMatrix = matrix
    
    if itemMatrix:getChildByName("blockBgImg") then
        itemMatrix:getChildByName("blockBgImg"):removeFromParent()
    end
    local _needVip = 2
    local _isOpen = false
    local _qiYong = "noSelected"
    if info["needVip"] and tonumber(info["needVip"]) > 0 then
        _needVip = 1
    end
    if info["open"] and tonumber(info["open"]) == 1 then
        _isOpen = true
    end
    if info["status"] and tonumber(info["status"]) == 1 then
        _qiYong = "selected"
    end

    local imgPath = {
        [1] = { --需要vip的
            selected = {
                bgImg = "res/Main/matrix/select_bg.png",
                posHave = "res/Main/matrix/liang_lan.png",
                posNo = "res/Main/matrix/an_lan.png",
            },
            noSelected = {
                bgImg = "res/Main/matrix/weixuan_lan.png",
                posHave = "res/Main/matrix/an_lan_block.png",
                posNo = "res/Main/matrix/an_lan.png",    
            }
        },
        [2] = { --不需要vip的
            selected = {
                bgImg = "res/Main/matrix/select_bg.png",
                posHave = "res/Main/matrix/liang_huang.png",
                posNo = "res/Main/matrix/an_hui.png",
            },
            noSelected = {
                bgImg = "res/Main/matrix/weixuan_hui.png",
                posHave = "res/Main/matrix/an_hui_block.png",
                posNo = "res/Main/matrix/an_hui.png",    
            }
        },
    }
    
    local imgTable = imgPath[_needVip][_qiYong]
    local bgImg = zc.createSprite(imgTable["bgImg"])
    bgImg:setAnchorPoint(0.5, 0.5)
    bgImg:setPosition(itemMatrix:getContentSize().width/2, itemMatrix:getContentSize().height/2)
    itemMatrix:addChild(bgImg)
    bgImg:setName("blockBgImg")

    local mapInfo = string.split(info["map"],"|")
    local map = {}
    for i = 1,#mapInfo do
        map[tonumber(mapInfo[i])] = "true"
    end
    local itemWidth = itemMatrix:getContentSize().width
    local itemHeight = itemMatrix:getContentSize().height
    for i = 1, 9 do
        local posx = itemWidth - 15 - math.floor((i-1)/3) * 25
        local posy = itemHeight - 15 - (i - math.floor((i-1)/3) * 3 - 1) * 25
        if map[i] then
            local posHave = zc.createSprite(imgTable["posHave"])
            posHave:setAnchorPoint(0.5, 0.5)
            posHave:setPosition(posx, posy)
            itemMatrix:addChild(posHave)
        else
            local posNo = zc.createSprite(imgTable["posNo"])
            posNo:setAnchorPoint(0.5, 0.5)
            posNo:setPosition(posx, posy)
            itemMatrix:addChild(posNo)
        end
    end

    if not _isOpen then
        local mask = zc.createSprite("res/Main/matrix/mask.png")
        mask:setAnchorPoint(0.5, 0.5)
        mask:setPosition(itemWidth/2, itemHeight/2)
        itemMatrix:addChild(mask)
    end



    do return end


    -- 数据
    local array = string.split( map, "|" )
    local pointSrc = "res/Main/matrixBattle/dot_"..open..".png"
    local lineSrc = "res/Main/matrixBattle/line_"..open.."_"
    -- 点
    local pointPos = self._formationPoint
    for i, v in ipairs( array ) do 
        local location = tonumber( v )
        local pointImg = cc.Sprite:create( pointSrc )
        pointImg:setPosition( pointPos[location] )
        matrix:addChild(pointImg, 2)
    end
    -- 线
    local transform = self._formationLineTrans
    local lineData = {}
    -- 判断任意两点x和y间隔，二层循环防止重复
    for i = 1, #array do 
        local locationI = tonumber( array[i] )
        for j = i + 1, #array do
            local locationJ = tonumber( array[j] )
            if ( math.abs( transform[locationI][1] - transform[locationJ][1]) == 0 and math.abs(transform[locationI][2] - transform[locationJ][2]) == 1) then
                lineData[#lineData + 1] = {x = locationI, y = locationJ, img = 1}
            elseif ( math.abs(transform[locationI][1] - transform[locationJ][1]) == 1 and math.abs(transform[locationI][2] - transform[locationJ][2]) == 0) then 
                lineData[#lineData + 1] = {x = locationI, y = locationJ, img = 2}
            elseif (math.abs(transform[locationI][1] - transform[locationJ][1]) == 1 and math.abs(transform[locationI][2] - transform[locationJ][2]) == 1) then   
                if (transform[locationI][1] - transform[locationJ][1]) + (transform[locationI][2] - transform[locationJ][2]) == 0 then 
                    lineData[#lineData + 1] = {x = locationI, y = locationJ, img = 3}
                else
                    lineData[#lineData + 1] = {x = locationI, y = locationJ, img = 4}
                end
            end
        end
    end
    for i, v in ipairs( lineData ) do
        local anchor = cc.p(1, 1)
        local path = 1
        local rotate = 0
        if v.img == 1 then
            -- |
            anchor = cc.p(0.5, 1)
            path = 1
            rotate = 0
        elseif v.img == 2 then
            -- -
            anchor = cc.p(0.5, 1)--cc.p(1, 0.5)
            path = 1
            rotate = 90
        elseif v.img == 3 then
            -- \
            anchor = cc.p(1, 0)
            path = 2
            rotate = 0
        elseif v.img == 4 then
            -- /
            anchor = cc.p(0, 1)--cc.p(1, 1)
            path = 2
            rotate = 90
        end
        local lineImg = cc.Sprite:create( lineSrc .. path ..".png" )
        lineImg:setRotation( rotate )
        lineImg:setAnchorPoint( anchor )
        lineImg:setPosition( pointPos[v.x] )
        lineImg:setScale(0.4)
        matrix:addChild(lineImg, 1)
    end
end
-- 点击阵法回调
function BattleUIMatrixLayer:clickMatrix( index )
    -- 检查是否满足开启条件
    if tonumber( PlayerData.info.vip ) < tonumber( self._formationList[index].needVip ) then
        zc.tipInfo(zc.lang("tips_vip_unenough_to_open_zhen"))
        return
    elseif tonumber( PlayerData.info.level ) < tonumber( self._formationList[index].needLevel ) then
        zc.tipInfo( zc.lang("tips_level_unenough_to_open_zhen") )
        return
    else
        -- 当前的阵法数据
        local formationNew = self:getFormationString()
        -- 阵法数据没改变
        if self._formationBack == formationNew then
            -- 直接换阵
            self:changeMatrix( index )
        else
            -- 保存阵法后换阵
            zc.sendHttp({
                apiName = "Formation_saveFormation",
                params = {info = formationNew,type = 1},
                successCallback = function(data)
                   PlayerData:setFormationsInfo( data.formations )
                   self:changeMatrix( index )
                end,
            })
        end
    end
end
-- 换阵
function BattleUIMatrixLayer:changeMatrix( index )
    -- 更新上阵的阵法数据
    self._formationData.status = 0
    self._formationData = self._formationList[index]
    self._formationData.status = 1
    self._formationBack = self:getFormationString()
    self._formationIndex = index
    -- 刷新阵法tableview
    self._formationTableview:reloadDataAndScrollToCurrentCell()
    -- 刷新武将数据，武将tableview，武将动画
    self:buildUpHeroData()
    -- 刷新阵法信息
    self:refreshFormationInfo()
end

-- 创建武将脚下
function BattleUIMatrixLayer:initBottom()
    -- 英雄
    for i = 1, 9 do
        -- 底部
        local bottom = cc.Sprite:create()
        bottom:setPosition( self._heroPos[i] )
        self:addChild( bottom, 300 )
        self._heroBottom[i] = bottom
        -- 几手
        local order = cc.Sprite:create()
        order:setAnchorPoint( 0, 0 )
        order:setPosition( 10, 8 )
        bottom:addChild( order )
        bottom._order = order
        -- 按钮
        local btn = ccui.Widget:create()
        btn:setContentSize( 120, 150 )
        btn:setAnchorPoint( 0.5, 0 )
        btn:setPosition( self._heroPos[i].x, self._heroPos[i].y )
        btn:setSwallowTouches( true )
        btn:onTouch(function(event)
            if self._moveIndex == 0 then
                self._moveIndex = i
            elseif self._moveIndex ~= i then
                return
            end
            if event.name == "began" then
                self._hero[i]:setLocalZOrder( 1000 )
                self._movePos = 0
            elseif event.name == "moved" then
                self._shadow:setVisible( true )
                local began = event.target:getTouchBeganPosition()
                local moved = event.target:getTouchMovePosition()
                self._hero[i]:setPosition( self._heroPos[i].x + moved.x - began.x, self._heroPos[i].y + moved.y - began.y )
                -- 检测点位置
                self._movePos = 0
                for j, v in ipairs( self._heroBottom ) do
                    if v._status == 1 then
                        -- 有英雄
                        if zc.isTouchContainsPoint( self._detect[j], moved ) then
                            self._movePos = j
                            break
                        end
                    elseif v._status == 2 then
                        -- 没英雄
                        if zc.isTouchContainsPoint( self._detect[j], moved ) then
                            self._movePos = j
                            break
                        end
                    end
                end
                for j, v in ipairs( self._heroBottom ) do
                    if j == self._movePos then
                        -- 交换位置
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_select.png" )
                    elseif v._status == 1 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_normal.png" )
                    elseif v._status == 2 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_normal.png" )
                    elseif v._status == 3 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_lock.png" )
                    elseif v._status == 4 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_disable.png" )
                    end
                end
                if zc.isTouchContainsPoint( self._heroDetect, moved ) then
                    self._downMatrix = true
                else
                    self._downMatrix = false
                end
            else
                local _currentGID = getGuideManager():getCurrentGID()
                local _tag = false
                if _currentGID == GuideKey.KEY_5004 then
                    _tag = true
                end
                for j, v in ipairs( self._heroBottom ) do
                    if v._status == 1 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_normal.png" )
                    elseif v._status == 2 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_normal.png" )
                    elseif v._status == 3 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_lock.png" )
                    elseif v._status == 4 then
                        self._heroBottom[j]:setTexture( "res/Main/matrixBattle/matrix_disable.png" )
                    end
                end
                self._shadow:setVisible( false )
                local function _goBack( ... )
                    self._hero[i]:runAction(
                        cc.Sequence:create(
                            cc.MoveTo:create( 0.05, self._heroPos[i] ),
                            cc.CallFunc:create(
                                function()
                                    self._hero[i]:setLocalZOrder( zc.battle.getZOrder( i ) + 500 )
                                    self._moveIndex = 0
                                end
                            )
                        )
                    )
                end
                if self._movePos ~= 0 then
                    -- 移动
                    if self._movePos ~= i then
                        if _tag and self._movePos ~= 2 then
                            _goBack()
                            return
                        end
                        if self._heroBottom[self._movePos]._status == 1 then
                            -- 交换英雄
                            print("交换英雄")
                            self._formationData["location"..i], self._formationData["location"..self._movePos] = self._formationData["location"..self._movePos], self._formationData["location"..i]
                            self._hero[i]:runAction(
                                cc.Sequence:create(
                                    cc.MoveTo:create( 0.05, self._heroPos[self._movePos] ),
                                    cc.CallFunc:create(
                                        function()
                                            self._hero[i]:setLocalZOrder( zc.battle.getZOrder( self._movePos ) + 500 )
                                        end
                                    )
                                )
                            )
                            self._hero[self._movePos]:runAction(
                                cc.Sequence:create(
                                    cc.MoveTo:create( 0.05, self._heroPos[i] ),
                                    cc.CallFunc:create(
                                        function()
                                            self._hero[self._movePos]:setLocalZOrder( zc.battle.getZOrder( i ) + 500 )
                                            self:buildUpHeroData()
                                            self._moveIndex = 0
                                        end
                                    )
                                )
                            )
                        else
                            -- 直接换位置
                            print("直接换位置")
                            self._formationData["location"..self._movePos] = self._formationData["location"..i]
                            self._formationData["location"..i] = 0
                            self._hero[i]:runAction(
                                cc.Sequence:create(
                                    cc.MoveTo:create( 0.05, self._heroPos[self._movePos] ),
                                    cc.CallFunc:create(
                                        function()
                                            self._hero[i]:setLocalZOrder( zc.battle.getZOrder( self._movePos ) + 500 )
                                            self:buildUpHeroData()
                                            self._moveIndex = 0
                                        end
                                    )
                                )
                            )
                        end
                        if _tag then
                            getGuideManager():setNextGID(_currentGID)
                            self:addGuide()
                        end
                    else
                        -- 回原位置
                        print("回原位置")
                        _goBack()
                    end
                else
                    -- 检测是否下阵
                    if not _tag and self._downMatrix then
                        if tonumber( self._formationData["location"..i] ) == tonumber( PlayerData.info.heroId ) then
                            -- 主角不能下阵
                            print("主角不能下阵")
                            zc.tipInfo(zc.lang("tips_mainrole_cannot_offline"))
                            self._hero[i]:runAction(
                                cc.Sequence:create(
                                    cc.MoveTo:create( 0.05, self._heroPos[i] ),
                                    cc.CallFunc:create(
                                        function()
                                            self._hero[i]:setLocalZOrder( zc.battle.getZOrder( i ) + 500 )
                                            self._moveIndex = 0
                                        end
                                    )
                                )
                            )
                        else
                            -- 下阵
                            print("下阵")
                            self._formationData["location"..i] = 0
                            self._hero[i]:runAction(
                                cc.Sequence:create(
                                    cc.ScaleTo:create( 0.05, 0 ),
                                    cc.CallFunc:create(
                                        function()
                                            self:buildUpHeroData()
                                            self._moveIndex = 0
                                        end
                                    )
                                )
                            )
                        end
                    else
                        print("不下阵")
                        _goBack()
                    end
                end
                -- 武将信息弹窗
                if _tag then
                    return
                end
                if cc.pGetDistance(event.target:getTouchBeganPosition(),event.target:getTouchEndPosition()) < 10 then
                    if tonumber( self._formationData["location"..i] ) ~= 0 then
                       zc.createHeroInfoPop({id = self._formationData["location"..i]})
                    end
                end
            end
        end)
        self:addChild( btn, 500 )
        self._heroBtn[i] = btn
        -- 坐标检测
        local detect = cc.Sprite:create()
        detect:setContentSize( 100, 200 )
        detect:setAnchorPoint( 0.5, 0 )
        detect:setPosition( self._heroPos[i] )
        self:addChild( detect, 500 )
        self._detect[i] = detect
    end
    -- 敌人
    for i = 1, 9 do
        local bottom = cc.Sprite:create( "res/Main/matrixBattle/matrix_disable.png" )
        bottom:setPosition( self._enemyPos[i] )
        self:addChild( bottom, 300 )
        bottom:setFlippedX( true )
        self._enemyBottom[i] = bottom
    end
end
-- 刷新武将
function BattleUIMatrixLayer:refreshHero()
    for k, v in pairs( self._hero ) do
        v:removeFromParent()
    end
    self._hero = {}
    local _guideIdx = self:getGuideIndex()
    local function _getGuideInfo(i)
        if not self._guideData then
            return nil
        end
        local _heroDatas = self._guideData[6][2]
        for k,v in pairs(_heroDatas) do
            if v[2] == 0 and v[3] == i then
                if _guideIdx <= 6 then
                    if v[12] == "zhangfei" then
                        local _data = {}
                        _data.id = v[1]
                        _data.modelId = v[12]
                        _data.weaponId = v[13]
                        _data.isZhuzhan = true
                        return _data
                    end
                elseif _guideIdx == 7 then
                    if v[12] ~= "guanyu" and v[12] ~= "zhangfei" and v[12] ~= "liubei" then
                        local _data = {}
                        _data.id = v[1]
                        _data.modelId = v[12]
                        _data.weaponId = v[13]
                        _data.isZhuzhan = false
                        return _data
                    end
                elseif _guideIdx == 9 then
                    if v[12] ~= "guanyu" then
                        local _data = {}
                        _data.id = v[1]
                        _data.modelId = v[12]
                        _data.weaponId = v[13]
                        _data.isZhuzhan = false
                        return _data
                    end
                else
                    local _data = {}
                    _data.id = v[1]
                    _data.modelId = v[12]
                    _data.weaponId = v[13]
                    _data.isZhuzhan = false
                    return _data
                end
            end
        end
        return nil
    end
    -- 解析阵法数据
    local array = string.split( self._formationData.map, "|" )
    local openMap = PlayerData:getFormationsOpenMap( self._formationData.formationId, self._formationData.level )
    local openArray = string.split( openMap, "|" )
    local order = 1
    local _playerHeroId = tonumber(PlayerData:getBaseInfoByTag("heroId")) or 0
    for i = 1, 9 do
        local _guideData = _getGuideInfo(i)
        if _guideData or inArray( tostring( i ), array ) then
            -- 阵法有这个阵位
            if _guideData or inArray( tostring( i ), openArray ) then
                -- 阵位已解锁
                local heroId = 0
                if _guideIdx <= 6 or not self._guideData then
                    heroId = tonumber( self._formationData["location"..i] ) or 0
                end
                if self._limitChallenge4Flag and heroId ~= _playerHeroId then 
                    _guideData = nil
                    heroId = 0
                    self._formationData["location"..i] = 0
                end
                if _guideData or heroId ~= 0 then
                    -- 有武将
                    local heroData = {}
                    if _guideData then
                        heroData = _guideData
                    else
                        for i, v in ipairs( self._heroList ) do
                            if tonumber( v.id ) == heroId then
                                heroData = v
                                break
                            end
                        end
                    end
                    -- dump( heroData, heroId )
                    local hero = zc.battle.createHeroArmatureWithResId( heroData.modelId )
                    -- hero:setOpacity(0)
                    hero:setScaleX( -1 )
                    hero:setPosition( self._heroPos[i] )
                    hero:getAnimation():play( "idle" )
                    hero:setLocalZOrder( zc.battle.getZOrder( i ) + 500 )
                    -- hero:runAction(cc.Sequence:create(cc.FadeIn:create(0.2 + 0.01 * i),cc.CallFunc:create(function()  
                        
                    -- end )))
                    -- 更换兵器
                    if not _guideData or _guideIdx <= 6 then
                        local weaponData = PlayerData:getWeaponsInfoById( heroId ) or {}
                        hero.weaponFile = zc.soldierChangeWeapon( hero, weaponData.weaponId or 0 )
                        self._recordHero[#self._recordHero + 1] = {
                            heroData.modelId, weaponData.weaponId
                        }
                    else
                        hero.weaponFile = zc.soldierChangeWeapon( hero, _guideData.weaponId or 0 )
                        self._recordHero[#self._recordHero + 1] = {
                            heroData.modelId, _guideData.weaponId
                        }
                    end
                    self:addChild( hero )

                    self._heroBottom[i]._order:setTexture( "res/Main/matrixBattle/order_"..order..".png" )
                    order = order + 1
                    self._heroBottom[i]._order:setVisible( true )
                    if _guideData and _guideData.isZhuzhan then
                        -- self._heroBottom[i]._status = 2
                        local _heroBar = zc.createSprite("res/image/battle/heroNameTab.png")
                        self:addChild(_heroBar, zc.battle.getZOrder( i ) + 500)
                        _heroBar:setPosition(self._heroPos[i].x, self._heroPos[i].y + 170)
                        local _label = zc.createLabel({
                            fontSize = 22,
                            text = zc.lang("zhuzhan_hero"),
                            ap = cc.p(0.5, 0.5),
                            stroke = true,
                            pixel = 1,
                            color = display.COLOR_WHITE,
                        })
                        _label:setPosition(_heroBar:getContentSize().width*0.5 + 8, _heroBar:getContentSize().height*0.5 + 2)
                        _heroBar:addChild(_label)
                    end
                    self._hero[i] = hero
                    -- 有英雄
                    self._heroBottom[i]._status = 1
                    self._heroBtn[i]:setTouchEnabled( true )
                else
                    self._heroBottom[i]._order:setVisible( false )
                    -- 没英雄，已开启
                    self._heroBottom[i]._status = 2
                    self._heroBtn[i]:setTouchEnabled( false )
                end
                self._heroBottom[i]:setTexture( "res/Main/matrixBattle/matrix_normal.png" )
            else
                -- 阵位没有解锁
                self._heroBottom[i]:setTexture( "res/Main/matrixBattle/matrix_lock.png" )
                self._heroBottom[i]._order:setVisible( false )
                -- 没开启
                self._heroBottom[i]._status = 3
                self._heroBtn[i]:setTouchEnabled( false )
            end
        else
            -- 阵法没有这个阵位
            self._heroBottom[i]:setTexture( "res/Main/matrixBattle/matrix_disable.png" )
            self._heroBottom[i]._order:setVisible( false )
            -- 没阵位
            self._heroBottom[i]._status = 4
            self._heroBtn[i]:setTouchEnabled( false )
        end
    end
end
-- 刷新敌人
function BattleUIMatrixLayer:refreshEnemy()
    for k, v in pairs( self._enemy ) do
        v:removeFromParent()
    end
    self._enemy = {}
    self._loadedEnemy = {}
    local _guideIdx = self:getGuideIndex()
    local function _getGuideInfo(i)
        if not self._guideData then
            return self._enemyList[i]
        end
        local _heroDatas = self._guideData[6][2]
        for k,v in pairs(_heroDatas) do
            if v[2] ~= 0 and v[3] == i then
                if _guideIdx == 7 then
                    if v[12] ~= "diaochan" then 
                        local _data = {}
                        _data.id = v[1]
                        _data.modelId = v[12]
                        _data.weaponId = v[13]
                        return _data
                    end
                else
                    local _data = {}
                    _data.id = v[1]
                    _data.modelId = v[12]
                    _data.weaponId = v[13]
                    return _data
                end
            end
        end
        return nil
    end
    

    for i = 1, 9 do
        local enemyData = _getGuideInfo(i) or {}
        -- dump(enemyData)
        if enemyData.id then
            local enemy = zc.battle.createHeroArmature({
                fileName = enemyData.modelId,
                heroDataWeaponId = enemyData.weaponId or 0,
                async = true,
                asyncCallback = function ( _armature,fileName,num )
                    print("------fileName-----num=", fileName, " , ", tostring(num))
                    if self._exist and _armature then
                        if num and tonumber(num) and tonumber(num) == 0 then
                            _armature:setOpacity(0)
                            _armature:runAction(cc.Sequence:create(cc.FadeIn:create(0.2 + 0.01 * i),cc.CallFunc:create(function()  
                                -- _armature:getAnimation():play("idle", -1, 1) 
                            end )))
                        end

                        self._loadedEnemy[i] = _armature
                        _armature:getAnimation():play( "idle" )
                        _armature.resFile = zc.battle.getHeroArmaturePathByResId(enemyData.modelId, enemyData.weaponId or 0)
                        --更换兵器
                        _armature.weaponFile = zc.soldierChangeWeapon( _armature, enemyData.weaponId or 0 )
                       
                        if self._guideData then
                            if _guideIdx <= 6 then
                                if enemyData.modelId == "jiaxu" then
                                    _armature:getAnimation():play("tielong", -1, 1)
                                end
                            elseif _guideIdx == 8 then
                                if enemyData.modelId == "diaochan" then
                                    _armature:getAnimation():play("tielong", -1, 1)
                                end
                            end
                        end
                    end
                end,
            })
            -- 敌人
            self._recordEnemy[#self._recordEnemy + 1] = {
                enemyData.modelId, enemyData.weaponId
            }
            -- 有敌人
            -- local enemy = zc.battle.createHeroArmatureWithResId( enemyData.modelId )
            enemy:setPosition( self._enemyPos[i] )
            -- enemy:getAnimation():play( "idle" )
            enemy:setLocalZOrder( zc.battle.getZOrder( i ) + 500 )
            --更换兵器
            -- zc.soldierChangeWeapon( enemy, enemyData.weaponId or 0 )
            self:addChild( enemy )
            self._enemy[i] = enemy
            self._enemyBottom[i]:setOpacity( 255*0.5 )
        else
            self._enemyBottom[i]:setOpacity( 255*0.2 )
        end
    end
end
-- 刷新底部阵法显示信息
function BattleUIMatrixLayer:refreshFormationInfo()
    local data = self._formationData
    self._formationLevel:setText( zc.getLevelTipShow()..data.level )
    self._formationName:setText( data.name )
    -- 加成数据
    -- 全体加成
    local totalData = {}
    for i = 1, 4 do
        local array = string.split(data["effect"..i],"|")
        if #array == 3 then
            local _isBase = false
            if data["effectLayer"..i] and (tonumber(data["effectLayer"..i])==1 or tonumber(data["effectLayer"..i])==2) then
                _isBase = true
            end
            local _target = nil
            if data["effectTarget"..i] and type(data["effectTarget"..i]) == "number" then
                _target = data["effectTarget"..i]
            end
            
            local newindex = #totalData+1
            totalData[newindex] = {}
            local effectStr = LANGUAGE_getAttributeTextById(array[1], _isBase)
            if _target then
                effectStr = LANGUAGE_getTargetTextById(_target)..effectStr
            end
            totalData[newindex].attname = effectStr
            totalData[newindex].startatt = tonumber(array[2])
            totalData[newindex].addatt = tonumber(array[3])
            totalData[newindex].id = tonumber(array[1])

            local curEff = totalData[newindex].startatt + (tonumber(data.level)-1)*totalData[newindex].addatt
            totalData[newindex].name = totalData[newindex].attname
            totalData[newindex].num = zc.getAttributeNumById(totalData[newindex].id, curEff)
        end
    end
    -- dump( totalData, "totalData" )
    -- 构造显示文字
    local totalAddition = nil
    for i, v in ipairs( totalData ) do
        totalAddition = totalAddition and totalAddition.." "..v.name.." "..v.num or v.name.." "..v.num
    end
    self._formationTotalAddition:setText( totalAddition or "" )
    -- 站位加成
    local specialData = {}
    local _data = gameData.getDataFromCache("formation_level", {formationId = data.formationId, level = data.level})
    for i = 1, 9 do
        local effect = string.split( _data["location"..i], "|" )
        if #effect == 3 then
            specialData[#specialData + 1] = {
                name = i .. zc.lang("number_position") .. zc.getEquipAttriNameByEffectType( effect[1], true, true ),
                num = zc.getAttributeNumById( effect[1], effect[2] ),
            }
        end
    end
    -- dump( specialData, "specialData" )
    -- 构造显示文字
    local specialAddition = nil
    for i, v in ipairs( specialData ) do
        specialAddition = specialAddition and specialAddition.." "..v.name.." "..v.num or v.name.." "..v.num
    end
    self._formationSpecialAddition:setText( specialAddition or "" )
end

-- 获取当前阵法的字符串数据
function BattleUIMatrixLayer:getFormationString()
    local formationString = tostring( self._formationData.formationId )..":"..self._formationData["location1"]
    for i = 2, 9 do
        formationString = formationString.."_"..self._formationData["location"..i]
    end
    -- print( "formationString =", formationString )
    return formationString
end
-- 更新数据，保存成战斗结果页需要的格式
function BattleUIMatrixLayer:saveData( data )
    -- 保存阵法
    if data.formations then
        PlayerData:setFormationsInfo( data.formations )
    end
    -- 结果类型(1 平局, 2 攻击方完胜, 3 攻击方大胜, 4 攻击方小胜, 5 攻击方完败, 6 攻击方大败, 7 攻击方小败)
    local battleStatus = 2
    local star = 0
    if data.warModels then
        local warModel = data.warModels
        battleStatus = tonumber( warModel[#warModel][5][1] ) or 2
        star = warModel[#warModel][5][6]
    else
        local _warModel = nil
        if self._guideData then
            data.warModel = self._guideData
            _warModel = self._guideData
        elseif data.warModel then
            _warModel = data.warModel
        end
        if _warModel then
            battleStatus = tonumber( _warModel[5][1] ) or 2
            star = _warModel[5][6]
        end
    end

    -- 获取的银币 元宝 加到item里面
    data.items = PlayerData:getShowIconDatas(data)

    -- 获得的经验
    data.addExp = {}
    if data.clientItems then
        for i, v in ipairs( data.clientItems ) do
            local itemType = tonumber(v.type) or 0
            if itemType == 5 then
                -- 武将经验
                local _num = tonumber(v.changeNum) or 0
                if _num > 0 then
                    local _index = tonumber( v.primaryId ) or 0
                    data.addExp[_index] = {
                        changeNum = v.changeNum,
                        levelUp = tonumber( v.levelUp ) or 0,
                    }
                end
            elseif itemType == 6 then
                -- 主角经验
                local _num = tonumber(v.changeNum) or 0
                if _num > 0 then
                    local _index = tonumber( PlayerData.info.heroId ) or 0
                    data.addExp[_index] = {
                        changeNum = v.changeNum,
                        levelUp = tonumber( v.levelUp ) or 0,
                    }
                end
            end
        end
    end
    
    -- 解锁新武将
    local newUnlockHeroId = tonumber( data.newUnlockHeroId ) or 0
    if newUnlockHeroId ~= 0 then 
        PlayerData.info.activeHeros[#PlayerData.info.activeHeros+1] = data.newUnlockHeroId
    end
    -- 更新获得物品和资源
    PlayerData:updataData({
        clientResources = data.clientResources,
        clientItems = data.clientItems,
        eliteTollgate = data.eliteTollgate,
        noAnimation = true
    })
    -- 解析武将
    for k,v in pairs(data.heros or {}) do 
        PlayerData:setGeneralInfoById(v.id, v)
    end

    return battleStatus, star
end
-- 开始战斗
function BattleUIMatrixLayer:startFight()
    if getGuideManager():getCurrentGID() == GuideKey.KEY_15008 then
        if not PlayerData:getFuncGuideInfo(407) then
            getGuideManager():sendFuncGuideTrigger(407)
        end
    end

    local infoData = self:getFormationString()
    local _apiName, _callBack
    local _bgNode = self._bgNode
    local _heros = self._hero or {}
    local _enemys = self._loadedEnemy or {}
    local _params = {
        formationInfo = infoData,
        enableFormationId = self._formationData.formationId,
    }
    local _errorCall = function(msg)
        if not tolua.isnull(_bgNode) then
            _bgNode:release()
        end
        for k,v in pairs(_heros) do
            if not tolua.isnull(v) then
                v:release()
            end
        end
        for k,v in pairs(_enemys) do
            if not tolua.isnull(v) then
                v:release()
            end
        end
        zc.showErrorDialog(msg)
    end

    local _normalCall = function(data, endCall, playCall)
        xpcall(function()
            local _heroTb = {}
            for k,v in pairs(_heros) do
                v:retain()
                v:removeFromParentAndCleanup(false)
                _heroTb[tostring(k)] = v
            end
            
            local _enemyTb = {}
            for k,v in pairs(_enemys) do
                v:retain()
                v:removeFromParentAndCleanup(false)
                _enemyTb[tostring(k)] = v
            end
            -- 移除资源
            self:removeArmature( true )
            LayerManager.removeLayout(nil, nil, false)
            local battle = zc.createBattle({
                data = data,
                leftTeams = {
                    _heroTb,
                },
                rightTeams = {
                    _enemyTb,
                },
                battleEndCallback = endCall,
            })
            for k,v in pairs(_heros) do
                v:release()
            end
            _heros = {}
            for k,v in pairs(_enemys) do
                v:release()
            end
            _enemys = {}
            _bgNode:release()
            if playCall then
                playCall(battle)
            else
                battle:play() 
            end
        end, _errorCall)
    end
    if self._battleType == BattleType.DrillField then  -- 校场挑战
        local drillFieldInfo = PlayerData:getHouseInfoByHouseId(8)
        if not drillFieldInfo then
            return
        end
        _apiName = "Training_challenge"
        _params.id = self._battleTypeData.drillPositionId
        _params.houseId = drillFieldInfo.id
        _callBack = function(data) 
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_12708 then
                zc.setGuideFuncNextStep(_currentGID)
            end
            data.canShare = true
            data.isFirstFight = true
            PlayerData:setLastSceneData({
                lastScene = "drillField", 
                func = function () 
                    zc.createDrillFieldLayer()
                end
            })
            _normalCall(data)
        end
    elseif self._battleType == BattleType.ARENA then
        -- 演武榜
        _apiName = "Arena_challenge"
        _params.index = self._battleTypeData.pvpIndex
        _callBack = function(data)
            local _currentGID = getGuideManager():getCurrentGID()
            if inArray(_currentGID, GuideGroup.KAI_ZHAN) then
                getGuideManager():guideOver()
                getGuideManager():setNextGID(_currentGID)
            end
            data.canShare = true
            data.battleResultData = {
                -- 是否显示排名军功积分变化
                arenaFlag = true,
                -- 排名
                befRank = data.data.attackerBeforeRank,
                aftRank = data.data.attackerAfterRank,
                -- 军功
                addBattleScore = data.data.addBattleScore,
                -- 积分
                addScore = data.data.addScores,
                -- 分享内容
                shareData = self._battleTypeData.shareData,
            }
            PlayerData:setLastSceneData({lastScene = "Arena", 
                func = function () 
                    zc.createArenaScene()
                end
            })
            _normalCall(data)
        end
    elseif self._battleType == BattleType.GodFight then
        -- 演武榜
        _apiName = "GodFight_challenge"
        _params.toUserId = self._battleTypeData.toUserId
        _callBack = function(data)
            local _currentGID = getGuideManager():getCurrentGID()
            if inArray(_currentGID, GuideGroup.KAI_ZHAN) then
                getGuideManager():guideOver()
                getGuideManager():setNextGID(_currentGID)
            end
            data.canShare = true
            data.fightScene = self._fightScene
            data.battleResultData = {
                -- 是否显示排名军功积分变化
                arenaFlag = true,
                -- 排名
                befRank = self._battleTypeData.mRank,
                aftRank = data.data.myRank,
            }
            PlayerData:setLastSceneData({
                lastScene = "GodFight", 
                myRank = data.data.myRank,
                rank = data.data.rank,
                rankList = data.data.rankList,
                fightTimes = data.data.fightTimes,
                buyTimes = data.data.buyTimes,
                cdTimes = data.data.cdTimes,
                cdTime = data.data.cdTime,
            })
            _normalCall(data)
        end
    elseif self._battleType == BattleType.GrabFight then
        -- 演武榜
        _apiName = "GrabFood_challenge"
        _params.id = self._battleTypeData.toUserId
        if self._battleTypeData.sType then
            _params.type = self._battleTypeData.sType
        end
        _callBack = function(data)
            local _currentGID = getGuideManager():getCurrentGID()
            if inArray(_currentGID, GuideGroup.KAI_ZHAN) then
                zc.setGuideFuncNextStep(_currentGID, false)
            end
            data.canShare = true
            data.fightScene = self._fightScene
            data.battleResultData = {
                -- 是否显示排名军功积分变化
                arenaFlag = true,
                -- 排名
                foodNums = data.data.nums
            }
            
            PlayerData:setLastSceneData({
                lastScene = "GrabFight", 
                times = data.data.times,
                rank = data.data.rank,
                target = data.data.target,
                ownGrabInfo = data.data.ownGrabInfo,
            })
            _normalCall(data)
        end
    elseif self._battleType == BattleType.WuShen_Tower then
        _apiName = "Wushen_challenge"
        _params.id = self._enemyData.id
        _callBack = function (data)
            local passedLayer = false
            if tonumber(self._enemyData.tollgateId) % 3 == 0 then
                if tonumber(data.data.pass) == 1 then --是否过关 1过关 0未过关
                    passedLayer = true
                end
            end
            PlayerData:setLastSceneData({
                lastScene = "WushenTower", 
                --这里记录是是否通关(是不是本层最后一关，是否胜利(胜利需要弹窗和选择buff))
                isWin = data.data.pass == 1 and true or false,
                passedLayer = passedLayer,
                star = self._enemyData.star,
                tollgateId = self._enemyData.tollgateId,
                info = data.data.info
            })
            data.canShare = true
            data.fightScene = self._fightScene
            data.wushenAwards = self._enemyData.awards
            _normalCall(data, function()
                if USE_PUSH_BATTLE then
                    local _scene = cc.Director:getInstance():getRunningScene()
                    local _lay = requires("src/layer/wushenshilian/BattleWushenResultPop.lua"):create({
                        scene = _scene,
                        replay = function(_canShare)
                            zc.dispatchEvent({
                                name = EVENT_NAME_BATTLE_REPLAY,
                                data = {canShare = _canShare}
                            }) 
                        end,
                        quit = function() 
                            cc.Director:getInstance():popScene()
                        end,
                        data = data
                    })
                    _scene:addChild(_lay)
                else
                    zc.createSourcePop("BattleWushenResult", "src/layer/wushenshilian/BattleWushenResultPop.lua", {
                        replay = function(_canShare)
                            zc.dispatchEvent({
                                name = EVENT_NAME_BATTLE_REPLAY,
                                data = {canShare = _canShare}
                            }) 
                        end,
                        quit = function() 
                            LayerManager.removeLayout() 
                        end,
                        data = data
                    })
                end
            end)
        end
    elseif self._battleType == BattleType.PVE then
        _apiName = "Tollgate_challenge"
        _params.id = self._enemyData.tollgateId
        _callBack = function (data, battleStatus, star) 
            if battleStatus then
                -- 胜利了，设置关卡状态
                PlayerData:setCurTollGateStatus(self._enemyData.tollgateId, {status = 2, numadd = 1, star = star})
            end
            local enemyid = self._enemyData.tollgateId
            data.enemyid = enemyid
            data.isFirstFight = self._isFirstFight
            local _curTollgateData = PlayerData:getCurTollGateData(self._enemyData.tollgateId) or {}
            data.clickFight = _curTollgateData.clickFight
            if getGuideManager():getCurrentGID() == 1 and not _curTollgateData.clickFight then
                if not self._isFirstFight then 
                    data.clickFight = true
                end
                _curTollgateData.clickFight = true
            end
            data.canShare = true
            data.fightScene = self._fightScene
            local _data = self:getGuideData(data)
            _normalCall(_data, nil, function( battle )
                --新手引导
                local function _doChange()
                    if battleStatus and _data.isFirstFight then
                        for k,v in pairs(GuideTollgateKey) do
                            if enemyid == v then
                                local _k = tonumber(k) or 0
                                if _k < 10 then
                                    _k = "0" .. _k 
                                end
                                getGuideManager():setCurrentGID(GuideKey["KEY_1" .. _k .. "00"], true)
                            end
                        end
                    end
                        
                    if _data.guide1 
                      or _data.guide2
                      or _data.guide3 then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "nanzhu" 
                              or hero:getResourceId() == "nvzhujue" 
                              or hero:getResourceId() == "nanzhu_shan" 
                              or hero:getResourceId() == "nvzhuqiang"
                              then
                                hero:playAction("change")
                                performWithDelay(hero, function()
                                    battle:play()
                                end, 2)
                                break
                            end
                        end
                        return
                    end
                    battle:play()
                end
                local function _checkTalk()
                    if _data.clickFight ~= true then
                        local _tollDb = gameData.getDataFromCacheWithPrimaryKey("tollgate", enemyid)
                        if _tollDb.prebattleTalk and _tollDb.prebattleTalk ~= "" then 
                            local guideScriptLayer = GuideScriptLayer:createWithParams({storyId = _tollDb.prebattleTalk, callback = _doChange})
                            battle:addChild(guideScriptLayer, 100)
                            return
                        end
                    end
                    _doChange()
                end
                local function _callBack()
                    if _data.guide2 then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "jiaxu" then
                                hero:playAction("tielong", true)
                                break
                            end
                        end
                    elseif _data.guide5 then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "liru" then
                                hero:setVisible(false)
                                break
                            end
                        end
                    elseif _data.guide6 then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "guojia" or hero:getResourceId() == "caocao" then
                                hero:setVisible(false)
                            end
                        end
                    elseif _data.guide7 then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "guanyu" 
                              or hero:getResourceId() == "zhangfei" 
                              or hero:getResourceId() == "diaochan" 
                              or hero:getResourceId() == "liubei" then
                                hero:setVisible(false)
                            end
                        end
                    elseif _data.guide8 then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "diaochan" then
                                hero:playAction("tielong", true)
                            elseif hero:getResourceId() == "yuanshao"
                              or hero:getResourceId() == "jushou" then
                                hero:setVisible(false)
                            end
                        end
                    elseif _data.guide9 then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "guanyu" then
                                hero:setVisible(false)
                                break
                            end
                        end
                    end
                    if _data.guide1 
                      or _data.guide2
                      or _data.guide3  then
                        for k,hero in pairs(battle:getAllHeroes()) do
                            if hero:getResourceId() == "zhangfei" then
                                hero:changeWeapon(10)
                                performWithDelay(hero, function()
                                    _checkTalk()
                                end, 2)
                                break
                            end
                        end
                    else
                        _checkTalk()
                    end
                end
                local function _checkGuide()
                    local _currentGID = getGuideManager():getCurrentGID()
                    if inArray(_currentGID, GuideGroup.KAI_ZHAN) then
                        getGuideManager():setNextGID(_currentGID)
                        getGuideManager():setObjTarget(nil)
                        local _currentGID = getGuideManager():getCurrentGID()
                        if inArray(_currentGID, GuideGroup.DUI_HUA) then --战斗对话 对话结束演示战斗
                            if _currentGID == 1007 
                              or _currentGID == 2007
                              or _currentGID == GuideKey.KEY_11508
                              then
                                getGuideManager():guideOver()
                                _callBack()
                            else
                                getGuideManager():setHeroFuncCallBack(_callBack)
                                getGuideManager():startGuide()
                            end
                        else
                            getGuideManager():guideOver()
                            _callBack()
                        end
                    else
                        _callBack()
                    end
                end
                _checkGuide()
            end)
        end
    elseif self._battleType == BattleType.TollgateHero then
        -- 名将副本
        _apiName = "TollgateHero_challenge"
        _params.tollgateId = self._battleTypeData.tollgateId
        _callBack = function(data, battleStatus)
            PlayerData:setLastSceneData({
                lastScene = "TollgateHero",
                challengeNum = data.data.challengeNum,
                chapterId = self._battleTypeData.chapterId,
                tollgateId = self._battleTypeData.tollgateId,
                battleStatus = battleStatus,
            })
            data.canShare = true
            data.isFirstFight = false--self._isFirstFight
            _normalCall(data)
        end
    elseif self._battleType == BattleType.QunXiong then --群雄战斗
        _apiName = "Captain_challenge"
        _params.id = self._battleTypeData.mainId
        _callBack = function(data)
            local hasItems = data.data.clientItems and next(data.data.clientItems) and true or false
            PlayerData:setLastSceneData({
                lastScene = "QunXion", 
                hasItems = hasItems,
            })
            data.qunxiongMainId = self._battleTypeData.mainId
            data.canShare = true
            --功能引导
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_12509 then
                zc.setGuideFuncNextStep(_currentGID)
            end
            _normalCall(data)
        end
    elseif self._battleType == BattleType.WorldBoss then
        -- 世界boss
        _apiName = "BossPve_challenge"
        _params.tollgateId = self._battleTypeData.tollgateId
        _callBack = function( data )
            data.canShare = false
            _normalCall(data)
        end
    elseif self._battleType == BattleType.WeaponCopy then
        _apiName = "WeaponCopy_challenge"
        _params.tollgateId = self._battleTypeData.tollgateId
        _params.isBoss = self._battleTypeData.isBoss
        _callBack = function(data)
            local _guideFlag = false
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_13209 then
                getGuideManager():setFuncGuideOver()
                _guideFlag = true
            end
            PlayerData:setLastSceneData({
                lastScene = "WeaponCopy",
                maxTollgateId = data.data.maxTollgateId,
                boss = data.data.boss,
                isNew = data.data.isNew,
                isFirst = data.data.isFirst,
                guideFlag = _guideFlag,
            })
            data.isBoss = self._battleTypeData.isBoss
            if data.isBoss == 1 then
                local _db = gameData.getDataFromCacheWithPrimaryKey("bossWeapon", self._battleTypeData.tollgateId)
                data.bossIcon = _db.icon
            end
            data.canShare = true
            data.isFirstFight = self._isFirstFight
            _normalCall(data)
        end
    elseif self._battleType == BattleType.Wushenshilian then
        _apiName = "Wushenshilian_challenge"
        _params.tollgateId = self._battleTypeData.tollgateId
        _callBack = function(data)
            --新手引导
            local _currentGID = getGuideManager():getCurrentGID()
            if inArray(_currentGID, GuideGroup.KAI_ZHAN) then
                zc.setGuideFuncNextStep(_currentGID)
            end
            local battleState = data.data.warModel[5][1] or 2
            local _success = false
            if inArray(tonumber(battleState), {2,3,4}) then
                _success = true
            end
            PlayerData:setLastSceneData({
                lastScene = "Wushenshilian",
                nowTollgate = data.data.nowTollgate,
                maxTollgate = data.data.maxTollgate,
                success = _success,
            })
            data.canShare = true
            data.warkey = data.data.warkey --武神试练warkey 不知道什么问题
            data.isFirstFight = false--self._isFirstFight
            _normalCall(data)
        end
    elseif self._battleType == BattleType.LimitChallenge then
        _apiName = "LimitChallenge_fight"
        _params.tollgateId = self._battleTypeData.tollgateId
        _params.type = self._battleTypeData.type
        _callBack = function(data)
            data.canShare = true
            data.warkey = data.data.warkey --武神试练warkey 不知道什么问题
            data.isFirstFight = self._isFirstFight
            _normalCall(data)
        end
    elseif self._battleType == BattleType.HeroCopy then
        _apiName = "Hero_CopyChallenge"
        _params.tollgateId = self._battleTypeData.tollgateId
        _params.type = self._battleTypeData.type
        _callBack = function(data)
        print("Hero_CopyChallenge .....")
            dump(data)
            data.canShare = true
            data.warkey = data.data.warkey --武神试练warkey 不知道什么问题
            data.isFirstFight = self._isFirstFight
            _normalCall(data)
        end
    end

    if not _apiName then
        return
    end

    local function gotoBattle()
        zc.sendHttp({
            apiName = _apiName,
            params = _params,
            successCallback = function(backdata)
                local battleStatus, star = self:saveData(backdata)
                if inArray( battleStatus, {2, 3, 4} ) then
                    -- 胜利了，设置关卡状态
                    battleStatus = true
                else
                    battleStatus = false
                end
                local _items = backdata.items or {}
                local _havePopDrop = nil
                for k,v in pairs(_items) do
                    if v.itemType == zc.mainTypeProp then
                        if v.itemId == 449 then
                            _havePopDrop = 1
                            break
                        elseif v.itemId == 450 then
                            _havePopDrop = 2
                            break
                        elseif v.itemId == 451 then
                            _havePopDrop = 3
                            break
                        elseif v.itemId == 452 then
                            _havePopDrop = 4
                            break
                        elseif v.itemId == 453 then
                            _havePopDrop = 5
                            break
                        end
                    end
                end
                local data = {data = backdata}
                data.havePopDrop = _havePopDrop
                data.battleType = self._battleType
                data.fightScene = self._fightScene
                data.bgNode = _bgNode
                data.bgNode:retain()
                data.bgNode:removeFromParentAndCleanup(false)
                _callBack(data, battleStatus, star)
                zc.createArmature({
                    parentPath = "res/Main/battle/battleresult/win/",
                    fileName = "win",
                    async = true,
                })
            end,
        })
    end

    --处理世界boss强攻
    if self._battleType == BattleType.WorldBoss then
        --如果是强攻，花费元宝清除时间再打
        if self._battleTypeData.isForce then 
            zc.sendHttp({
                apiName = "BossPve_clearCD",
                successCallback = function( backData )
                    self._battleTypeData.isForce = false
                    PlayerData:updataData( backData )
                    gotoBattle()
                end
            })
        else
            --直接攻打
            gotoBattle()
        end
    else
        gotoBattle()
    end
end

function BattleUIMatrixLayer:getGuideIndex()
    local _tollDB = gameData.getDataFromCacheWithPrimaryKey("tollgate", self._enemyData.tollgateId)
    local _dataIndex = tonumber(_tollDB.battle) or 0
    return _dataIndex
end

function BattleUIMatrixLayer:getGuideData( data )
    local _data = data
    if self._guideData then
        _data.data.warModel = self._guideData
        local _idx = self:getGuideIndex()
        data["guide" .. _idx] = 1
    end
    return _data
end

---新手引导
function BattleUIMatrixLayer:addGuide()
    local _currentGID = getGuideManager():getCurrentGID()
    if inArray(_currentGID, GuideGroup.KAI_ZHAN) then
        -- if self._battleType == BattleType.PVE then
            getGuideManager():setObjTarget(self._fight)
            getGuideManager():startGuide()
        -- end
    elseif inArray(_currentGID, GuideGroup.DUI_HUA) then
        if _currentGID == GuideKey.KEY_5003 then
            local function _callBack()
                getGuideManager():setNextGID(_currentGID)
                self:addGuide()
            end
            getGuideManager():setHeroFuncCallBack(_callBack)
            getGuideManager():startGuide()
        end
    elseif _currentGID == GuideKey.KEY_5004 then
        local heroId = tonumber( self._formationData["location"..2] ) or 0
        if heroId == GuideHeroId.KEY_5004 then
            getGuideManager():setNextGID(_currentGID)
            self:addGuide()
        else
            getGuideManager():setObjTarget(self._heroBtn[6])
            getGuideManager():startGuide()
            local _hand = getGuideManager().handSprite

            local _startPos = cc.p(_hand:getPositionX() + _hand:getContentSize().width*1.7,
                 _hand:getPositionY() - _hand:getContentSize().height*0.25)
            local _pos1 = cc.p(self._heroBottom[6]:getPosition())
            local _pos2 = cc.p(self._heroBottom[2]:getPosition())
            local _movePos = cc.pSub(_pos2, _pos1)
            local function _runMyAction()
                _hand:setPosition(_startPos)
                _hand:runAction(cc.RepeatForever:create(cc.Sequence:create(
                    cc.DelayTime:create(0.1),
                    cc.MoveBy:create(0.5, _movePos),
                    cc.DelayTime:create(1),
                    cc.CallFunc:create(function()
                        _hand:setPosition(_startPos)
                    end)
                )))
            end
            _runMyAction()
        end
    elseif _currentGID == GuideKey.KEY_2003 then
        zc.addGuideBtn(self._guideBtnHero,self._scale)
        -- if self._battleType == BattleType.DrillField then
        --     getGuideManager():setObjTarget(self._fight)
        --     getGuideManager():startGuide()
        -- end
    elseif _currentGID == GuideKey.KEY_15007 then
        print("GuideKey.KEY_15007")
        getGuideManager():setCurrentGID(GuideKey.KEY_15008)
        getGuideManager():setObjTarget(self._fight)
        getGuideManager():startGuide()
    end
end

-- 删除不在阵上的人物，从当前界面直接返回时调用
function BattleUIMatrixLayer:removeArmature( _battle )
    local _forms = PlayerData:getNowFormationFiles()
    local _remove = {}
    local adm = ccs.ArmatureDataManager:getInstance()
    -- 己方武将
    for i, v in ipairs( self._recordHero ) do
        -- 如果已上阵，不删除
        local _path, _resId = zc.battle.getHeroArmaturePathByResId( v[1], v[2] )
        if not _forms[_resId] and not _remove[_path] then
            -- print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~删除人物资源=", _path)
            adm:removeArmatureFileInfo( _path )
            _remove[_path] = true
        end
    end
    -- 敌人
    if not _battle then
        -- 进入战斗，不删除敌人
        for i, v in ipairs( self._recordEnemy ) do
            local _path, _resId = zc.battle.getHeroArmaturePathByResId( v[1], v[2] )
            if not _forms[_resId] and not _remove[_path] then
                -- print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~删除人物资源=", _path)
                adm:removeArmatureFileInfo( _path )
                _remove[_path] = true
            end
        end
    end
end

function BattleUIMatrixLayer:onCleanup()
    self._exist = false
    ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo( "res/animation/kaizhan/kaizhan.csb", true )
    self:removeAllChildren()
    helper.collectMemory()
end


return BattleUIMatrixLayer