--[[
-- 国家帮助
-- @author lph
-- Creation  2016-4-14
]]
local CountryHelpLayer = class("CountryHelpLayer",function ()
	local layer = cc.Sprite:create()
    layer:setContentSize( 565, 464 )
    layer:setAnchorPoint(cc.p(0,0))
    return layer
end)

function CountryHelpLayer:create(params)
	return CountryHelpLayer.new(params)
end

function CountryHelpLayer:ctor(params)
    self:initData(params)
	self:initUI()
end

function CountryHelpLayer:initData(params)
    self._params = params
    self._size = self:getContentSize()
    --押镖的帮助
    self:setHelpInfo()
end
function CountryHelpLayer:setHelpInfo()
    self._helpsData = {}
    local dartInfo = PlayerData:getDartCountryHelp()
    for i = 1, #dartInfo do
        local v = dartInfo[i]
        if v and v.level then
            if v.etime and PlayerData:getTime() < tonumber(v.etime) then --如果已经超过时间         
                table.insert(self._helpsData,dartInfo[i])
            end
        end
    end
    if self._params.helps then
        for i = 1, #self._params.helps do
            self._helpsData[#self._helpsData+1] = self._params.helps[i]
        end
    end
end
function CountryHelpLayer:initUI()

	self:initHelpsList()
    --下方信息
    local attr_bg = ccui.Scale9Sprite:create( cc.rect( 589/2, 0, 1, 66 ), "res/Main/role/trinket/sellDi.png" )
    attr_bg:setContentSize( cc.size(565, 66) )
    attr_bg:setAnchorPoint(cc.p(0.5, 0))
    attr_bg:setPosition(cc.p(self:getContentSize().width/2, 0))
    self:addChild(attr_bg,1)

    local _tipBg = zc.getScaleNode("res/Main/role/common/shadow_bg.png", cc.size(355, 43))
    _tipBg:setAnchorPoint(cc.p(0,0.5))
    _tipBg:setPosition( 20, attr_bg:getContentSize().height/2 )
    attr_bg:addChild( _tipBg )
    cc.SpriteFrameCache:getInstance():addSpriteFrames("res/Main/home/homeUI.plist", "res/Main/home/homeUI.pvr.ccz")
    local _tipSp = zc.getSprite({
        src = "home_icon_info.png",
        plist = true,
        pos = cc.p( 15, 43/2 ),
        scale = 0.5,
    })
    _tipBg:addChild( _tipSp )

    local _tipLabel = zc.createLabel({
        text = zc.lang("dart_help_tip"),
        outLine = true,
        fontSize = 20,
        ap = cc.p(0, 0.5),
        pos = cc.p(25, 43/2),
        color = cc.c3b(203,183,123)
    })
    _tipBg:addChild( _tipLabel )

    --一键帮助
    local _button = zc.createCommomBtn({
        tag = 5,
        pos = cc.p( 480, attr_bg:getContentSize().height/2 ),
        text = zc.lang("country_onekey_help"),
        fontSize = 30,
        ap = cc.p(0.5,0.5),
        scale = 0.6,
        touchScale = 0.9,
        endCallback = function() 
            self:oneKeyHelp()
        end
    })
    attr_bg:addChild( _button )
end

function CountryHelpLayer:initHelpsList()
    local _tableSize = cc.size(565, 464-66)
    local _tableView = zc.ZCTableView:create(_tableSize)
    _tableView:setPosition(0, 66)
    _tableView:setInertia(true)
    _tableView:setBounceable(true)
    _tableView:setDirection(zctech.SCROLLVIEW_DIRECTION_VERTICAL)
    _tableView:setDelegate()
    _tableView:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    self:addChild(_tableView)
    self._helpsTable = _tableView

    local _cellSize = cc.size(565, 125)

    local function numberOfCellsInTableView( table )
        return #self._helpsData
    end
    local function cellSizeForTable( table, index )
        return _cellSize.height , _cellSize.width
    end
    local function tableCellAtIndex( table, index )
        local cell = table:dequeueCell()
        if cell then
            cell:removeAllChildren()
        else
            cell = zc.ZCTableViewCell:new()
            cell:setContentSize(_cellSize)
        end

        local _di = zc.getSprite({
            src = "res/Main/country/main/country_item_bg.png",
            pos = cc.p(_cellSize.width/2, 55),
        })

        cell:addChild(_di)
        self:createOneCell(_di, index)
        return cell
    end
    _tableView:registerScriptHandler(numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW )
    _tableView:registerScriptHandler(cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX )
    _tableView:registerScriptHandler(tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX )
    _tableView:reloadData()

    self:setEmptyState()
end
--设置空状态
function CountryHelpLayer:setEmptyState()
    if not self._tipNoItemLab then
        local tipNoItemLab = zc.createSprite("res/Main/common/empty.png")
        tipNoItemLab:setPosition(self._size.width/2, self._size.height/2)
        self:addChild(tipNoItemLab)
        self._tipNoItemLab = tipNoItemLab
    end
    if not self._helpsData or table.nums(self._helpsData) == 0 then
        self._tipNoItemLab:setVisible(true)
    else
        self._tipNoItemLab:setVisible(false)
    end
end
function CountryHelpLayer:createOneCell( cell, index )
    local _help_data = self._helpsData[index+1]
    if _help_data then
        if _help_data["isDartHelp"] then
            -- 头像
            local _heroIcon = zc.createPlayerHead({iconId=_help_data.iconId,level=_help_data.level})
            _heroIcon:setPosition(cc.p(70, 58))
            cell:addChild(_heroIcon)
            
            local _member_name_bg = zc.createSprite("res/Main/country/main/member_name_bg.png")
            _member_name_bg:setAnchorPoint(0, 0.5)
            _member_name_bg:setPosition(118, 75)
            cell:addChild(_member_name_bg)
            --成员名称
            local _member_name_label = zc.createLabel({
                text = _help_data.userName,
                fontSize = 20,
                ap = cc.p(0,0.5),
                outLine = true,
                pos = cc.p(10, _member_name_bg:getContentSize().height/2),
            })
            _member_name_bg:addChild(_member_name_label)

            local dartDescBg = zc.getScaleNode("res/Main/country/main/country_notice_bg_s.png", cc.size(275, 40))
            dartDescBg:setAnchorPoint(cc.p(0, 0.5))
            dartDescBg:setPosition(cc.p(112, 32))
            cell:addChild(dartDescBg)

            local dartLevel = tonumber(_help_data["dartLevel"]) or 1            
            local levelImg = zc.createSprite( zc.getHeroQualificationsPath( DartLevelConfig[dartLevel], 2 ) )
            levelImg:setScale( 0.6 )
            levelImg:setPosition(30, 40/2)
            dartDescBg:addChild(levelImg)
            local tipLab = zc.createLabel({
                text = zc.lang("level_dart_help_apply_for"),
                fontSize = 20,
                ap = cc.p(0, 0.5),
                pos = cc.p(50, 40/2),
                color = cc.c3b(72,30,24),
            })
            dartDescBg:addChild(tipLab)

            local _help_button
            _help_button = zc.createPushButton({
                normalFile = "res/Main/country/main/country_dart_btn_u.png",
                selectedFile = "res/Main/country/main/country_dart_btn_u.png",
                disableFile = "res/Main/country/main/country_dart_btn_g.png",
                needSwallow = false,
                touchScale = 0.9,
                endCallback = function()
                    if _help_data.etime and tonumber(_help_data.etime) < PlayerData:getTime() then
                        local _dartInfo = PlayerData._dartCountryHelpInfo or {}
                        for i = 1, #_dartInfo do
                            local v = _dartInfo[i]
                            if v and v.userId and tonumber(v.userId) == tonumber(_help_data.userId) then
                                table.remove(_dartInfo,i)
                                break
                            end
                        end
                        self:setHelpInfo()
                        self:updateHelpTableView()
                        zc.tipInfo(zc.lang("tips_dart_finished"))
                        return
                    end
                    zc.sendHttp({
                        apiName = "CountryDart_appleSupport",
                        params = {targetUserId=_help_data.userId,time=_help_data.getDartTime},
                        successCallback = function (data)
                            zc.tipInfo(zc.lang("tips_dart_to_help_success"))
                            _help_data["alreadyHelp"] = true
                            self:updateHelpTableView()
                            PlayerData:deletDartHelpByTime(_help_data.createTime)
                        end,
                        errorCallback = function(_errorCode,_msg)
                            zc.tipInfo(_msg)
                            if tonumber(_errorCode) == 390 then
                                local _dartInfo = PlayerData._dartCountryHelpInfo or {}
                                for i = 1, #_dartInfo do
                                    local v = _dartInfo[i]
                                    if v and v.userId and tonumber(v.userId) == tonumber(_help_data.userId) then
                                        table.remove(_dartInfo,i)
                                        break
                                    end
                                end
                                self:setHelpInfo()
                                self:updateHelpTableView()
                            end
                        end,
                    })
                end,
                pos = cc.p( 475, 55 ),
            })
            cell:addChild(_help_button)
            if tonumber(GAMEUID) == tonumber(_help_data.requestUserId) or _help_data["alreadyHelp"] then
                _help_button:setEnable(false)
            end
        else
            -- 头像
            local _heroIcon = zc.createPlayerHead({iconId=_help_data.requestUserIconId,level=_help_data.requestUserLevel})
            _heroIcon:setPosition(cc.p(70, 58))
            cell:addChild(_heroIcon)
            
            local _member_name_bg = zc.createSprite("res/Main/country/main/member_name_bg.png")
            _member_name_bg:setAnchorPoint(0, 0.5)
            _member_name_bg:setPosition(118, 75)
            cell:addChild(_member_name_bg)
            --成员名称
            local _member_name_label = zc.createLabel({
                text = _help_data.requestUserName,
                fontSize = 20,
                ap = cc.p(0,0.5),
                outLine = true,
                pos = cc.p(10, _member_name_bg:getContentSize().height/2),
            })
            _member_name_bg:addChild(_member_name_label)

            --进度
            self:initProgressTimer(cell,_help_data)

            local _text_desc = ""
            local _status = tonumber(_help_data.status) or 0
            if tonu_status == 1 or _status == 2 then
                local _level = tonumber(_help_data.houseLevel) or 1
                _text_desc = _level .. zc.lang("level_1") .. LANGUAGE_getHouseNameById(_help_data.houseId)
            end
            --建筑描述
            local _help_desc_label = zc.createLabel({
                text = _text_desc,
                fontSize = 22,
                ap = cc.p(0,0.5),
                outLine = false,
                pos = cc.p(285, 58-25),
                color = cc.c3b(76,18,18),
            })
            cell:addChild(_help_desc_label)

            --帮助
            local _disable = zc.createSprite("res/Main/country/main/country_help_btn_d.png")
            zc.setGray(_disable, true)
            local _help_button
            _help_button = zc.createPushButton({
                normalFile = "res/Main/country/main/country_help_btn_u.png",
                selectedFile = "res/Main/country/main/country_help_btn_d.png",
                disableNode = _disable,
                needSwallow = false,
                touchScale = 0.9,
                endCallback = function()
                    zc.sendHttp({
                        apiName = "Country_help",
                        params = {houseIds=_help_data.housePrimaryId},
                        successCallback = function (data)
                           zc.tipInfo(zc.lang("tips_country_to_help_success"))
                           self._helpsData = data.helps
                           self:updateHelpTableView()
                        end,
                    })
                end,
                pos = cc.p( 475, 55 ),
            })
            cell:addChild(_help_button)   
            if tonumber(GAMEUID) == tonumber(_help_data.requestUserId) or self:isHelpedByUserId(_help_data) or tonumber(_help_data.helpNum) >= tonumber(_help_data.helpNumLimit) then 
                _help_button:setEnable(false)
            end
        end
    end
end

function CountryHelpLayer:initProgressTimer(_bg,_helpData)
    local expImgBg = cc.Sprite:create("res/Main/home/home_create_time_prograss3.png")
    expImgBg:setAnchorPoint(cc.p(0, 0.5))
    local expGressImg = cc.Sprite:create("res/Main/home/home_create_time_prograss4.png")
    expGressImg:setAnchorPoint(cc.p(0, 0.5))
    self.expPrgress = cc.ProgressTimer:create(expGressImg)
    self.expPrgress:setAnchorPoint(cc.p(0, 0.5))
    self.expPrgress:setType(cc.PROGRESS_TIMER_TYPE_BAR)
    self.expPrgress:setBarChangeRate(cc.p(1, 0))
    self.expPrgress:setMidpoint(cc.p(0, 0))
    _bg:addChild(expImgBg, 0)
    expImgBg:setPosition(cc.p(120,58-25))
    self.expPrgress:setPosition(cc.p(120,58-25))
    _bg:addChild(self.expPrgress, 1)
    local _percent = tonumber(_helpData.helpNum)/tonumber(_helpData.helpNumLimit)*100
    self.expPrgress:setPercentage(_percent)

    --帮助数量
    local _help_num_label = zc.createLabel({
        text = _helpData.helpNum .. "/" .. _helpData.helpNumLimit,
        fontSize = 18,
        outLine = true,
        pos = cc.p(self.expPrgress:getPositionX()+self.expPrgress:getContentSize().width/2, self.expPrgress:getPositionY()),
        color = display.COLOR_GREEN,
    })
    _bg:addChild(_help_num_label, 1)
end

-- 一键帮助
function CountryHelpLayer:oneKeyHelp()
    local _houseIds = ""
    local _helpTempData = {}
    local _dartHelp = {}
    for k,v in pairs(self._helpsData) do
        if v and v.helpUserList then
            local _flag = self:isHelpedByUserId(v)
            if tonumber(GAMEUID) ~= tonumber(v.requestUserId) and not _flag then 
                _helpTempData[#_helpTempData+1] = v
            end
        else
            _dartHelp[#_dartHelp+1] = v
        end
    end
    for k,v in pairs(_helpTempData) do
        if k == #_helpTempData then
            _houseIds =  _houseIds .. v.housePrimaryId
        else
            _houseIds =  _houseIds .. v.housePrimaryId .. "|"
        end
    end
    if _houseIds == "" then
        zc.tipInfo(zc.lang("tips_country_have_no_to_help"))
        return 
    end

    zc.sendHttp({
        apiName = "Country_help",
        params = {houseIds=_houseIds},
        successCallback = function (data)
           zc.tipInfo(zc.lang("tips_country_to_help_success"))
           self._helpsData = _dartHelp
           for k,v in pairs(data.helps) do
                self._helpsData[#self._helpsData+1] = v
           end
           self:updateHelpTableView()
        end,
    })
end

--判断当前建筑是否被帮助过
function CountryHelpLayer:isHelpedByUserId(_helpData)
    if _helpData.helpUserList == "" then
        return false
    else
        return inArray(GAMEUID,_helpData.helpUserList)
    end
    return false
end


--更新帮助列表
function CountryHelpLayer:updateHelpTableView()
    self._helpsTable:reloadDataAndScrollToCurrentCell()
    self:setEmptyState()
end

return CountryHelpLayer