--[[
	-- 校场武将界面
	-- @author lph
	-- Creation  2015-12-30
]]
local DrillFieldHeroListLayer = class("DrillFieldHeroListLayer", function ()
    return zc.createNormalDialog({
        closeCallBack = function()
            LayerManager.removeLayout(nil, nil, false)
        end
    })
end)

function DrillFieldHeroListLayer:create( params )
	return DrillFieldHeroListLayer.new( params )
end

function DrillFieldHeroListLayer:onCleanup()
    self:removeAllChildren()
    local _forms = PlayerData:getNowFormationFiles()
    for k, v in pairs( self._record ) do
        -- 如果已上阵，不删除
        local _path, _resId = zc.battle.getHeroArmaturePathByResId( k )
        if not _forms[_resId] then
            ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo( _path )
        end
    end
    helper.collectMemory()
end

function DrillFieldHeroListLayer:ctor( params )
    self._record = {}
	self:initData( params )
    if #self._heroList == 0 then
        -- 没有武将可以训练
        self._exist = false
    else
        -- 有武将可以训练
        self._exist = true
        self:show()
    	self:initUI()
    	self:refreshHeroInfo()
    end
end

function DrillFieldHeroListLayer:initData( params )
	self._typeData = params.typeData
	table.sort(self._typeData, function( a, b )
		return tonumber( a.time ) < tonumber( b.time )
	end)
	self._position = params.position
	self._beginDrillFun = params.beginDrillFun
    self._rcntFunc = params.rcntFunc
	-- 正在训练的武将数据
	local trainingStatus = {}
	for i, v in ipairs( params.positionData ) do
		if v.hero and v.hero.id then
			trainingStatus[tonumber( v.hero.id )] = true
		end
	end
	-- 可以训练的武将数据
	self._heroList = {}
    local _heroId = tonumber( params.heroId ) or 0
    if _heroId ~= 0 then
        -- 如果是转生
        for i, v in ipairs( PlayerData:getGeneralInfo() ) do
            if tonumber(v.status) == 1 and tonumber(v.id) == _heroId then
                self._heroList[#self._heroList + 1] = v
            end
        end
        self._rcntFlag = true
    else
    	-- 主角id
    	local mainRoleId = tonumber(PlayerData.info.heroId)
    	for i, v in ipairs( PlayerData:getGeneralInfo() ) do
    		if tonumber(v.status) == 1 and tonumber(v.id) ~= mainRoleId then
    			-- 上阵武将并且武将不是主角
                if not trainingStatus[tonumber( v.id )] then
                    self._heroList[#self._heroList + 1] = v
                end
    		end
    	end
    end
    --新手引导将孙坚放到最前面
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_12711 then
        table.sort(self._heroList, function( a, b )
            return tonumber( a.heroId ) < tonumber( b.heroId )
        end)
    else
        -- 启用阵法
        local _formationList = PlayerData:getFormationsInfo()
        local _formationData = nil
        for i, v in pairs( _formationList ) do
            local _status = tonumber( v.status ) or 0
            if _status == 1 then
                _formationData = v
            end
        end
        -- 上阵武将
        local _upHero = {}
        for i = 1, 9 do
            local _id = _formationData["location"..tostring( i )] or 0
            _upHero[tonumber( _id )] = true
        end
        table.sort(self._heroList, function( a, b )
            local _id_a = tonumber( a.id ) or 0
            local _id_b = tonumber( b.id ) or 0
            local _up_a = _upHero[_id_a] or false
            local _up_b = _upHero[_id_b] or false
            if _up_a ~= _up_b then
                return _up_a
            else
                return tonumber( a.id ) > tonumber( b.id )
            end
        end)
    end

	-- 默认选中第一个英雄
	self._heroIndex = 1
	-- 默认选中第一种训练消耗
	self._checkIndex = 1
	-- 默认的训练类型
	self._typeIndex = 1

	self._size = self:getContentSize()
end

function DrillFieldHeroListLayer:initUI()
	-- 背景
    local bg = zc.createNormalPop({
        tag = 2,
        src = "",
        bianZOrder = 1,
    })
    bg:setPosition(zc.width*0.5, zc.height*0.5 - 10)
    self:addChild(bg)
    local _bgSize = bg:getContentSize()
    self._bgSize = _bgSize
    self._bg = bg

    local _titleImg = bg:getChildByName("titleImg")
    local title = zc.getSprite({
        src = "res/Main/drillField/drillField_title.png",
        ap = cc.p(0.5, 0),
        pos = cc.p(120, _bgSize.height - _titleImg:getContentSize().height - 15)
    })
    bg:addChild(title, 1)

    -- 武将列表
	self:initHeroList()
	-- 武将信息
	self:initHeroInfo()
	-- 训练
    self:initTrainInfo()
    -- 转生
    self:initRebirthInfo()

    -- 屏蔽层
    local _touchLayer = zc.createPushButton({
        touchSize = self._size,
        pos = cc.p( self._size.width/2, self._size.height/2 ),
        needSwallow = true,
    })
    _touchLayer:setEnable(false)
    self:addChild( _touchLayer, 110 )
    self._touchLayer = _touchLayer
end
-- 武将列表
function DrillFieldHeroListLayer:initHeroList()
	-- 背景
	local bg = cc.Sprite:create( "res/Main/matrix/matrix_generalbg.png" )
	bg:setAnchorPoint( 0, 0 )
	bg:setPosition( -6, 9 )
	self._bg:addChild( bg )
	-- title
	local title = cc.Sprite:create( "res/Main/matrix/matrix_bar1.png" )
	title:setAnchorPoint( 0, 0 )
	title:setPosition( 15, 390 )
	bg:addChild( title )

    local _label = zc.createRichLabel({
        text = zc.lang("role_choose"),
        fontSize = 20,
        anchor = cc.p(0, 0.5),
        pos = cc.p(15, title:getContentSize().height*0.5 + 5),
    })
    _label:setGradientColor(cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1))
    title:addChild(_label)
	-- tableview
	local tableView = zc.ZCTableView:create(cc.size(190, 400))
    tableView:setDirection(cc.SCROLLVIEW_DIRECTION_VERTICAL)
    tableView:setVerticalFillOrder(cc.TABLEVIEW_FILL_TOPDOWN)
    tableView:setPosition(cc.p(13, 0))
    tableView:setDelegate()
    bg:addChild(tableView)
    local cellWidth = 190
    local cellHeight = 100
    local space = 10
    local function cellSizeForTable(table, index) 
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex(table, index)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
        end
        cell:removeAllChildren()
        -- 更新界面
        for i = 1, 2 do
        	local data = self._heroList[index*2 + i]
        	if data then
        		local icon = zc.createIconByData({
	                data = {
	                    mainType = zc.mainTypeSoldier,
	                    type = data.heroId, 
	                    id = data.heroId,
	                    quality = data.star,
	                    level = zc.getLevelTipShow()..data.level,
                        reincarnate = data.reincarnate,
	                },
                    quality = data.quality,
                    showLevel = true,
	                listener = function()
                        local _index = index*2 + i
                        if self._heroIndex ~= _index then
    	                	self._heroIndex = _index
    	                	self:refreshHeroInfo()
                        end
	                end
	            })
	            icon:setScale( 0.65 )
	            icon:setPosition( ( cellWidth - space )/4*( 2*i - 1 ) + space/2, cellHeight/2 )
	            cell:addChild( icon )
        	end
        end
        return cell
    end
    local function numberOfCellsInTableView()
        return math.ceil( #self._heroList/2 )
    end
    tableView:registerScriptHandler(numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    tableView:registerScriptHandler(cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX)
    tableView:registerScriptHandler(tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX)
    tableView:reloadData()
    self._heroTableview = tableView
end
-- 武将信息
function DrillFieldHeroListLayer:initHeroInfo()
	-- bg
    local _width = 666
    local bg = zc.createNormalPop({
        width = _width,
        height = 490,
        showSide = false,
    })
    bg:setAnchorPoint(0, 0)
    bg:setPosition(198, 0)
    self._bg:addChild( bg )
    self._heroInfoBg = bg
    -- 法阵背景
    local heroBg = zc.createSpine({
        fileName = "jsxxyxg",
        actName = "animation",
    })
    heroBg:setPosition(220, 320)
    bg:addChild(heroBg)
    self._heroBg = heroBg
    -- local heroBg = cc.Sprite:create( "res/Main/role/role_magic_bg.png" )
    -- heroBg:setPosition( 220, 330 )
    -- bg:addChild( heroBg )
    -- 经验条
    local expBg = cc.Sprite:create( "res/Main/drillField/drillField_heroExp_bg_1.png" )
    expBg:setAnchorPoint( 0, 0.5 )
    expBg:setPosition( heroBg:getPosition() - 80, heroBg:getPositionY() - 110)
    expBg:setScale( 0.65 )
    bg:addChild( expBg )
    local expBar = cc.ProgressTimer:create(cc.Sprite:create("res/Main/drillField/drillField_heroExp_1.png"))
    expBar:setType(cc.PROGRESS_TIMER_TYPE_BAR)
    expBar:setMidpoint(cc.p(0, 0.5))
    expBar:setBarChangeRate(cc.p(1, 0))
    expBar:setPosition(expBg:getContentSize().width/2, expBg:getContentSize().height/2)
    expBg:addChild(expBar)
    self._expBar = expBar
    local expAnm = zc.createSpine({
        fileName = "zhuansheng_ef",
        actName = "EXP",
        autoRemove = false,
        loop = true,
        needInit = false,
    })
    expAnm:setScaleX( 1.1 )
    expAnm:setPosition( expBg:getPositionX() + expBg:getContentSize().width/3, expBg:getPositionY() )
    bg:addChild( expAnm )
    self._expAnm = expAnm
    -- 等级
    local levelBg = cc.Sprite:create( "res/Main/common/avator_levelbg.png" )
    levelBg:setPosition( expBg:getPositionX(), expBg:getPositionY() )
    bg:addChild( levelBg )
    local level = zc.createLabel({
        fontSize = 22,
        ap = cc.p(0.5, 0.5),
        pos = cc.p(levelBg:getContentSize().width/2, levelBg:getContentSize().height/2),
        outLine = true,
	})
    levelBg:addChild( level )
    self._level = level

    -- 名字背景 
    local nameBg = zc.getScaleNode("res/Main/role/common/name_bg.png", cc.size(230, 30))
    nameBg:setPosition(heroBg:getPositionX() + 270, heroBg:getPositionY() + 30)
    bg:addChild(nameBg)

    local _titleLabel = zc.createRichLabel({
        text = "",
        fontSize = 22,
        pos = cc.p(nameBg:getPositionX(), nameBg:getPositionY() + 35),
    })
    bg:addChild(_titleLabel)
    -- _titleLabel:enableShadow(cc.size(2, -2), 0xffffffff, 1)
    self._titleLabel = _titleLabel

    local _titleLeftSp = zc.getSprite({
        src = "res/Main/common/lace.png",
        ap = cc.p(1, 0),
        pos = cc.p(-10, 8),
    })
    _titleLabel:addChild(_titleLeftSp)

    local _titleRightSp = zc.getSprite({
        src = "res/Main/common/lace.png",
        ap = cc.p(1, 0),
        pos = cc.p(0, 8),
    })
    _titleRightSp:setScaleX(-1)
    _titleLabel:addChild(_titleRightSp)
    _titleLabel._titleRightSp = _titleRightSp
    -- 名字
    local name = zc.createLabel({
        fontSize = 22,
        ap = cc.p(0.5, 0.5),
        pos = cc.p(nameBg:getContentSize().width/2, nameBg:getContentSize().height/2),
        outLine = true,
    })
    nameBg:addChild( name )
    self._name = name
    -- 兵器图标
    local _titleImg = self._bg:getChildByName("titleImg")
    local weaponBg = zc.getSprite({
        src = "res/Main/role/common/weapon_icon_bg.png",
        ap = cc.p(0.5, 0),
        pos = cc.p(275, 0),
    })
    _titleImg:addChild(weaponBg)

    local weaponSp = cc.Sprite:create()
    weaponSp:setPosition( weaponBg:getContentSize().width*0.5 + 2, -10)
    weaponBg:addChild( weaponSp )
    self._weaponSp = weaponSp
    -- 属性
	self._attr = {}
	for i = 1, 3 do
        local _attrBar = zc.getScaleNode("res/Main/role/common/name_bg.png", cc.size(230, 30))
        _attrBar:setPosition(nameBg:getPositionX(), nameBg:getPositionY() - i*34)
        bg:addChild(_attrBar)

        local _label = zc.createRichLabel({
            text = LANGUAGE_getAttributeTextById(i) .. ":",
            fontSize = 20,
            anchor = cc.p(0, 0.5),
            pos = cc.p(5, _attrBar:getContentSize().height*0.5),
        })
        _label:setGradientColor(cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1))
        _attrBar:addChild(_label)

        local num = zc.createLabel({
            fontSize = 20,
            ap = cc.p(1, 0.5),
            pos = cc.p(_attrBar:getContentSize().width - 5, _attrBar:getContentSize().height*0.5 ),
            outLine = true,
        })
        _attrBar:addChild( num )
        self._attr[i] = num
	end
end
-- 刷新武将信息
function DrillFieldHeroListLayer:refreshHeroInfo()
	local data = self._heroList[self._heroIndex]
	-- dump( data, "data" )
    -- 星星
    zc.createHeroStar( self._heroInfoBg, data.star,cc.p(78, 420))
    -- 经验条
    self._expBar:setPercentage( data.exp/data.expLimit*100 )
    -- 等级
    self._level:setText( data.level )
    -- 英雄
    if self._armature then
    	self._armature:removeFromParent()
    	self._armature = nil
    end
    local _pos = cc.p(self._heroBg:getPositionX(), self._heroBg:getPositionY() - 80)
    _pos = self._heroBg:getParent():convertToWorldSpace(_pos)
    _pos = self._bg:convertToNodeSpace(_pos)
    local armature = zc.battle.createHeroArmatureWithResId(data.modelId)
    self._record[data.modelId] = true
    armature:setPosition(_pos)
    self._bg:addChild( armature, 1 )
    armature:setScaleX( -1.2 )
    armature:setScaleY( 1.2 )
    armature:getAnimation():play("idle")
    self._armature = armature
    local function animationEvent(armatureBack,movementType,movementID)
        if movementType == ccs.MovementEventType.complete then
            self._armature:getAnimation():play("idle")
        end
    end
    self._armature:getAnimation():setMovementEventCallFunc(animationEvent)
    -- 更换兵器
    zc.soldierChangeWeapon( armature, ( PlayerData:getWeaponsInfoById( data.id ) or {} ).weaponId or 0 )
    -- 按钮
    local heroBtn = zc.createPushButton({
        touchSize = cc.size( 200, 210 ),
        pos = cc.p( armature:getPositionX(), armature:getPositionY() + 100 ),
        endCallback = function()
            local _action = zc.random(1, 3)
            if self._action == _action then
                if _action < 3 then
                    _action = _action + 1
                else 
                   _action = 1 
                end
            end
            self._action = _action
            local _actions = {"atk3", "atk", "atk0"}
            self._armature:getAnimation():play( _actions[_action], -1, 0 )
        end
    })
    self._bg:addChild(heroBtn, 1)
    
	-- 名字
    if self._titleLabel then
        local _godName = data.title or ""
        self._titleLabel:setString("[color=ff4c1212]" .. _godName .. "[/color]")
        self._titleLabel._titleRightSp:setPositionX(self._titleLabel:getContentSize().width + 10)
    end
    local _nameText = data.name
    local _newAdvance = tonumber(data.newAdvance) or 0
    if _newAdvance > 0 then
        _nameText = _nameText .. "+" .. _newAdvance
    end
	self._name:setString( _nameText )
	-- 兵器图标
	self._weaponSp:setTexture( zc.getHeroTypeBigImg( gameData.getDataFromCacheWithPrimaryKey("hero", data.heroId).weaponType ) )
	-- 进度条
    local _config = {
        "force", "wisdom", "soma"
    }
	for i=1,3 do
        self._attr[i]:setText( data["attribute"][_config[i]] )
    end

	local _reincarnateLevel = 50
    self._attrStep = "1.0"
    local _reincarnateNum = data.reincarnate + 1
    local _data = gameData.getDataFromCache("hero_reincarnate", {level = _reincarnateNum,heroType = 1})
    if next(_data) ~= nil then 
        _reincarnateLevel = _data.needLevel
        self._attrStep = tostring(_data.attackStep) .. "/" .. tostring(_data.defenseStep) .. "/" .. tostring(_data.attackStrategyStep) .. "/" .. tostring(_data.defenseStrategyStep) .. "/" .. tostring(_data.hpStep)
    end
    if tonumber( data.level ) == tonumber( _reincarnateLevel ) then
    	-- 转生
        self._trainInfoBg:setVisible( false )
        self._rebirthInfoBg:setVisible( true )
        -- 动画需求经验条满
        self._expBar:setPercentage( 100 )
        self._expAnm:setVisible( true )
    else
    	-- 训练
    	self:refreshTrainInfo()
        self._trainInfoBg:setVisible( true )
        self._rebirthInfoBg:setVisible( false )
        self._expAnm:setVisible( false )
    end
end
-- 训练信息
function DrillFieldHeroListLayer:initTrainInfo()
	-- bg
    local bgSize = cc.size(self._heroInfoBg:getContentSize().width - 8, 170)
    local bg = zc.getScaleNode("res/Main/role/trinket/sellDi.png", bgSize)
    bg:setAnchorPoint(0, 0)
    bg:setPosition(4, 5)
    self._heroInfoBg:addChild(bg)
    self._trainInfoBg = bg
    
    -- 训练花费
    local tipBg = zc.getScaleNode("res/Main/role/common/name_bg.png", cc.size(130, 32))
    tipBg:setPosition(95, 120)
    bg:addChild( tipBg )
    local costLabel = zc.createLabel({
        text = zc.lang("drill_pay")..": ",
        fontSize = 22,
        pos = cc.p(tipBg:getContentSize().width/2, tipBg:getContentSize().height/2),
        outLine = true,
    })
    tipBg:addChild(costLabel)
    -- 选项
    self._checkBox = {}
    local choiceConfig = {
        zc.dataTypeMoney,
        zc.dataTypeGold,
	}

    local choiceBgSize = cc.size(220, 43)
    for i, v in ipairs( choiceConfig ) do
        -- 训练选择背景
        self._checkBox[i] = {}
        local _size = cc.size(230, 45)
        local _normalNode = zc.getScaleNode("res/Main/common/checkBar_u.png", _size)
        local _selectedNode = zc.getScaleNode("res/Main/common/checkBar_d.png", _size)
	    local checkBox = zc.createPushButton({
			normalNode = _normalNode,
			selectedNode = _selectedNode,
			pos = cc.p(280 + 240*(i-1), tipBg:getPositionY()),
			endCallback = function()
				if self._checkIndex ~= i then
					self._checkBox[self._checkIndex].btn:setEnable( true )
                    self._checkBox[self._checkIndex].check:setVisible( false )
					self._checkBox[i].btn:setEnable( false )
                    self._checkBox[i].check:setVisible( true )
					self._checkIndex = i
				end
			end
		})
		bg:addChild( checkBox )

        local _check = zc.createSprite("res/Main/common/check.png")
        checkBox:addChild(_check)
        _check:setPosition(26, checkBox:getContentSize().height*0.5)
        _check:setVisible(false)

		self._checkBox[i].btn = checkBox
        self._checkBox[i].check = _check
		local icon = zc.getSprite({
            src = zc.getImgByType(v),
            pos = cc.p(76, checkBox:getContentSize().height*0.5),
        })
    	checkBox:addChild(icon)
    	local price = zc.createLabel({
	        fontSize = 22,
	        ap = cc.p(0, 0.5),
	        pos = cc.p( icon:getPositionX() + 25, checkBox:getContentSize().height*0.5),
	        outLine = true,
    	})
    	checkBox:addChild(price)
		self._checkBox[i].price = price
    end
    self._checkBox[self._checkIndex].btn:setEnable( false )
    self._checkBox[self._checkIndex].check:setVisible( true )
    

    -- 训练时间背景
    local trainTimeBg = ccui.Scale9Sprite:create( cc.rect( 8, 8, 38, 38 ), "res/Main/drillField/drillField_text_bg_s.png" )
    trainTimeBg:setContentSize( 200, 30 )
    trainTimeBg:setPosition( 295, 50 )
    bg:addChild( trainTimeBg )

    -- 训练时间
    local tipBg = zc.getScaleNode("res/Main/role/common/name_bg.png", cc.size(130, 32))
    tipBg:setPosition(95, trainTimeBg:getPositionY())
    bg:addChild( tipBg )
    local trainTimeLabel = zc.createLabel({
        text = zc.lang("drill_time")..": ",
        fontSize = 22,
        pos = cc.p(tipBg:getContentSize().width*0.5, tipBg:getContentSize().height*0.5),
        outLine = true,
    })
    tipBg:addChild( trainTimeLabel )
    local trainTime = zc.createLabel({
        fontSize = 22,
        pos = cc.p(trainTimeBg:getContentSize().width/2, trainTimeBg:getContentSize().height/2),
        outLine = true,
    })
    trainTimeBg:addChild( trainTime )
    self._trainTime = trainTime
    -- 减
    local sub = zc.createPushButton({
    	normalFile = "res/Main/common/button/common_btn8_u.png",
    	selectedFile = "res/Main/common/button/common_btn8_d.png",
    	pos = cc.p( 195, trainTimeBg:getPositionY() ),
    	endCallback = function()
    		if self._typeIndex > 1 then
    			self._typeIndex = self._typeIndex - 1
    			self:refreshTrainInfo()
    		end
    	end
	})
    bg:addChild( sub )
    -- 加
    local add = zc.createPushButton({
    	normalFile = "res/Main/common/button/common_btn2_u.png",
    	selectedFile = "res/Main/common/button/common_btn2_d.png",
    	pos = cc.p( 395, trainTimeBg:getPositionY() ),
    	endCallback = function()
    		if self._typeIndex < #self._typeData then
    			self._typeIndex = self._typeIndex + 1
    			self:refreshTrainInfo()
    		end
    	end
	})
    bg:addChild( add )
    -- 开始训练
    local startTrainBtn = zc.createCommomBtn({
        tag = 1,
        text = zc.lang("begin_drill"),
        pos = cc.p(self._heroInfoBg:getContentSize().width - 130, trainTimeBg:getPositionY()), 
        listener = function()
            self:startTrain()
        end,
    })
    bg:addChild( startTrainBtn )
    self._startTrainBtn = startTrainBtn
end
-- 刷新训练信息
function DrillFieldHeroListLayer:refreshTrainInfo()
	local data = self._typeData[self._typeIndex]
	self._checkBox[1].price:setText( data.costMoney )
	self._checkBox[2].price:setText( data.costGold )
	self._trainTime:setText( zc.timeToClock( data.time, false ) )
end
-- 转生界面
function DrillFieldHeroListLayer:initRebirthInfo()
	-- bg
    local bgSize = cc.size(self._heroInfoBg:getContentSize().width - 8, 170)
    local bg = zc.getScaleNode("res/Main/role/trinket/sellDi.png", bgSize)
    bg:setAnchorPoint(0, 0)
    bg:setPosition(4, 5)
    self._heroInfoBg:addChild(bg)
    self._rebirthInfoBg = bg
	-- 说明
	local tipBg = zc.getScaleNode("res/Main/role/common/name_bg.png", cc.size(450, 140))
	tipBg:setAnchorPoint( 0, 0.5 )
	tipBg:setPosition(15, bgSize.height*0.5 )
	bg:addChild( tipBg )

    local tipY = tipBg:getContentSize().height - 18
    for i = 1, 4 do
        local icon = zc.getSprite({
            src = "res/Main/vip/vip_dot.png",
            pos = cc.p(24, tipY),
            scale = 0.9,
        })
        tipBg:addChild( icon )
        local tip = zc.createLabel({
            text = zc.lang("reincarnate_desc_" .. i),
            fontSize = 18,
            ap = cc.p(0, 1),
            pos = cc.p(icon:getPositionX() + 18, tipY + 10),
            outLine = true,
            width = 400,
        })
        tipBg:addChild(tip)
        tipY = tipY - tip:getContentSize().height - 3
    end
	
    -- 开始转生
    local startRebirthBtn = zc.createCommomBtn({
        tag = 1,
        text = zc.lang("reincarnate"),
        pos = cc.p(self._heroInfoBg:getContentSize().width - 100, 78), 
        listener = function() 
            self:startRebirth()
        end,
    })
    bg:addChild( startRebirthBtn )
end

-- 开始训练
function DrillFieldHeroListLayer:startTrain()
    if not self._position then
        zc.tipInfo(zc.lang("tips_unselected_train_position"))
        return
    end
	if self._checkIndex == 1 then
		-- 银币
		if zc.getNumByTypeId( zc.mainTypeResource, zc.dataTypeMoney ) < tonumber( self._typeData[self._typeIndex].costMoney ) then
			zc.tipInfo( zc.lang("tips_money_deficiency") )
			return
		end
	elseif self._checkIndex == 2 then
		-- 元宝
		if zc.getNumByTypeId( zc.mainTypeResource, zc.dataTypeGold ) < tonumber( self._typeData[self._typeIndex].costGold ) then
			zc.tipInfo( zc.lang("tips_gold_unenough") )
			return
		end
	end
	local houseData = PlayerData:getHouseInfoByHouseId(8)
	zc.sendHttp({
		apiName = "Training_training",
		params = {
			houseId = houseData.id,
			heroId = self._heroList[self._heroIndex].id,
			type = self._typeData[self._typeIndex].type,
			costType = self._checkIndex,
			position = self._position,
		},
		successCallback = function(data)
    		-- dump(data)
            --功能引导
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_12711 then
                zc.setGuideFuncNextStep(_currentGID)
            end
    		PlayerData:updataData(data)
    		self._beginDrillFun(data)
            LayerManager.removeLayout(nil, nil, false)
        end
    })
end
-- 开始转生
function DrillFieldHeroListLayer:startRebirth()
	zc.sendHttp({
        apiName = "Hero_reincarnate",
        params = {id = self._heroList[self._heroIndex].id},
        successCallback = function (data)
            -- dump(data)
            PlayerData:updateGeneralInfo(data)
            self._touchLayer:setEnable( true )
            self:showRebirthAnimation(function()
                self._touchLayer:setEnable( false )
                if self._rcntFlag then
                    if self._rcntFunc then
                        self._rcntFunc( data )
                    end
                    LayerManager.removeLayout(nil, nil, false)
                else
                    self:refreshHeroInfo()
                    self._heroTableview:reloadData()
                end
            end)
        end
    })
end

function DrillFieldHeroListLayer:showRebirthAnimation(callback)
    -- 人物
    local _heroAnm = zc.createSpine({
        fileName = "zhuansheng_ef",
        actName = "zhuansheng",
        autoRemove = true,
        loop = false,
        needInit = false,
        removeCallback = callback,
    })
    _heroAnm:setScale( 2 )
    local _posX, _posY = self._armature:getPosition()
    _heroAnm:setPosition( _posX, _posY + 140 )
    self._bg:addChild( _heroAnm, 3 )
end

function DrillFieldHeroListLayer:onEnter()
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_12711 then
        getGuideManager():addMaskLayer()
        performWithDelay(self,function ()
            getGuideManager():removeMaskLayer()
            getGuideManager():setObjTarget(self._startTrainBtn)
            getGuideManager():startGuide()
        end,0.5) 
    end
end


return DrillFieldHeroListLayer