--[[
    FileName: GeneralPreviewLayer.lua
    Author: andong
    Date: 2016-03-23
    Purpose: xx界面
]]
local GeneralPreviewLayer = class( "GeneralPreviewLayer", function ()
    return zc.createNormalDialog()
end)
function GeneralPreviewLayer:create(parmas)
    return GeneralPreviewLayer.new(parmas)
end
function GeneralPreviewLayer:ctor(parmas)
    self:initData(parmas)
    self:initUI()
end

function GeneralPreviewLayer:initData(parmas)
    if parmas and parmas.callback then
        self._callback = parmas.callback
    end
    self._subTag = nil
    self.seatNum = 3

    --玩家等级和酒馆等级
    self._playerLevel = PlayerData:getUserLevel()
    local _houseInfo = PlayerData:getHouseInfoByHouseId(9) or {}
    self._houseLevel = tonumber(_houseInfo["level"]) or 1
    --初始化开启状态
    self:initTabOpenState()

    local ten_luck_draw = gameData.getDataFromCache( "ten_luck_draw" )
    local _pictureList = {}
    local _listTb = {}
    local rewardMap = {} --记录重复
    for k,v in pairs(ten_luck_draw) do
        if v and v.scroll then
            local itemType = tonumber(v.itemType)
            local itemId = tonumber(v.itemId)

            if itemType and itemType == 2 then
                local scroll = tonumber(v.scroll) or 1
                local _open = self:checkTabOpen(scroll)
                if _open then
                    if not _pictureList[scroll] then
                        _pictureList[scroll] = {}
                    end
                    if not rewardMap[scroll] then
                        rewardMap[scroll] = {}
                        _listTb[scroll] = scroll
                    end
                    local _mapList = rewardMap[scroll]
                    if not _mapList[itemId] then
                        local _list = _pictureList[scroll]
                        local idx = #_list + 1
                        _list[idx] = v
                        local gengeralInfo = gameData.getDataFromCacheWithPrimaryKey("hero",  itemId) or {}
                        _list[idx].qualifications = tonumber(gengeralInfo.qualifications) or 1
                        _list[idx].title = gengeralInfo.title
                        _mapList[itemId] = idx
                    end
                else
                    if scroll ~= 5 then
                        _listTb[scroll] = scroll
                    end
                end                
            end
        end 
    end
    table.sort(_listTb, function( a, b )
        local scrollA = tonumber(a) or 1
        local scrollB = tonumber(b) or 1
        if scrollA == 5 then
            return true
        end
        if scrollB == 5 then
            return false
        end
        return scrollA < scrollB
    end)
    --
    self._qualificationsToColor = {
        "green",--"c",
        "blue",--"b",
        "blue",--"b+",
        "purple",--"a",
        "purple",--"a+",
        "orange",--"s",
        "orange",--"s+",
        "orange",--"ss",
        "orange",--"ss+",
    }
    local _tb, _tab, _flag
    for k,v in pairs(_pictureList) do
        _tb = {}
        _tab = {}
        for key, value in pairs(v) do
            _flag = 0
            for i = 1, #_tb do
                if _tb[i] == value.qualifications then
                    _flag = i
                    break
                end
            end
            if _flag == 0 then
                _flag = #_tb + 1
                table.insert(_tb, value.qualifications)
                _tab[_flag] = {}
            end
            table.insert(_tab[_flag], value)
        end
        table.sort(_tab, function( a, b )
            local _qualificationsA = tonumber(a[1].qualifications) or 0
            local _qualificationsB = tonumber(b[1].qualifications) or 0
            return _qualificationsA > _qualificationsB
        end)
        for key,value in pairs(_tab) do
            table.sort(value, function( a, b )
                local _idA = tonumber(a.id) or 0
                local _idB = tonumber(b.id) or 0
                return _idA < _idB
            end)
        end
        _pictureList[k] = _tab
    end

    self._listTb = _listTb
    self._pictureList = _pictureList

    self.seatNum = table.nums(self._pictureList)
    if self.seatNum < table.nums(self._openState) then
        self.seatNum = self.seatNum + 1
    end
end

function GeneralPreviewLayer:initUI()
    self:createSecondBg()

    self:createSubTab()
end

function GeneralPreviewLayer:createSecondBg() 
    --背景
    local subbg = zc.createNormalPop({
        srcTag = 1,
        width = 970,
        height = 520+31,
    })
    subbg:setPosition(zc.width*0.5, zc.height/2-21 )
    self:addChild(subbg)

    local _titleImg = zc.getSprite({
        src = "res/Main/common/popBg/titleBg2.jpg",
        ap = cc.p(1, 1),
        name = "titleImg",
    })
    _titleImg:setPosition(cc.p(subbg:getContentSize().width-3, subbg:getContentSize().height-3))
    subbg:addChild(_titleImg,2)

    local titleLeft = zc.createSprite("res/Main/common/popBg/titleBg2.jpg")
    titleLeft:setAnchorPoint(0, 1)
    titleLeft:setPosition(3, _titleImg:getPositionY())
    subbg:addChild(titleLeft,1)

    --标题
    local subbg_title = zc.createImageView({
        src = "res/Main/pub/manual/preview_title.png",
        pos = cc.p(0, subbg:getPositionY() + subbg:getContentSize().height/2),
        ap = cc.p(0,0.5)
    })
    self:addChild(subbg_title,10)

    local bgImg2 = zc.getScaleNode("res/Main/otherPlayer/otherPlayer_tag_bg.png", cc.size(154, 473))
    bgImg2:setAnchorPoint(cc.p(0, 0))
    bgImg2:setPosition(cc.p(1, 1))
    bgImg2:setName("bgImg2")
    subbg:addChild(bgImg2)
    local splitY = cc.Sprite:create( "res/Main/common/tab_split_y.png" )
    splitY:setPosition(154,473/2+16)
    bgImg2:addChild(splitY)

    self._subTabBg = subbg
end

function GeneralPreviewLayer:createSubTab() --二级tab
    local subTab = self._subTabBg:getChildByName("bgImg2")
    local subTabSize = subTab:getContentSize()

    local itemHeight = 130
    local innerHeight = itemHeight * self.seatNum + 10
    if innerHeight < subTabSize.height-28 then
        innerHeight = subTabSize.height-28
    end
    local ifBunce = self.seatNum > 2 and true or false
    local myscrollview = zc.createScrollview({
        pos = cc.p(0, 0),
        size = cc.size(subTabSize.width,subTabSize.height-28),
        innerSize = cc.size(subTabSize.width, innerHeight),
        dir = ccui.ScrollViewDir.vertical,
        bounceEnable = false,
    })
    subTab:addChild(myscrollview, 1)

   --tab
    local initPosY = zc.height/2 + 70
    local function getNormal(_tag)
        local scaleSize = cc.size(136, 136)
        local _normal = zc.getScaleNode("res/Main/common/img_empty.png", scaleSize)
        local _open = self:checkTabOpen(_tag)
        local _path = ""
        if _open then
            _path = "res/Main/pub/cardExtract/card_u_".._tag..".png"
        else
            _path = "res/Main/pub/cardExtract/card_c_".._tag..".png"
        end
        local _n = zc.createSprite(_path)
        _n:setScale(0.6)
        _n:setPosition(cc.p(_normal:getContentSize().width/2, _normal:getContentSize().height/2))
        _normal:addChild(_n)
        return _normal
    end
    self._subTab = {}

    for i = 1, self.seatNum do
        local labColor = cc.c3b(247, 188, 12)
        local _scroll = self._listTb[i]
        local tabBtn = zc.createPushButton({
            normalNode = getNormal( _scroll ),
            pos = cc.p(subTabSize.width/2, innerHeight-75-(i-1)*130),
            anchor = cc.p(0.5, 0.5),
            needEnableWhenMoving = true,
            needSwallow = false,
            endCallback = function() end,
        })
        tabBtn:setTag(_scroll)
        myscrollview:addChild(tabBtn)

        tabBtn:setTouchEndedCall(function()
            self:changeSubTab(tabBtn)
        end)
        if i == 1 then
            self:changeSubTab(tabBtn)
        end
        self._subTab[i] = tabBtn
    end
end
--检查是否开启
function GeneralPreviewLayer:checkTabOpen(_tag)
    return self._openState[tonumber(_tag)] or false
end

function GeneralPreviewLayer:initTabOpenState()
    local openState = {}
    local hero_handbook_card = gameData.getDataFromCache( "hero_handbook_card" )
    for k,v in pairs(hero_handbook_card) do
        local _isOpen = true
        local handBook = v or {}
        if tonumber(handBook.id) == 5 then
            _isOpen = zc.getFunctionStatus(45, false)
        else
            local _level = tonumber(handBook["level"]) or 0
            local plevel = tonumber(handBook["plevel"]) or 0
            if (self._houseLevel < _level) or (self._playerLevel < plevel) then
                _isOpen = false
            end
        end
        openState[tonumber(handBook.id)] = _isOpen
    end
    self._openState = openState
    self.hero_handbook_card = hero_handbook_card
end

function GeneralPreviewLayer:changeSubTab(_node) --切换二级tab
    local _tag = _node:getTag()
    local _open = self:checkTabOpen(_tag)
    if not _open then
        zc.tipInfo(zc.lang("tips_juanzhou_unopen"))
        return
    end

    local function getNormal(_tag)
        local scaleSize = cc.size(136, 136)
        local _normal = zc.getScaleNode("res/Main/common/img_empty.png", scaleSize)
        local _n = zc.createSprite("res/Main/pub/cardExtract/card_u_".._tag..".png")
        _n:setScale(0.6)
        _n:setPosition(cc.p(_normal:getContentSize().width/2, _normal:getContentSize().height/2))
        _normal:addChild(_n)
        return _normal
    end
    local function getSelected(_tag)
        local se1 = zc.createSprite("res/Main/research/qunxiong/select_down.png")
        local _s = zc.createSprite("res/Main/pub/cardExtract/card_d_".._tag..".png")
        _s:setScale(0.6)
        _s:setPosition(cc.p(se1:getContentSize().width/2, se1:getContentSize().height/2))
        se1:addChild(_s)
        return se1
    end

    if not self._selectedSubBtn or self._selectedSubBtn ~= _node then
        if not self._selectedSubBtn then
            self._selectedSubBtn = _node
            self._selectedSubBtn:setStateNormal(getSelected(_tag))
            self._selectedSubBtn:setLocalZOrder(1)
        else
            self._selectedSubBtn:setStateNormal(getNormal(self._subTag))
            self._selectedSubBtn:setLocalZOrder(0)
            self._selectedSubBtn = _node
            self._selectedSubBtn:setStateNormal(getSelected(_tag))
            self._selectedSubBtn:setLocalZOrder(1)
        end
        self._subTag = _tag -- 1同符合约 2势如破竹 3三国无双

        self._exchangeData = self._pictureList[_tag] or {}
        self:refreshTableInfo()
    end
end

-- 右边的table
--[[
    self._exchangeData是所有的卡片信息
    self._exchangeData[i]是某个项中的卡片信息
]]
function GeneralPreviewLayer:refreshTableInfo()

    if self._mytable then
        self._mytable:reloadData()
        return
    end

    local tableSize = cc.size(807, 467)

    local myTable = zc.ZCTableView:create( tableSize )
    myTable:setPosition(157, 1)
    myTable:setInertia(true) --设置惯性
    myTable:setBounceable(true)
    myTable:setDirection(zctech.SCROLLVIEW_DIRECTION_VERTICAL) --设置横向纵向
    myTable:setDelegate()
    myTable:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    
    self._subTabBg:addChild(myTable)

    local cellSize = cc.size(807, 210)
    local __h, __w = 0, 0
    local function cellSizeForTable(table,idx)
        local _idx = idx + 1
        local _quelityCount = #self._exchangeData[_idx]
        local _verticalCount = math.ceil( _quelityCount / 4 )
        local _cellSize = cc.size(cellSize.width, cellSize.height * _verticalCount + 30)
        if _verticalCount > 1 then
            return _cellSize.height - 10 * (_verticalCount - 1), _cellSize.width
        else
            return _cellSize.height, _cellSize.width
        end
    end

    local function numberOfCellsInTableView(table)
        return #self._exchangeData
    end
    
    local function tableCellAtIndex(table,idx)
        local cell = table:dequeueCell()
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
        end
        __h, __w = cellSizeForTable(table, idx)
        cell:setContentSize(__w, __h)
        cell:removeAllChildren()
        self:createCell(cell, idx)
        return cell
    end
    myTable:registerScriptHandler(numberOfCellsInTableView,cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    myTable:registerScriptHandler(cellSizeForTable,cc.TABLECELL_SIZE_FOR_INDEX)
    myTable:registerScriptHandler(tableCellAtIndex,cc.TABLECELL_SIZE_AT_INDEX)
    
    myTable:reloadData()
    self._mytable = myTable
end

function GeneralPreviewLayer:createCell(cell, nowIdx)
    local  _nowIndex = nowIdx + 1
    local cellSize = cell:getContentSize()
    local _allDatas = self._exchangeData[_nowIndex]
    local _quelityNum = #_allDatas
    local _verticalNum = math.ceil( _quelityNum / 4)
    
    
     -- 将领等级
    local _levelIconPath 
    local _tagLevel = _allDatas[1].qualifications
    local heroLevelPath = "res/Main/pub/manual/hero_S.png"
    if _tagLevel == 6 then
        heroLevelPath = "res/Main/pub/manual/hero_S.png"
        _levelIconPath = "res/Main/common/qualifications/s_s.png"
    elseif _tagLevel == 4 then
        heroLevelPath = "res/Main/pub/manual/hero_A.png"
        _levelIconPath = "res/Main/common/qualifications/a_s.png"
    elseif _tagLevel == 2 then
        heroLevelPath = "res/Main/pub/manual/hero_B.png"
    elseif _tagLevel == 1 then
        heroLevelPath = "res/Main/pub/manual/hero_C.png"
    end
    local cellSize = cell:getContentSize()
    
    local _heroLevelSprite = zc.getSprite({
        src = heroLevelPath,
        ap = cc.p(0, 0.5),
        pos = cc.p(40, cellSize.height -2 ) 
        })
    _heroLevelSprite:setPositionY( cellSize.height - _heroLevelSprite:getContentSize().height / 2)
    cell:addChild(_heroLevelSprite)
    -- 获取抽取得次数，如果为空，或者nil则不显示
    local times = self.hero_handbook_card[tostring(self._subTag)]["pinzhi" .. _tagLevel]
    if times and string.len(tostring(times)) ~= 0 then
        local zhaomuTipBg = zc.getScaleNode("res/Main/role/trinket/attrDi.png", cc.size(0, 0))
        zhaomuTipBg:setAnchorPoint(cc.p(0, 0.5))
        zhaomuTipBg:setPosition( _heroLevelSprite:getPositionX() + _heroLevelSprite:getContentSize().width + 10, _heroLevelSprite:getPositionY() - 5)
        cell:addChild(zhaomuTipBg)

        local iconModel = zc.getSprite({
            src = _levelIconPath,
        })
        local modelSize = iconModel:getContentSize()
        local discribeLabel = zc.createRichLabel({
            text = string.format(zc.lang("tip_zhaomu_discribe"), tostring(times), _levelIconPath, modelSize.width * 0.35, modelSize.height * 0.35),
            pos = cc.p(10, 0),
            fontSize = 20,
            anchor = cc.p(0, 0),
            color = cc.c3b(141, 124, 96),
            })
        zhaomuTipBg:setContentSize(cc.size(discribeLabel:getContentSize().width + 20,30))
        zhaomuTipBg:addChild(discribeLabel)
    end
    
    

    for j = 1, _verticalNum do
        for i = 1, 4 do
            local _idx = (j-1) * 4 + i
            local _data = _allDatas[_idx]
            if not _data then
                break
            end

            local imgPath = "res/Main/pub/manual/preview_cellBg_" .. self._qualificationsToColor[tonumber(_data.qualifications)] .. ".png"
            if not zc.checkFileExist(imgPath) then
                imgPath = "res/Main/pub/manual/preview_cellBg_green.png"
            end
            local cellImg = zc.createSprite(imgPath)
            local cellImgSize = cellImg:getContentSize()

            local x = 113 + (i-1) * 194
            local y = cellImgSize.height / 2 + (cellImgSize.height + 5) * (_verticalNum - j)
            cellImg:setPosition(x, y)
            cell:addChild(cellImg)
            
            local nameBg = zc.getScaleNode("res/Main/role/trinket/attrDi.png", cc.size(107, 27))
            nameBg:setAnchorPoint(cc.p(0.5, 1))
            nameBg:setPosition( cellImgSize.width/2, cellImgSize.height - 11 )
            cellImg:addChild(nameBg)

            local nameLab = zc.createLabel({
                fontSize = 18,
                pos = cc.p(nameBg:getContentSize().width/2, nameBg:getContentSize().height/2),
            })
            nameBg:addChild(nameLab)

            local titleBg = zc.getScaleNode("res/Main/role/common/shadow_bg.png", cc.size(136, 33))
            titleBg:setAnchorPoint(0.5, 0.5)
            titleBg:setPosition(cellImgSize.width/2, 38 ) 
            cellImg:addChild(titleBg)
        
            local _titleLab = zc.createRichLabel({
                fontSize = 20,
            })
            _titleLab:setAnchorPoint(0.5, 0.5)
            _titleLab:setPosition( titleBg:getContentSize().width/2, titleBg:getContentSize().height/2 )  
            _titleLab:setGradientColor(cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1))
            titleBg:addChild(_titleLab)

            local cardBtn, _info = zc.createIconByData({
                data = {
                    mainType = _data.itemType,
                    type = _data.itemId, 
                    id = _data.itemId,
                },
                showQuality = true,
                pos = cc.p(cellImgSize.width/2, cellImgSize.height/2+13),
                listener = function()
                    zc.createHeroInfoPop({heroId = _data.itemId, showHero = true})
                end,
            })
            cellImg:addChild(cardBtn)
            cardBtn:setScale(80/120)


            nameLab:setString(_info.name)
            _titleLab:setString(_data.title)
        end
    end 
end

function GeneralPreviewLayer:onCleanup()
    if self._callback and type(self._callback) == "function" then
        self._callback()
    end
end

return GeneralPreviewLayer
