-- FileName: RoleBreakthroughLayer.lua
-- Author: wangming
-- Date: 2016-09-29
-- Purpose: 征战界面
--[[TODO List]]
local RoleBreakthroughLayer = class("RoleBreakthroughLayer", function()
	return zc.createNormalDialog({
		src = "",
		opacity = 0,
		showFlower = false,
		showBack = false,
	})
end)

function RoleBreakthroughLayer:create( params )
	return RoleBreakthroughLayer.new( params )
end

function RoleBreakthroughLayer:ctor( params )
	self._roleLayer = params.roleLayer
	self._checkSelected = false
	self:setSwallowTouches(false)

	self:initBase(params.id)
	self:initUI()
	self:selfFresh(params.id, true)
end

function RoleBreakthroughLayer:onEnter()
	self:addGuide()
end

function RoleBreakthroughLayer:initBase( id )
	if id then
		self._id = id
	end
    self._heroData = PlayerData:getGeneralInfoById(self._id)
	self._newAdvance = tonumber(self._heroData.newAdvance) or 0
	self._isMain = tonumber(PlayerData.info.heroId) == self._id
	if id then
		self._effDbs = gameData.getDataFromCache("hero_stage", {heroId = self._heroData.heroId})
		if #self._effDbs < 0 and next(self._effDbs) ~= nil then
			self._effDbs = {self._effDbs}
		end
	end
end

function RoleBreakthroughLayer:selfFresh( id, isCtor )
	-----数据
	if not isCtor then
		self:initBase(id)
	end
	local _dbData = zc.getBreakThroughInfo({
		id = id,
		heroId = self._heroData.heroId,
		stageLevel = self._newAdvance + 1,
		qualifications = self._heroData.qualifications
	})
	self._dbData = _dbData

	local _costData = {}
	if next(_dbData) ~= nil then
		self._breakthroughBtn:setEnable(true)
		if self._isMain then
			self._checkBox:setVisible(false)
		else
			self._checkBox:setVisible(true)
		end

		for i=1, 3 do
			if string.len(_dbData["needStuff" .. i] or "") > 0 then
				local _info = string.split(_dbData["needStuff" .. i], "|")
				table.insert(_costData, _info)
			end
		end
		local _sameCardNum = tonumber(_dbData.sameCardNum) or 0
		if _sameCardNum > 0 then
			self._spCost1 = {
				zc.mainTypeSoldierFragment,
				self._heroData.heroId,
				_sameCardNum,
			} 
		else
			self._spCost1 = nil
		end
		if _dbData.generalCardNum and string.len(_dbData.generalCardNum) > 0 then
			local _info = string.split(_dbData.generalCardNum, "|")
			local _num = tonumber(_info[3]) or 0
			if _num > 0 then
				self._spCost2 = _info
			else
				self._spCost2 = nil
			end
		else
			self._spCost2 = nil
		end
		if not self._spCost2 or not self._spCost1 then
			self._checkBox:setVisible(false)
		else
			self._checkBox:setVisible(true)
		end
	else
		self._spCost1 = nil
		self._spCost2 = nil
		self._breakthroughBtn:setEnable(false)
		self._checkBox:setVisible(false)
	end

	self._costData = _costData
	self:freshCosts()
	self:freshIcons()
end

function RoleBreakthroughLayer:freshCosts()
	for i=1, 4 do
		self._costDis[i]:removeAllChildren()
		self._costDis[i]:setVisible(false)
	end

	if #self._costData > 0 then
		for i=1, #self._costData do
			local _size = self._costDis[i]:getContentSize()
			local v = self._costData[i]
			local _num = tonumber(v[3]) or 0
			local _icon = zc.createIconByData({
				data = {
					mainType = v[1],
	                type = v[2],
	                id = v[2],
	                num = _num,
				},
				showCount = true,
				pos = cc.p(_size.width*0.5, _size.height*0.5),
			})
			_icon:setScale(0.6)
			_icon:setTouchEndedShowTip()
			self._costDis[i]:addChild(_icon)
			self._costDis[i]:setVisible(true)
			local _lab = _icon:getChildByName("countLabel")
			if _lab then
				local _flag = true
				local _cost = tonumber(zc.getNumByTypeId(v[1], v[2])) or 1
				_lab:setColor(_cost >= _num and display.COLOR_WHITE or display.COLOR_RED)
			end
		end
	end
	self:freshSpCosts()
end

function RoleBreakthroughLayer:freshSpCosts()
	local _idx = #self._costData + 1
	self._costDis[_idx]:removeAllChildren()
	self._costDis[_idx]:setVisible(false)
	if self._isMain then
		return
	end
	if not self._dbData then
		return
	end
	local v = self._checkSelected and self._spCost2 or self._spCost1
	if not v then
		return
	end
	local _db = gameData.getDataFromCacheWithPrimaryKey("item", self._spCost2[2])
	self._checkLab:setString(zc.lang("use") .. tostring(_db.name))
	local _size = self._costDis[_idx]:getContentSize()
	local _icon = zc.createIconByData({
		data = {
			mainType = v[1],
            type = v[2],
            id = v[2],
            num = v[3],
		},
		showCount = true,
		pos = cc.p(_size.width*0.5, _size.height*0.5),
	})
	_icon:setScale(0.6)
	_icon:setTouchEndedShowTip()
	self._costDis[_idx]:addChild(_icon)
	self._costDis[_idx]:setVisible(true)
	local _lab = _icon:getChildByName("countLabel")
	if _lab then
		local _flag = true
		if self._checkSelected then
			local _info = PlayerData:getBagInfoByID(v[2]) or {}
			local _fragmentNum = tonumber(_info.num) or 0
			local _costNum = tonumber(v[3]) or 1
			if _fragmentNum < _costNum then
				_flag = false
			end
		else
			local _info = PlayerData:getFragmentInfoByHeroId(self._heroData.heroId) or {}
			local _fragmentNum = tonumber(_info.num) or 0
			local _costNum = tonumber(v[3]) or 1
			if _fragmentNum < _costNum then
				_flag = false
			end
		end
		_lab:setColor(_flag and display.COLOR_WHITE or display.COLOR_RED)
	end

end

function RoleBreakthroughLayer:freshIcons()
	local _quality = tonumber(self._heroData.quality) or 1	
	local _limitFlag = false
	for i=1, 2 do
		local _di = self._iconDis[i]
		_di:removeAllChildren()
		local _size = _di:getContentSize()
		local _tmpAdvance =  self._newAdvance + i - 1
		local _text = zc.lang("breakthrough") .. " +" .. _tmpAdvance
		if i == 2 then
			_quality = self._dbData.quality or _quality
			if next(self._dbData) == nil then
				_text = zc.lang("already_achieved_limit")
				_limitFlag = true
			end
		end
		_di._label:setString(_text)
		local _icon = zc.createIconByData({
			data = {
				mainType = zc.mainTypeSoldier,
	            type = self._heroData.heroId,
	            id = self._heroData.heroId,
			},
			quality = _quality,
			pos = cc.p(_size.width*0.5, _size.height*0.5),
		})
		_di:addChild(_icon)
	end
	self._costLimit:setVisible(_limitFlag)
	self._costTip:setVisible(not _limitFlag)
	self._breakthroughBtn:setVisible(not _limitFlag)

	self._scrollView:removeAllChildren()
	local _node = cc.Node:create()
    self._scrollView:addChild(_node)
    local _tmpSize = self._scrollView:getContentSize()
    local _partY = 14
    local _startY = -_partY
    if #self._effDbs > 0 then
	    for i=1, #self._effDbs do
	    	local _db = self._effDbs[i]
	    	local _tipLabel = zc.createRichLabel({
				text = zc.lang("breakthrough") .. tostring(_db.stage) .. ":",
				fontSize = 24,
				anchor = cc.p(0.5, 1),
				pos = cc.p(115, _startY),
			})
			_tipLabel:setGradientColor(cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1))
			_node:addChild(_tipLabel)

			local _showText = ""
			local _skillId = tonumber(_db.skill) or 0
			local _skillDb = nil
			if _skillId ~= 0 then
				_skillDb = gameData.getDataFromCacheWithPrimaryKey("skill", _skillId)
			end
			if _skillDb and next(_skillDb) ~= nil then
				_showText = zc.lang("new") .. zc.lang("skill") .. "：" .. _skillDb.name
			else
				for j=1, 3 do
					local _effId = tonumber(_db["effectId" .. j]) or 0
					if _effId ~= 0 then
						if j ~= 1 then
							_showText = _showText .. "\n"
						end
						local _target = tonumber(_db["effectTarget" .. j]) or 0
						if _target == 0 then
							_showText = _showText .. zc.lang("all_heros")
						end
						local _num = tonumber(_db["value" .. j]) or 0
						_num = zc.getAttributeNumById(_effId, _num)
						_showText = _showText .. zc.getEquipAttriNameByEffectType(_effId, true ) .. "+" .. _num
					end
				end
			end
			local _color = display.COLOR_GRAY
			if i <= self._newAdvance then
				_color = display.COLOR_WHITE
			elseif i == self._newAdvance + 1 then
				_color = display.COLOR_GREEN
			end
			local _infoLab = zc.createLabel({
				text = _showText,
				fontSize = 24,
				ap = cc.p(0.5, 1),
				width = 260,
				pos = cc.p(310, _startY),
				color = _color,
			})
			_infoLab:setHorizontalAlignment(cc.TEXT_ALIGNMENT_CENTER)
			_node:addChild(_infoLab)
			local _tmpY = _infoLab:getContentSize().height
			if _tmpY < _tipLabel:getContentSize().height then
				_tmpY = _tipLabel:getContentSize().height
			end
			if i%2 == 0 then
				local _tmpDi = zc.createSprite("res/Main/role/breakthrough/cellDi.png")
				_tmpDi:setAnchorPoint(0.5, 1)
				_tmpDi:setPosition(_tmpSize.width*0.5, _startY + 4)
				local _h = _tmpDi:getContentSize().height
				_tmpDi:setScaleY((_tmpY + 8)/_h)
				_node:addChild(_tmpDi, -1)
			end
			_startY = _startY - _tmpY - _partY
	    end
	    local _totleH = math.abs(_startY)
	    if _totleH < _tmpSize.height then
	    	_totleH = _tmpSize.height
	    end
	    _node:setPositionY(_totleH)
	    _tmpSize.height = _totleH
	    self._scrollView:setInnerContainerSize(_tmpSize)
	end
end

function RoleBreakthroughLayer:initUI()
	local _tmpSize = cc.size(0, 0)
    --底部
    local _bottomDiImg = zc.getSprite({
    	src = "res/Main/levy/bg_down.jpg",
    	pos = cc.p(zc.width*0.5, 25),
    })
    self:addChild(_bottomDiImg, 2)
    local _bottomDiSize = cc.size(zc.width, _bottomDiImg:getContentSize().height*0.5 + _bottomDiImg:getPositionY())
    local _bottomDi = cc.Node:create()
    self:addChild(_bottomDi, 2)

    local _totleBtn = zc.createPushButton({
        normalFile = "res/Main/role/breakthrough/totleBtn.png",
        pos = cc.p(120, 110),
        endCallback = function()
			self._roleLayer:goBreakthroughLayer(false)
        end,
    })
    self:addChild(_totleBtn, 2)

    _tmpSize.width = 180
    _tmpSize.height = 45
    local _normalNode = zc.getScaleNode("res/Main/common/checkBar_u.png", _tmpSize)
    local _selectedNode = zc.getScaleNode("res/Main/common/checkBar_d.png", _tmpSize)
    local _checkBox = zc.createPushButton({
		normalNode = _normalNode,
		selectedNode = _selectedNode,
		pos = cc.p(290, _bottomDiSize.height*0.5),
		endCallback = function()
			self._checkSelected = not self._checkSelected
			self._check:setVisible(self._checkSelected)
			self:freshSpCosts()
		end,
	})
	_bottomDi:addChild(_checkBox)
	self._checkBox = _checkBox

    local _check = zc.createSprite("res/Main/common/check.png")
    _check:setPosition(26, _tmpSize.height*0.5)
    _checkBox:addChild(_check)
    _check:setVisible(false)
	self._check = _check

	local _checkLab = zc.createLabel({
		fontSize = 20,
		ap = cc.p(0, 0.5),
		pos = cc.p(50, _tmpSize.height*0.5),
	}) 
	_checkBox:addChild(_checkLab)
	self._checkLab = _checkLab


    local _costTip = zc.getSprite({
    	src = "res/Main/role/breakthrough/costTips.png",
    	pos = cc.p(_bottomDiSize.width*0.5 - 150, _bottomDiSize.height*0.5)
	})
	_bottomDi:addChild(_costTip)
	self._costTip = _costTip
	self._costDis = {}
	local _startX = _costTip:getPositionX() + _costTip:getContentSize().width + 5
	_tmpSize.width = 80
	_tmpSize.height = 80
	for i=1, 4 do
		local _infoNode = zc.getScaleNode("res/Main/role/trinket/attrDi.png", _tmpSize)
		_infoNode:setPosition(_startX + (i-0.5)*(_tmpSize.width + 5), _bottomDiSize.height*0.5)
		_bottomDi:addChild(_infoNode)
		self._costDis[i] = _infoNode
	end

    local _breakthroughBtn = zc.createCommomBtn({
		tag = 1,
		text = zc.lang("breakthrough_2"),
		scale = 1.2,
		pos = cc.p(_bottomDiSize.width - 180, _bottomDiSize.height*0.5 - 5),
		endCallback = function()
        	local _currentGID = getGuideManager():getCurrentGID()
        	if _currentGID == GuideKey.KEY_14306 then
	        	getGuideManager():setFuncGuideOver()
	        end
			local _params = {heroId = self._heroData.id}
			local _flag = self._checkSelected and 2 or 1
			if self._isMain or
			  (not self._spCost1 and not self._spCost2) then
				_flag = 0
			elseif not self._spCost1 then
				_flag = 2
			elseif not self._spCost2 then
				_flag = 1
			end
			_params.type = _flag
			if _flag == 1 and self._spCost1 then
				local _info = PlayerData:getFragmentInfoByHeroId(self._heroData.heroId) or {}
				local _fragmentNum = tonumber(_info.num) or 0
				local _costNum = tonumber(self._spCost1[3]) or 1
				if _fragmentNum < _costNum then
					zc.tipInfo(zc.lang("tips_hero_fragment_unenough"))
					return
				end
				_params.id = _info.id
			elseif _flag == 2 and self._spCost2 then
				local _itemId = tonumber(self._spCost2[2]) or 0
				local _info = PlayerData:getBagInfoByID(_itemId) or {}
				local _dbInfo = gameData.getDataFromCacheWithPrimaryKey("item", _itemId)
				local _fragmentNum = tonumber(_info.num) or 0
				local _costNum = tonumber(self._spCost2[3]) or 1
				if _fragmentNum < _costNum then
					zc.tipInfo(tostring(_dbInfo.name) .. LANGUAGE_ROLE.not_enough .."！")
					return
				end
				_params.id = _info.id
			end
			zc.sendHttp({
				apiName = "Hero_advance",
				params = _params,
				successCallback = function( backData )
					local _nowInfo = clone(self._heroData)
					PlayerData:updataData(backData)
					PlayerData:updateGeneralInfo(backData.heroInfo)
					self:selfFresh()
					self._roleLayer:refreshInfo()
					local _data = {
						isBreakthrough = true,
						mainInfo = self._heroData,
						nowHeroInfo = _nowInfo,
						effInfo = self._effDbs[self._newAdvance],
					}
					zc.createSourcePop("TalentChange", "src/layer/talent/TalentChangePop.lua", _data)
				end
			})
		end
	})
	_bottomDi:addChild(_breakthroughBtn)
	self._breakthroughBtn = _breakthroughBtn

    local _costLimit = zc.getSprite({
    	src = "res/Main/role/breakthrough/upToLimit.png",
    	pos = cc.p(_breakthroughBtn:getPositionX(), _breakthroughBtn:getPositionY())
	})
	_bottomDi:addChild(_costLimit)
	self._costLimit = _costLimit
	_costLimit:setVisible(false)

	--right
	local _rightSize = cc.size(450, 320)
	local _tmpH = 10
    local _rightBg = zc.getScaleNode("res/Main/ornaments/general_tips_s_bg.png", _rightSize)
    _rightBg:setPosition(zc.width - 260, _bottomDiSize.height - _tmpH)
    _rightBg:setAnchorPoint(0.5, 0)
    self:addChild(_rightBg)

    local _arrowY = _rightSize.height + 90
	local _arrow = zc.createSprite("res/Main/role/equip/arrow_2.png")
    _arrow:setPosition(_rightSize.width*0.5, _arrowY)
    _rightBg:addChild(_arrow)

	_tmpSize.width = 120
	_tmpSize.height = 33
	self._iconDis = {}
    for i=1, 2 do
    	local _di = zc.createSprite("res/Main/role/role_magic_bg.png")
		_di:setPosition(_rightSize.width*0.5 - 100 + (i-1)*200, _arrowY)
    	_rightBg:addChild(_di)
    	_di:setScale(0.7)
    	self._iconDis[i] = _di

		local _nameNode = zc.getScaleNode("res/Main/role/trinket/attrDi.png", _tmpSize)
		_nameNode:setPosition(_di:getPositionX(), _arrowY - 75)
		_rightBg:addChild(_nameNode)

		local _label = zc.createRichLabel({
			text = "",
			fontSize = 20,
			pos = cc.p(_tmpSize.width*0.5, _tmpSize.height*0.5),
		})
		_label:setGradientColor(cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1))
		_nameNode:addChild(_label)
		_di._label = _label
    end

    _tmpSize.width = _rightSize.width
    _tmpSize.height = _rightSize.height - _tmpH*2

    local _scrollView = zc.createScrollview({
        dir = ccui.ScrollViewDir.vertical,
        size = _tmpSize,
        pos = cc.p(0, _tmpH),
    }) 
    _rightBg:addChild(_scrollView)
    self._scrollView = _scrollView
end

function RoleBreakthroughLayer:addGuide()
	local _currentGID = getGuideManager():getCurrentGID()
	if _currentGID == GuideKey.KEY_14304 then
		zc.setGuideFuncNextStep(_currentGID, false)
        getGuideManager():setHeroFuncCallBack(function()
        	getGuideManager():setObjTarget(self._breakthroughBtn, 0.8)
            getGuideManager():startGuide()
        end)
        getGuideManager():startGuide()
	end
end

return RoleBreakthroughLayer
