--[[
    培养
    唐实聪
    2016.3.25
    2016.8.20重做
]]
local RoleCultureLayer = class("RoleCultureLayer", function()
    local layer = cc.Sprite:create()
    layer:setContentSize( 629, 539 )
    return layer
end)

function RoleCultureLayer:create( params )
    return RoleCultureLayer.new( params )
end

function RoleCultureLayer:ctor( params )

    --显示武、智、体
    local _tips = {
        {path="res/Main/role/culture/tip_wu.png",pos=cc.p(-200, 431)},
        {path="res/Main/role/culture/tip_zhi.png",pos=cc.p(20, 277)},
        {path="res/Main/role/culture/tip_ti.png",pos=cc.p(-140, 137)},
    }
    for i = 1, 3 do
        local tipSp = zc.createSprite(_tips[i].path)
        tipSp:setAnchorPoint(1, 0.5)
        tipSp:setPosition(_tips[i].pos)
        self:addChild(tipSp)
    end

    self:initData( params )
    self:initUI()
    self:refreshHero( params )

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "cleanup" == event then
            self:onCleanup()
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

function RoleCultureLayer:onEnter()
    -- 顶部信息栏
    LayerManager.getTopInfoLayer():setPopInfo( "RoleCultureLayer" )
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_11302 then
        performWithDelay(self, function()
            getGuideManager():setObjTarget(self._cultureBtn)
            getGuideManager():startGuide()
        end, 0.01)
    end
end

function RoleCultureLayer:onCleanup()
    self._exist = false
end

function RoleCultureLayer:initData( params )
    self._armatureX = 232
    self._armatureY = 64

    self._cultureConfig = LANGUAGE_culture_tips()

    self._attrConfig = {
        [1] = {
            name = zc.lang("wuli"),
            key = {
                "force",
                "forceLimit",
            },
        },
        [2] = {
            name = zc.lang("wisdom"),
            key = {
                "wisdom",
                "wisdomLimit",
            },
        },
        [3] = {
            name = zc.lang("tili"),
            key = {
                "soma",
                "somaLimit",
            },
        },
    }

    -- 消耗信息
    self._consumeList = gameData.getDataFromCache( "hero_culture" )

    self._choiceIndex = tonumber( params.index ) or 1

    local _size = self:getContentSize()
    self._width = _size.width
    self._height = _size.height

    self._exist = true
end

function RoleCultureLayer:initUI()
    -- 背景
    local _bg = cc.Sprite:create( "res/Main/role/common/role_bg.png" )
    _bg:setPosition( self._width/2, self._height/2 )
    self:addChild( _bg )

    local _middleX = 307

    -- 武将阴影层
    local _shadowHeroWidth = 567
    local _shadowHeroHeight = 320
    local _shadowHero = ccui.Scale9Sprite:create( cc.rect( 11, 10, 30, 30 ), "res/Main/role/common/shadow_bg.png" )
    _shadowHero:setContentSize( _shadowHeroWidth, _shadowHeroHeight )
    _shadowHero:setAnchorPoint( 0.5, 0 )
    _shadowHero:setPosition( _middleX, self._height - 373 )
    _bg:addChild( _shadowHero )
    self._shadowHero = _shadowHero

    -- 顶部
    local _topBg = cc.Sprite:create( "res/Main/role/common/top_bg.jpg" )
    _topBg:setAnchorPoint( 0.5, 1 )
    local _topBgPos = cc.p( _middleX, self._height - 5 )
    _topBgPos = _bg:convertToWorldSpace( _topBgPos )
    _topBgPos = _shadowHero:convertToNodeSpace( _topBgPos )
    _topBg:setPosition( _topBgPos )
    _shadowHero:addChild( _topBg )
    local _topBgSize = _topBg:getContentSize()
    local _topBgWidth = _topBgSize.width
    local _topBgHeight = _topBgSize.height
    _topBg:setScaleX( 594/_topBgWidth )

    -- 名字
    local _nameBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
    _nameBg:setContentSize( 408, 30 )
    _nameBg:setPosition( _topBgWidth/2, _topBgHeight/2 )
    _topBg:addChild( _nameBg )
    local _name = zc.createRichLabel({
        fontSize = 23,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 245, 163 ),
    })
    _name:setPosition( _topBgWidth/2, _topBgHeight/2 )
    _topBg:addChild( _name )
    self._name = _name

    -- 武器图标
    local _weaponIconBg = cc.Sprite:create( "res/Main/role/common/weapon_icon_bg.png" )
    _weaponIconBg:setAnchorPoint( 0, 0 )
    _weaponIconBg:setPosition( 0, 0 )
    _topBg:addChild( _weaponIconBg )
    local _weaponIcon = cc.Sprite:create()
    _weaponIcon:setPosition( _weaponIconBg:getContentSize().width/2 + 2, -10 )
    _weaponIconBg:addChild( _weaponIcon )
    self._weaponIcon = _weaponIcon

    -- 称号
    local _selectTitle = zc.createPushButton({
        normalFile = "res/Main/role/attribute/title_u.png",
        selectedFile = "res/Main/role/attribute/title_d.png",
        touchSize = cc.size(100,100),
        needSwallow = false,
        touchScale = 0.9,
        pos = cc.p( 390, _nameBg:getContentSize().height/2 ),
        endCallback = function()
            zc.sendHttp({
                apiName = "Chenghao_get",
                successCallback = function (net_data)
                    local function refresh()
                        local _nameText = zc.getLevelTipShow()..self._heroData.level.." "..PlayerData.info.userName
                        local _newAdvance = tonumber(self._heroData.newAdvance) or 0
                        if _newAdvance > 0 then
                            _nameText = _nameText .. "+" .. _newAdvance
                        end
                        self._name:setString( _nameText.."  [color=ffe400ff]< "..zc.getGodNameById( PlayerData:getBaseInfoByTag( "godName" ) ).." >[/color]" )
                    end
                    net_data.callback = refresh
                    zc.createSourcePop("RoleTitlePop", "src/layer/role/RoleTitlePop.lua", net_data)
                end
            })
        end,
    })
    _nameBg:addChild( _selectTitle )
    self._selectTitle = _selectTitle
    
    -- 光
    local _spine = zc.getSpine( "jsxxyxg" )
    _spine:setPosition( self._armatureX, self._armatureY + 105 )
    _spine:playAction( "animation", true )
    _shadowHero:addChild( _spine )

    -- 武将
    local _heroBtn = zc.createPushButton({
        touchSize = cc.size( 200, 210 ),
        pos = cc.p( self._armatureX, self._armatureY + 105 ),
        endCallback = function()
            local _action = zc.random(1, 3)
            if self._heroAction and self._heroAction == _action then
                if _action < 3 then
                    _action = _action + 1
                else 
                   _action = 1 
                end
            end
            self._heroAction = _action
            local _actions = {"atk3", "atk", "atk0"}
            if self._heroArmature then
                self._heroArmature:getAnimation():play( _actions[_action], -1, 0 )
            end
        end
    })
    _shadowHero:addChild( _heroBtn )
    
    -- 战斗力
    local _fightBg = cc.Sprite:create( "res/Main/role/common/fight_bg.png" )
    _fightBg:setAnchorPoint( 0.5, 0 )
    _fightBg:setPosition( self._armatureX, 0 )
    _shadowHero:addChild( _fightBg, 2 )
    local _fightNum = zc.createFntLabel({
        fnt = "res/Main/common/fnt/fight_change.fnt",
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 90, _fightBg:getContentSize().height/2 ),
        addwidth = -8,
    })
    _fightNum:setScale( 0.7 )
    _fightBg:addChild( _fightNum )
    self._fightNum = _fightNum

    -- 阵营
    local _camp = cc.Sprite:create()
    _camp:setAnchorPoint( 0, 0 )
    _camp:setPosition( 0, 0 )
    _shadowHero:addChild( _camp )
    self._camp = _camp

    -- 选择
    local _choiceTitle = cc.Sprite:create( "res/Main/role/culture/choice_title.png" )
    _choiceTitle:setPosition( _shadowHeroWidth - 104, 287 )
    _shadowHero:addChild( _choiceTitle )
    local _choiceBg = cc.Sprite:create( "res/Main/role/culture/choice_bg.png" )
    _choiceBg:setPosition( _choiceTitle:getPositionX(), _choiceTitle:getPositionY()/2 )
    _shadowHero:addChild( _choiceBg )
    local _choiceBgSize = _choiceBg:getContentSize()
    local _choiceBgWidth = _choiceBgSize.width
    local _choiceBgHeight = _choiceBgSize.height
    -- 培养方式
    local _myVip = tonumber( PlayerData:getBaseInfoByTag( "vip" ) ) or 0
    self._choice = {}
    for i, v in ipairs( self._cultureConfig ) do
        local _choice = nil
        local _choicePosY = ( _choiceBgHeight - 40 )/8*( 9 - 2*i ) + 20
        local _needVip = tonumber( self._consumeList[tostring( i )].needVip ) or 0
        if _myVip >= _needVip then
            _choice = cc.Sprite:create( "res/Main/role/culture/unlock.png" )
            _choice:setPosition( _choiceBgWidth/2, _choicePosY )
            _choiceBg:addChild( _choice )
            self._choice[i] = _choice
            local _choiceName = zc.createLabel({
                text = v,
                color = display.COLOR_WHITE,
                color2 = cc.c3b( 72, 37, 0 ),
                outLine = true,
                pixel = 1,
                fontSize = 22,
                pos = cc.p( 90, _choice:getContentSize().height/2 + 3 ),
            })
            _choice:addChild( _choiceName )
            _choice._choiceName = _choiceName
            local _choiceBtn = zc.createPushButton{
                touchSize = _choice:getContentSize(),
                pos = cc.p( _choice:getContentSize().width/2, _choice:getContentSize().height/2 ),
                endCallback = function() end,
            }
            _choice:addChild( _choiceBtn )
            _choice._choiceBtn = _choiceBtn
            _choiceBtn:setTouchEndedCall(function()
                if self._choiceIndex ~= i then
                    self._choiceIndex = i
                    self._selected:setPositionY( self._choice[self._choiceIndex]:getPositionY() + 5 )
                    self:refreshConsume()
                end
            end)
        else
            _choice = cc.Sprite:create( "res/Main/role/culture/lock.png" )
            _choice:setPosition( _choiceBgWidth/2, _choicePosY )
            _choiceBg:addChild( _choice )
            self._choice[i] = _choice
            local _choiceName = zc.createLabel({
                text = v,
                color = cc.c3b( 122, 42, 0 ),
                color2 = cc.c3b( 255, 232, 128 ),
                outLine = true,
                pixel = 0.8,
                fontSize = 22,
                pos = cc.p( 90, _choice:getContentSize().height/2 + 3 ),
            })
            _choice:addChild( _choiceName )
            _choice._choiceName = _choiceName
            local _choiceBtn = zc.createPushButton{
                touchSize = _choice:getContentSize(),
                pos = cc.p( _choice:getContentSize().width/2, _choice:getContentSize().height/2 ),
                endCallback = function() end,
            }
            _choice:addChild( _choiceBtn )
            _choice._choiceBtn = _choiceBtn
            _choiceBtn:setTouchEndedCall(function()
                zc.tipInfo( zc.lang("vip").._needVip.. zc.lang("kaiqi") ..v )
            end)
        end
    end
    local _selected = cc.Sprite:create( "res/Main/common/check.png" )
    _selected:setPosition( 50, self._choice[self._choiceIndex]:getPositionY() + 5 )
    _choiceBg:addChild( _selected )
    self._selected = _selected

    -- 培养
    local _cultureBgWidth = 594
    self._cultureBgWidth = _cultureBgWidth
    local _cultureBgHeight = 124
    local _cultureBg = ccui.Scale9Sprite:create( cc.rect( 20, 10, 549, 71 ), "res/Main/role/trinket/sellDi.png" )
    _cultureBg:setContentSize( _cultureBgWidth, _cultureBgHeight )
    _cultureBg:setAnchorPoint( 0.5, 0 )
    _cultureBg:setPosition( _middleX, 35 )
    _bg:addChild( _cultureBg )
    self._attr = {}
    for i, v in ipairs( self._attrConfig ) do
        local _attrBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
        _attrBg:setContentSize( 220, 30 )
        _attrBg:setPosition( 125, _cultureBgHeight/2 + 70 - 35*i )
        _cultureBg:addChild( _attrBg )
        local _attrText = zc.createRichLabel({
            text = v.name,
            fontSize = 20,
            anchor = cc.p( 0, 0.5 ),
            pos = cc.p( 5, _attrBg:getContentSize().height/2 ),
        })
        _attrText:setGradientColor( cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1) )
        _attrBg:addChild( _attrText )
        local _attrNum = zc.createLabel({
            fontSize = 20,
            ap = cc.p( 1, 0.5 ),
            pos = cc.p( 100, _attrText:getPositionY() ),
        })
        _attrBg:addChild( _attrNum )
        local _attrNumLimit = zc.createLabel({
            text = "/1000",
            fontSize = 20,
            ap = cc.p(0, 0.5),
            pos = cc.p(_attrNum:getPositionX(), _attrText:getPositionY()),
            -- color = cc.c3b(222,176,93),
            color = cc.c3b(250,210,160),
        })
        _attrBg:addChild(_attrNumLimit)
        local _attrAddNum = zc.createLabel({
            fontSize = 20,
            ap = cc.p( 0, 0.5 ),
            pos = cc.p( 160, _attrText:getPositionY() ),
        })
        _attrBg:addChild( _attrAddNum )
        local _attrArrow = cc.Sprite:create()
        _attrArrow:setPosition( 206, _attrText:getPositionY() )
        _attrBg:addChild( _attrArrow )
        self._attr[i] = {
            _attrNum = _attrNum,
            _attrAddNum = _attrAddNum,
            _attrArrow = _attrArrow,
            _attrNumLimit = _attrNumLimit,
        }
    end
    -- 保留
    local _keepBtn = zc.createCommomBtn({
        tag = 2,
        text = LANGUAGE_ROLE.keep,
        pos = cc.p( _cultureBgWidth - 250, _cultureBgHeight/2 ),
        endCallback = function()
            zc.sendHttp({
                apiName = "Hero_doCulture",
                params = {id = self._heroData.id, type = 2},
                successCallback = function(backData)
                    -- dump( backData, "保留" )
                    -- 刷新数据
                    PlayerData:setGeneralInfoById( self._heroData.id, backData.hero )
                    self:onFightChange( backData.hero )
                    -- 刷新界面
                    self:refreshAttr()
                    --功能引导
                    local _currentGID = getGuideManager():getCurrentGID()
                    if _currentGID == GuideKey.KEY_11303 then
                        getGuideManager():setFuncGuideOver()
                    end
                end,
            })
        end
    })
    _cultureBg:addChild( _keepBtn )
    self._keepBtn = _keepBtn
    -- 培养
    local _consumeIcon = cc.Sprite:create()
    _consumeIcon:setAnchorPoint( 0, 0.5 )
    _consumeIcon:setPosition( _cultureBgWidth - 105, _cultureBgHeight/2 + 42 )
    _cultureBg:addChild( _consumeIcon )
    self._consumeIcon = _consumeIcon
    local _consumeNum = zc.createLabel({
        fontSize = 24,
        ttf = zc.getFont( 1 ),
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 40, 20 ),
        outLine = true,
    })
    _consumeIcon:addChild( _consumeNum )
    self._consumeNum = _consumeNum
    local _cultureBtn = zc.createCommomBtn({
        tag = 1,
        pos = cc.p( _cultureBgWidth - 90, _cultureBgHeight/2 ),
        endCallback = function()
            if self._choice[self._choiceIndex]._consume.num <= zc.getNumByTypeId( zc.mainTypeResource, self._choice[self._choiceIndex]._consume.id ) then
                zc.sendHttp({
                    apiName = "Hero_culture",
                    params = {id = self._heroData.id, type = self._choiceIndex},
                    successCallback = function(data)
                        -- dump( data, "培养" )
                        -- 刷新资源
                        PlayerData:updataData( data )
                        -- 刷新数据
                        PlayerData:setGeneralInfoById( self._heroData.id, data.hero )
                        -- 刷新次数
                        PlayerData:setLimitInfoById( "culture"..self._choiceIndex, data.cultureNum )
                        -- 刷新界面
                        self:refreshAttr()
                        --功能引导
                        local _currentGID = getGuideManager():getCurrentGID()
                        if _currentGID == GuideKey.KEY_11302 then
                            zc.setGuideFuncNextStep(_currentGID)
                            getGuideManager():addMaskLayer()
                            performWithDelay(self, function()
                                getGuideManager():removeMaskLayer()
                                getGuideManager():setObjTarget(self._keepBtn)
                                getGuideManager():startGuide()
                            end, 0.1)
                        end
                    end,
                })
            else
                zc.tipInfo( zc.getNameById( self._choice[self._choiceIndex]._consume.id )..LANGUAGE_ROLE.not_enough )
                --功能引导
                local _currentGID = getGuideManager():getCurrentGID()
                if _currentGID == GuideKey.KEY_11302 then
                    getGuideManager():setFuncGuideOver()
                end
            end
        end
    })
    _cultureBg:addChild( _cultureBtn )
    self._cultureBtn = _cultureBtn
    -- 已达到最大值
    local _maxLabel = zc.createLabel({
        text = LANGUAGE_ROLE.attribute_level_max,
        fontSize = 25,
        ttf = zc.getFont( 1 ),
        ap = cc.p( 0.5, 0.5 ),
        pos = cc.p( _cultureBgWidth - 170, _cultureBgHeight/2 ),
        outLine = true,
        -- width = 150,
    })
    _cultureBg:addChild( _maxLabel )
    self._maxLabel = _maxLabel

    local _frame = ccui.Scale9Sprite:create( cc.rect( 16, 0, 32, 511 ), "res/Main/common/frame.png" )
    _frame:setContentSize( 611, 513 )
    _frame:setAnchorPoint( 0, 1 )
    local _framePos = cc.p( 2, self._height )
    _framePos = _bg:convertToWorldSpace( _framePos )
    _framePos = _shadowHero:convertToNodeSpace( _framePos )
    _frame:setPosition( _framePos )
    _shadowHero:addChild( _frame )
end
-- 创建武将
function RoleCultureLayer:createHero( _changeWeapon )
    if self._heroArmature then
        self._heroArmature:removeFromParent()
        self._heroArmature = nil
    end
    local _armature = zc.battle.createHeroArmature({
        fileName = self._heroData.modelId,
        heroDataWeaponId = self._weaponData and self._weaponData.weaponId or 0,
        async = true,
        asyncCallback = function( _armature_ , fileName , num )
            if self._exist and _armature_ then
                zc.soldierChangeWeapon( _armature_, self._weaponData and self._weaponData.weaponId or 0 )
                if _changeWeapon then
                    _armature_:getAnimation():play("change2_done")
                else
                    _armature_:getAnimation():play("idle")
                end
                local function animationEvent(armatureBack,movementType,movementID)
                    if movementType == ccs.MovementEventType.complete then
                        armatureBack:getAnimation():play("idle")
                    end
                end
                _armature_:getAnimation():setMovementEventCallFunc(animationEvent)
            end
        end
    })
    _armature:setPosition( self._armatureX, self._armatureY )
    _armature:setScaleX( 1.2 )
    _armature:setScaleY( 1.2 )
    self._shadowHero:addChild( _armature, 1 )
    self._heroArmature = _armature
    self._record[#self._record + 1] = {
        self._heroData.modelId, self._weaponData and self._weaponData.weaponId or 0,
    }
end
-- 点击按钮刷新属性
function RoleCultureLayer:refreshAttr()
    -- 判断是否已经培养过
    local _cultureFlag = false
    if self._heroData.cultureSwap then
        for k, v in pairs( self._heroData.cultureSwap ) do
            if tonumber( v ) ~= 0 then
                _cultureFlag = true
                break
            end
        end
    end
    -- 属性是否满
    local _maxFlag = true
    local _cultureData = self._heroData.culture
    if tonumber( _cultureData.force ) >= tonumber( _cultureData.forceLimit ) and
        tonumber( _cultureData.wisdom ) >= tonumber( _cultureData.wisdomLimit ) and
        tonumber( _cultureData.soma ) >= tonumber( _cultureData.somaLimit ) then
        _maxFlag = true
    else
        _maxFlag = false
    end
    -- 是否没有上升
    local _noUpFlag = true
    -- 是否没有下降
    local _noDownFlag = true
    -- 设置属性显示
    -- dump(self._heroData.attribute,"self._heroData.attribute")
    for i, v in ipairs( self._attr ) do
        local _key = self._attrConfig[i].key
        v._attrNum:setText( self._heroData.attribute[_key[1]] )
        v._attrNumLimit:setText( "/".. (self._heroData.culture[_key[2]] + (tonumber(self._heroList[_key[1]]) or 0) ) )
        if _cultureFlag then
            local _addNum = tonumber( self._heroData.cultureSwap[_key[1]] ) - tonumber( self._heroData.culture[_key[1]] )
            local _addText = ""
            local _addColor = display.COLOR_WHITE
            local _arrowPath = ""
            local _arrowDegree = 0
            if _addNum > 0 then
                _addText = "+".._addNum
                _addColor = zc.COLOR_GREEN
                _arrowPath = "res/Main/common/arrow_green.png"
                _arrowDegree = -90
                _noUpFlag = false
            elseif _addNum < 0 then
                _addText = _addNum
                _addColor = zc.COLOR_RED
                _arrowPath = "res/Main/common/arrow_red.png"
                _arrowDegree = 90
                _noDownFlag = false
            end
            v._attrAddNum:setText( _addText )
            v._attrAddNum:setColor( _addColor )
            v._attrAddNum:setVisible( true )
            v._attrArrow:setTexture( _arrowPath )
            v._attrArrow:setRotation( _arrowDegree )
            v._attrArrow:setVisible( true )
        else
            v._attrAddNum:setVisible( false )
            v._attrArrow:setVisible( false )
        end
    end
    -- 计算消耗
    for i = 1, 4 do
        -- 培养消耗
        local _consumeList = self._consumeList[tostring( i )]
        -- dump( _consumeList, "_consumeList" )
        local _consumeNum = 0
        local _type = tonumber( _consumeList.expendType ) or 1
        if _type == 1 then
            -- 元宝根据次数递增
            _consumeNum = tonumber( _consumeList.expendNum ) + tonumber( _consumeList.expendAddNum )*tonumber( PlayerData:getLimitInfoById( "culture"..i ) )
        elseif _type == 2 then
            -- 银币根据主角等级递增
            _consumeNum = tonumber( _consumeList.expendNum )*tonumber( PlayerData.info.level )
        end
        self._choice[i]._consume = {id = _type, num = _consumeNum}
    end
    -- 设置按钮显示
    _cultureBgWidth = self._cultureBgWidth
    _cultureBgHeight = self._cultureBgHeight
    if _cultureFlag then
        -- 如果培养过
        self._maxLabel:setVisible( false )
        if _noUpFlag and _noDownFlag then
            -- 如果属性都没变，显示两个按钮
            self._keepBtn:setVisible( true )
            self._keepBtn:setPositionX( _cultureBgWidth - 250 )
            self._cultureBtn:setVisible( true )
            self._cultureBtn:setPositionX( _cultureBgWidth - 90 )
            self._consumeIcon:setVisible( true )
        elseif _noUpFlag then
            -- 如果属性没有上升，只显示培养
            self._keepBtn:setVisible( false )
            self._cultureBtn:setVisible( true )
            self._cultureBtn:setPositionX( _cultureBgWidth - 170 )
            self._consumeIcon:setVisible( true )
        elseif _noDownFlag then
            -- 如果属性没有下降，只显示保留
            self._keepBtn:setVisible( true )
            self._keepBtn:setPositionX( _cultureBgWidth - 170 )
            self._cultureBtn:setVisible( false )
            self._consumeIcon:setVisible( false )
        else
            -- 如果属性有升有降，显示两个按钮
            self._keepBtn:setVisible( true )
            self._keepBtn:setPositionX( _cultureBgWidth - 250 )
            self._cultureBtn:setVisible( true )
            self._cultureBtn:setPositionX( _cultureBgWidth - 90 )
            self._consumeIcon:setVisible( true )
        end
    else
        -- 如果没有培养
        self._keepBtn:setVisible( false )
        if _maxFlag then
            -- 属性全满
            self._maxLabel:setVisible( true )
            self._cultureBtn:setVisible( false )
            self._consumeIcon:setVisible( false )
        else
            self._maxLabel:setVisible( false )
            self._cultureBtn:setPositionX( _cultureBgWidth - 170 )
            self._cultureBtn:setVisible( true )
            self._consumeIcon:setVisible( true )
        end
    end

    self:refreshConsume()
end
-- 战斗力变化响应
function RoleCultureLayer:onFightChange( data )
    zc.numChangeAnimation({
        label = self._fightNum,
        pos = cc.p( self._fightNum:getContentSize().width/2, 20 ),
        callback = function()
            self._fightNum:setString( data.fighting )
        end
    })
end

-- 更换武将刷新
function RoleCultureLayer:refreshHero( params )
    self:refreshHeroData( params )

    self:refreshHeroUI()
end
function RoleCultureLayer:refreshHeroData( params )
    -- 数据
    self._id = tonumber( params.id ) or 1

    -- 武将数据
    self._heroData = PlayerData:getGeneralInfoById( self._id )

    -- 武将信息，静态表，用来显示资质和阵营和兵器图标
    self._heroList = gameData.getDataFromCacheWithPrimaryKey("hero", self._heroData.heroId)

    -- 兵器数据
    self._weaponData = PlayerData:getWeaponsInfoById( self._id )

    -- 是否是主角
    local _mainRoleId = tonumber( PlayerData.info.heroId ) or 0
    self._isMainRole = _mainRoleId == self._id

    self._record = params.record
end
function RoleCultureLayer:refreshHeroUI()
    -- 名字
    local _nameText = zc.getLevelTipShow()..self._heroData.level.." "..( self._isMainRole and PlayerData.info.userName or  self._heroData.name )
    local _reincarnate = tonumber( self._heroData.reincarnate ) or 0
    if _reincarnate > 0 then
        _nameText = _reincarnate .. zc.lang("turn") .. _nameText
    end
    local _newAdvance = tonumber(self._heroData.newAdvance) or 0
    if _newAdvance > 0 then
        _nameText = _nameText .. "+" .. _newAdvance
    end
    local _titleText = self._heroData.title
    self._name:setString( _nameText.."  [color=ffe400ff]< ".._titleText.." >[/color]" )if self._isMainRole then
        self._selectTitle:setVisible( true )
    else
        self._selectTitle:setVisible( false )
    end
    -- 兵器图标
    local _weaponType = 1
    if self._isMainRole then
        -- 主角
        _weaponType = tonumber( self._weaponData.type ) or 1
    else
        _weaponType = tonumber( self._heroList.weaponType ) or 1
    end
    self._weaponIcon:setTexture( zc.getHeroTypeBigImg( _weaponType ) )
    -- 星级
    zc.createHeroStar( self._shadowHero, self._heroData.star, cc.p( 32, 277 ) )
    -- 战斗力
    self._fightNum:setString( self._heroData.fighting )
    -- 阵营
    self._camp:setTexture( "res/Main/role/camp_"..self._heroList.camp..".png" )
    -- 武将
    self:createHero()
    -- 刷新界面
    self:refreshAttr()
end
-- 刷新培养消耗
function RoleCultureLayer:refreshConsume()
    local _consumeData = self._choice[self._choiceIndex]._consume
    local _consumeIconInfo = zc._getIconInfo({data={mainType=1,type=_consumeData.id}})
    self._consumeIcon:setTexture( _consumeIconInfo.img )
    self._consumeNum:setText( _consumeData.num )
    self._consumeIcon:setPositionX( self._cultureBtn:getPositionX() - ( self._consumeIcon:getContentSize().width + self._consumeNum:getContentSize().width )/2 )
    if _consumeData.num > zc.getNumByTypeId( zc.mainTypeResource, _consumeData.id ) then
        self._consumeNum:setColor( zc.COLOR_RED )
    else
        self._consumeNum:setColor( display.COLOR_WHITE )
    end
    self._cultureBtn:setText( self._cultureConfig[self._choiceIndex] )
end

return RoleCultureLayer