--[[
	FileName: RoleTitlePop.lua
	Author: andong
	Date: 2016-03-17
	Purpose: 选择称号弹窗
]]
local RoleTitlePop = class( "RoleTitlePop", function ()
    return ZCPopLayer:create()
end)
function RoleTitlePop:create(params)
	return RoleTitlePop.new(params)
end
function RoleTitlePop:ctor(params)
	self:initData(params)
	self:initUI()
	self:show()
end
function RoleTitlePop:initData(params)
	self._callback = params.callback
	self._titleList = {}
	if params and params.list then
        for i = 1, table.nums(params.list) do
            local v = params.list[i]
            if v and v.type then
                local titleList = gameData.getDataFromCache("title", {displayPosition=v["type"]} ) or {}
                local titleInfo = titleList[tonumber(v["rank"])] or {}
                if titleInfo and titleInfo["name"] and titleInfo["name"] ~= "" then
                    local idx = #self._titleList+1
                    self._titleList[idx] = v                 
                    self._titleList[idx].titleInfo = titleInfo                 
                end
            end
        end
	end
	self._titleNums = table.nums(self._titleList)
end
function RoleTitlePop:initUI()
    local popBg = zc.createNormalPop({
        tag = 1,
        titleTag = 3,
        showSide = false,
        height = 450+31,
        width = 377,
        bianZOrder = 0
    })
    popBg:setPosition(zc.width/2, zc.height/2)
    self:addContent(popBg)
    self._popBg = popBg
    local popSize = cc.size(377,450)
    self._popSize = cc.size(377,450)

    local middle_bg = zc.createGeneralPopScale({
        tag = 3,
        width = 353,
        height = 382,
    })
    middle_bg:setAnchorPoint(cc.p(0.5, 0))
    middle_bg:setPosition(cc.p(self._popSize.width/2-2, 15))
    self._popBg:addChild(middle_bg)
    self._middle_bg = middle_bg

    self:createTableview()

end
function RoleTitlePop:createTableview()
    local tableSize = cc.size(self._middle_bg:getContentSize().width, self._middle_bg:getContentSize().height-10)
    local tableNode = self._middle_bg
    local myTableBg = zc.createSprite()
    myTableBg:setContentSize( tableSize )
    myTableBg:setAnchorPoint(cc.p(0.5, 0.5))
    myTableBg:setPosition(cc.p(tableNode:getContentSize().width/2-3, tableNode:getContentSize().height/2))
    tableNode:addChild(myTableBg)


    local tipNoItemLab = zc.createSprite("res/Main/common/empty.png")
    tipNoItemLab:setPosition(tableSize.width/2, tableSize.height/2)
    myTableBg:addChild(tipNoItemLab)
    tipNoItemLab:setVisible(false)
    self._tipNoItemLab = tipNoItemLab
    if self._titleNums == 0 then
        tipNoItemLab:setVisible(true)
    end
    local myTable = zc.ZCTableView:create(cc.size(myTableBg:getContentSize().width-4,myTableBg:getContentSize().height-4))
    myTable:setPosition(2,2)
    myTable:setInertia(true) --设置惯性
    myTable:setBounceable(true)
    myTable:setDirection(zctech.SCROLLVIEW_DIRECTION_VERTICAL) --设置横向纵向
    myTable:setDelegate()
    myTable:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    myTableBg:addChild(myTable)
    self._myTableview = myTable
    local cellSize = cc.size(tableSize.width, 100)
    local function cellSizeForTable(table,idx)
        return cellSize.height, cellSize.width
    end
    local function numberOfCellsInTableView(table)
        return self._titleNums
    end
    local function tableCellAtIndex(table,idx)
        local nowIdx = idx + 1
        local cell = table:dequeueCell()
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            cell:setContentSize(cc.size(cellSize.width, cellSize.height))
        else
        	cell:removeAllChildren()
        end
        self:createCell(cell, nowIdx)
        return cell
    end
    myTable:registerScriptHandler(numberOfCellsInTableView,cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    myTable:registerScriptHandler(cellSizeForTable,cc.TABLECELL_SIZE_FOR_INDEX)
    myTable:registerScriptHandler(tableCellAtIndex,cc.TABLECELL_SIZE_AT_INDEX)
    myTable:reloadData()
end
function RoleTitlePop:createCell(cell, nowIdx)
    local cellSize = cell:getContentSize()
    local cellData = self._titleList[nowIdx]
    local titleInfo =cellData.titleInfo or {}

    local title_list_bg = zc.getScaleNode("res/Main/role/title_list_bg.png", cc.size(332, 92))
    title_list_bg:setPosition(cc.p(cellSize.width/2, cellSize.height/2))
    cell:addChild(title_list_bg)

    local chenghaoImgPath = "res/Main/rankList/rank_"..cellData["rank"]..".png"
    --设置称号
    local titleBtn = zc.createPushButton({
    	normalFile = chenghaoImgPath,
    	needSwallow = false,
        needEnableWhenMoving = true,
        touchScale = 0.9,
    	endCallback = function()
    		zc.sendHttp({
    		    apiName = "Chenghao_set",
    		    params = {type=cellData["type"],rank=cellData["rank"]},
    		    successCallback = function (net_data)
    		    	zc.tipInfo(zc.lang("tips_set_success"))
    		    	--更新称号
    		    	PlayerData.info.godName = titleInfo["id"] or ""

                    local PlayerId = PlayerData.info.heroId
                    PlayerData:setGeneralInfoById(PlayerId,{title=titleInfo["name"]})
                    LayerManager.removeLayout(true)
    		    end
    		})
    	end,
    	pos = cc.p(cc.p(cellSize.width/2, cellSize.height/2+10))
    })
    cell:addChild(titleBtn)
    local titleStr = titleInfo["name"] or ""
    local top3Title = zc.createLabel({
        text = titleStr,
        stroke = false,
        fontSize = 18,
        fontName = zc.getFont(1),
        ap = cc.p(0.5, 0.5),
        pos = cc.p(61, 16),
        color = display.COLOR_GOLD,
    })
    titleBtn:addChild(top3Title, 1)
end

function RoleTitlePop:onCleanup()
	if self._callback then
		self._callback()
	end
end

return RoleTitlePop