--[[
	战法技能预览
	参数：id = 技能静态id，level = 技能等级
]]
local ZhanFaSkillPreviewPop = class("ZhanFaSkillPreviewPop", function()
	return ZCPopLayer:create()
end)

function ZhanFaSkillPreviewPop:create( params )
	return self.new( params )
end

function ZhanFaSkillPreviewPop:ctor( params )
	-- self:show()
	self:initData( params )
	self:initUI()
end

function ZhanFaSkillPreviewPop:initData( params )
	dump(params)

	self.skillIndex = tonumber(params.index)
	self.skillType = tonumber(params.skillData.skillType)
	self.equipInfoList = {}

	--组织下技能，被吃武将的信息
	if self.skillType == 3 then
		self.skillId = tonumber(params.heroData.moraleSkillId)
		self.skillLevel = tonumber(params.heroData.moraleSkillLevel)
		for i=1,4 do
			if tonumber(params.heroData.tactics["tplace1" .. i]) ~= 0 then
				local info = {}
				info.id = tonumber(params.heroData.tactics["tplace1" .. i])
				info.lv = tonumber(params.heroData.tactics["tlevel1" .. i])
				table.insert(self.equipInfoList, info)
			end
		end
	elseif self.skillType == 4 then
		if params.index == 1 then
			self.skillId = tonumber(params.heroData.combineSkillId)
			self.skillLevel = tonumber(params.heroData.combineSkillLevel)
		else
			self.skillId = tonumber(params.heroData["combineSkillId" .. self.skillIndex])
			self.skillLevel = tonumber(params.heroData["combineSkillLevel" .. self.skillIndex])
		end

		for i=1,8 do
			if tonumber(params.heroData.tactics["tplace2" .. i]) ~= 0 then
				local info = {}
				info.id = tonumber(params.heroData.tactics["tplace2" .. i])
				info.lv = tonumber(params.heroData.tactics["tlevel2" .. i])
				table.insert(self.equipInfoList, info)
			end
		end
	end

	self._desc, self._skillData = zc.buildSkillDesc(self.skillId, self.skillLevel)
	-- dump(self.equipInfoList)

	-- self.weaponTypeList = self:getEquipType()

	self._size = self:getContentSize()

	-- self.desc_list = PlayerData:getDesc(self.equipInfoList,self._skillData)
	-- dump(self.desc_list,"----****----测试")
end

function ZhanFaSkillPreviewPop:initUI()
	--大概高度
	local preHeight = 0

	if #self.equipInfoList ~= 0 then
		preHeight = 0 + math.min(#self.equipInfoList,4)*50
	end

	-- 背景
	local background = ccui.Scale9Sprite:create( cc.rect( 15, 15, 70, 70 ), "res/Main/common/common_inner17.png" )
	background:setContentSize( 460, 305 + preHeight )
	background:setPosition( self._size.width/2, self._size.height/2 )
	self:addContent( background )
	local bgSize = background:getContentSize()

	-- 技能图标
	local iconBg = cc.Sprite:create( "res/Main/role/starup/iconBg.png" )
	iconBg:setPosition( 90, bgSize.height - 80 )
	background:addChild( iconBg )
	local icon = zc.createIconByData({
		data = {
			mainType = zc.mainTypeSkill,
			type = self.skillId,
			id = self.skillId,
		},
		sexId = self._sexId,
	})
	icon:setPosition( iconBg:getPosition() )
	background:addChild( icon )
	-- 技能名字
	local skillName = zc.createLabel({
		text = self._skillData.name or "",
        fontSize = 22,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 170, bgSize.height - 60 ),
        color = display.COLOR_YELLOW,
        outLine = true,
        pixel = 2,
	})
	background:addChild( skillName )

	-- 技能等级
	if self.skillLevel > 0 then
		local skillLevel = zc.createLabel({
			text = zc.getLevelTipShow()..self.skillLevel,
	        fontSize = 22,
	        ap = cc.p( 0, 0.5 ),
	        pos = cc.p( skillName:getPositionX() + skillName:getContentSize().width + 20, skillName:getPositionY() ),
	        color = display.COLOR_YELLOW,
	        outLine = true,
	        pixel = 2,
		})
		background:addChild( skillLevel )
	end

	-- 合体技能
	local heroName = {}
	for i = 1, 4 do
		local heroId = tonumber( self._skillData["heroId"..i] ) or 0
		if heroId ~= 0 then
			heroName[#heroName + 1] = gameData.getDataFromCacheWithPrimaryKey("hero", heroId).name
		end
	end

	local heroText = nil
	if #heroName > 0 then
		-- 有合体的英雄
		heroText = heroName[1]
		for i = 2, #heroName do
			heroText = heroText.."、"..heroName[i]
		end
		heroText = LANGUAGE_heti_skill_unrich_tip(heroText, "")
	elseif self._stageSkillActive then
		heroText = zc.lang("breakthrough") .. "+" .. self._stageSkillActive .. LANGUAGE_ROLE.unlock
	end
	if heroText then
		local heroLabel = zc.createLabel({
			text = heroText,
	        fontSize = 20,
	        ap = cc.p( 0, 0.5 ),
	        pos = cc.p( 170, bgSize.height - 110 ),
	        color = display.COLOR_YELLOW,
	        outLine = true,
	        pixel = 2,
		})
		background:addChild(heroLabel)
	end
	-- 分隔线
	local split = ccui.Scale9Sprite:create( cc.rect( 0, 0, 1, 2 ), "res/Main/common/spline.png" )
	split:setContentSize( bgSize.width - 20, 2 )
	split:setPosition( bgSize.width/2, 150 )
	background:addChild( split )

	-- 技能说明背景
	local descBg = ccui.Scale9Sprite:create( cc.rect( 18, 18, 4, 4 ), "res/Main/common/descDi.png" )
	descBg:setContentSize( bgSize.width - 35, 115 )
	descBg:setPosition( bgSize.width/2, 80 )
	background:addChild( descBg )
	local desc = zc.createLabel({
        text = self._desc,
        width = descBg:getContentSize().width - 20,
        fontSize = 20,
        ap = cc.p( 0.5, 1 ),
        pos = cc.p( descBg:getContentSize().width/2 + 20, descBg:getContentSize().height ),
        outLine = true,
    })
	background:addChild( desc )

	local tbPosConfig = {-355/2 + 40, -355/2 + 90, -355/2 + 140, -355/2 + 190, -355/2 + 200}

	if #self.equipInfoList ~= 0 then
		iconBg:setScale(0.8)
		icon:setScale(0.8)
		split:setPositionY(bgSize.height - 155)
		descBg:setVisible(false)
		desc:setPositionY(bgSize.height - 165)

		local skillbg = zc.createSprite("res/Main/role/add_attr.png" )
		skillbg:setPosition(cc.p( descBg:getContentSize().width/2 + 17,desc:getPositionY() - desc:getContentSize().height -skillbg:getContentSize().height/2 - 10))
		background:addChild(skillbg)
		local attrLabel = zc.createLabel({
	        text = zc.lang("fujiashuxing"),
	        fontSize = 20,
	        color = ccc3(228, 155, 41),
	        pos = cc.p( descBg:getContentSize().width/2 + 17, desc:getPositionY() - desc:getContentSize().height-skillbg:getContentSize().height/2 - 10 ),
	        outLine = true,
	    })
		background:addChild(attrLabel)

		local height_desc = desc:getContentSize().height
		local height_skilbg = skillbg:getContentSize().height
		local distance = height_desc + height_skilbg + 20 + 215

		--附加属性tbView
		local tbView = zc.ZCTableView:create( cc.size(bgSize.width - 40 ,215))
	    tbView:setDirection( cc.SCROLLVIEW_DIRECTION_VERTICAL )
	    tbView:setPosition(20, split:getPositionY() - distance)
	    tbView:setDelegate()
	    tbView:setVerticalFillOrder(cc.TABLEVIEW_FILL_TOPDOWN)
	    if #self.equipInfoList < 5 then
	    	tbView:setBounceable(false)
	    end

	    local function cellSizeForTable(table, cellIndex) 
        	return 50, bgSize.width - 30
	    end
	    local function tableCellAtIndex(table, cellIndex) 
	        local cell = table:dequeueCell()
	        cellIndex = cellIndex + 1

	        local nowSkillLevel = self.equipInfoList[cellIndex].lv
	        local nowSkillId = self.equipInfoList[cellIndex].id
	        
	        local data = PlayerData:getSignGeneralById(nowSkillId)
	        local nowWeaponType = data.weaponType
	        
	        local desc_list = PlayerData:getDesc(self.equipInfoList[cellIndex],self._skillData)
	        local desc = desc_list[nowSkillId] or zc.lang("skill")
	        -- 创建模板
	        if cell == nil then
	            cell = zc.ZCTableViewCell:new()
	            local cellBg = ccui.Scale9Sprite:create(cc.rect(12, 11, 14, 10), "res/Main/role/common/name_bg.png")
				cellBg:setContentSize(bgSize.width - 40,40)
				cellBg:setPosition(ccp(bgSize.width/2 - 20, 25))
	            cell:addChild(cellBg)

	            local weaponIcon = zc.createSprite("res/Main/role/common/weapon_icon_1.png" )
	            weaponIcon:setPosition(ccp(20, 20))
	            cellBg:addChild(weaponIcon)
	            cell.weaponIcon = weaponIcon
	            weaponIcon:setScale(0.5)
	            local des = zc.createRichLabel({
			        text = desc,
			        fontSize = 18,
			        anchor = cc.p(0, 0.5),
			        color = ccc3(228, 155, 41),
			        pos = cc.p( 40, 20 ),
			        outLine = true,
			    })
	            cellBg:addChild(des)

	            cell.des = des
	        end

	        -- 更新界面
	        cell.weaponIcon:setTexture("res/Main/role/common/weapon_icon_" .. nowWeaponType .. ".png")
	        cell.des:setString(desc)

	        return cell
	    end
	    local function numberOfCellsInTableView()
	    	local num = #self.equipInfoList
	        return num
	    end
	    tbView:registerScriptHandler(cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX)
	    tbView:registerScriptHandler(tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX)
	    tbView:registerScriptHandler(numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
	    background:addChild( tbView )
	    tbView:reloadData()
	end
end

--获取装备的将的武器类型
function ZhanFaSkillPreviewPop:getEquipType()
	local listData = self.equipInfoList

	local list = {}
    for i=1,#listData do
    	local id = listData[i].id
        local data = PlayerData:getSignGeneralById(id)
    	list[tonumber(id)] = tonumber(tonumber(data.weaponType))
    end

    return list
end

return ZhanFaSkillPreviewPop