--[[
    FileName: SelfInfoPop.lua
    Author: andong
    Date: 2016-03-17
    Purpose: 设置弹窗
]]
local SelfInfoPop = class( "SelfInfoPop", function ()
    return zc.createNormalDialog({
        closeCallBack = function()
            LayerManager.removeLayout(nil, nil, false)
        end
    })
end)

function SelfInfoPop:create(params)
    return SelfInfoPop.new(params)
end

function SelfInfoPop:ctor(params)
    self:initData(params)
    self:initUI()
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_REFRESH_NAME_SELFINFO,
        callback = function()
            self:refreshUserName()
        end
    })
end

function SelfInfoPop:refreshUserName()
    self._selfName:setString(self._playerInfo.userName)
end

function SelfInfoPop:initData(params)
    --玩家资源信息
    cc.SpriteFrameCache:getInstance():addSpriteFrames("res/Main/home/homeUI.plist", "res/Main/home/homeUI.pvr.ccz")
    self._fontColor = cc.c3b(230, 224, 163)
    local resourceInfo = PlayerData:getResourcesInfo()
    self._resourceInfo = table.copy(resourceInfo)
    self._callback = params.callback
    --玩家资料信息
    local playerInfo = PlayerData:getBaseInfo()
    self._playerInfo = playerInfo
    --体力恢复间隔
    local curTime = PlayerData:getTime() --当前时间
    self._interval = tonumber(resourceInfo.energyInterval) --时间间隔
    self._lastTime = self._interval - (tonumber(curTime) - tonumber(resourceInfo.energyTime))%self._interval --当前点体力恢复时间 
    --体力回满总时间
    self._totalTime = self._lastTime + (tonumber(resourceInfo.energyLimit)-tonumber(resourceInfo.energy)-1)*self._interval
    self._time = self._lastTime
        
    if self._time < 0 then
        self._time = 0 
    end
    if tonumber(resourceInfo.energyLimit) <= tonumber(resourceInfo.energy) then
        self._time = 0 
        self._totalTime = 0
    end
    --计算木矿和铁矿
    self._woodInfo = {
        num = 0, --矿数量
        yield = 0, --每小时产量
        numLimit = 0, --容量
    }
    self._ironInfo = {
        num = 0, --矿数量
        yield = 0, --每小时产量
        numLimit = 0, --容量
    }
    for i = 1, #PlayerData.houseinfo.houses do
        local houseInfo = PlayerData.houseinfo.houses[i]
        if houseInfo.houseId == 6 then --木材
            self._woodInfo.num = self._woodInfo.num + 1
            self._woodInfo.yield = self._woodInfo.yield+tonumber(houseInfo.info.yield)
            self._woodInfo.numLimit = self._woodInfo.numLimit+tonumber(houseInfo.info.numLimit)
        elseif houseInfo.houseId == 7 then --铁矿
           self._ironInfo.num = self._ironInfo.num + 1
            self._ironInfo.yield = self._ironInfo.yield+tonumber(houseInfo.info.yield)
            self._ironInfo.numLimit = self._ironInfo.numLimit+tonumber(houseInfo.info.numLimit)
        end
    end
end

function SelfInfoPop:initUI()
    local popSize = cc.size(500, 300)
    local popBg = zc.createNormalPop({
        tag = 2,
        src = "res/Main/common/popBg/bgWhite.png",
        bianZOrder = 1,
    })
    popBg:setPosition(zc.width*0.5, zc.height*0.5)
    self:addChild(popBg)
    self._popBg = popBg

    local popSize = popBg:getContentSize()

    local _title = zc.getSprite({
        src = "res/Main/setting/infoTitle.png",
        ap = cc.p(0, 0),
        pos = cc.p(-70, popSize.height - 60),
    })
    popBg:addChild(_title, 1)

    local leftBg = cc.Node:create()
    leftBg:setContentSize(388, 459)
    leftBg:setAnchorPoint(0, 0)
    popBg:addChild(leftBg)
    self._leftBg = leftBg

    local rightBg = zc.createNormalPop({
        width = 540,
        height = popSize.height - 15
    })
    rightBg:setAnchorPoint(1, 0)
    rightBg:setPosition(popSize.width, 0)
    popBg:addChild(rightBg)
    self._rightBg = rightBg


    self:initLeft()
    self:initRight()
end

function SelfInfoPop:initLeft()
    local scaleSize = self._leftBg:getContentSize()
    local _shadow = zc.getScaleNode("res/Main/common/popBg/shadow_white.png", cc.size(scaleSize.width, 12))
    _shadow:setAnchorPoint(0.5, 1)
    _shadow:setPosition(scaleSize.width*0.5, scaleSize.height)
    self._leftBg:addChild(_shadow, 1)
    --主角图

    local head,_src = PlayerData:getMainRoleHeadImg()
    if not zc.checkFileExist(_src) then
        _src = "res/Main/hero/body/101.png"
    end

    local _view = zc.createScrollview({
        size = scaleSize,
        innerSize = scaleSize,
        touchEnable = false,
    }) 
    self._leftBg:addChild(_view)
    _view:addChild(zc.getSprite({
        src = _src,
        ap = cc.p(0.5,1),
        pos = cc.p(scaleSize.width*0.5, scaleSize.height+20)
    }))

    --显示log按钮
    local touchCount = 0
    SHOW_LOG_BUTTON_TEST = false
    local showLogBtn = zc.createPushButton({
        normalNode = zc.getScaleNode("res/Main/common/img_empty.png", cc.size(150, 150)),
        needSwallow = false,
        endCallback = function()
            touchCount = touchCount + 1
            if touchCount > 15 then
                SHOW_LOG_BUTTON_TEST = true
                cc.Director:getInstance():setDisplayStats(true)
                touchCount = 0
            end
        end,
        anchor = cc.p(0.5, 0.5),
        pos = cc.p(scaleSize.width/2, scaleSize.height/2)
    })
    self._leftBg:addChild(showLogBtn, 100)
    
end
function SelfInfoPop:initRight()
    local rightBg = self._rightBg
    local rightSize = self._rightBg:getContentSize()

    local settingBtn = zc.createPushButton({
        normalFile = "res/Main/setting/setting_btn_n.png",
        selectedFile = "res/Main/setting/setting_btn_h.png",
        needSwallow = false,
        touchScale = 0.9,
        pos = cc.p(rightSize.width - 45, rightSize.height - 45),
        endCallback = function()
            zc.createSourcePop("Setting", "src/layer/setting/SettingPop.lua")
        end,
    })
    rightBg:addChild(settingBtn) 
    --vip
    local vipBg = zc.createPushButton({
        normalNode = zc.createSpriteWithFrameName("vipDi.png"),
        pos = cc.p(55, 420),
        touchScale = 0.9,
        touchSize = cc.size( 80, 80 ),
        needSwallow = true,
        endCallback = function ()
            zc.createVipPop()
        end
    }) 
    rightBg:addChild(vipBg)
    local vipSp = zc.getSprite({
        src = (APP_REVIEW and "res/Main/setting/vip_review.png" or "vip.png"),
        plist = (not APP_REVIEW),
        pos = cc.p(vipBg:getContentSize().width*0.5, 12),
    })
    vipBg:addChild(vipSp)
    local vipNum = zc.createFntLabel({
        fnt = "res/Main/vip/fnt/vip.fnt",
        text = PlayerData.info.vip,
        pos = cc.p(vipBg:getContentSize().width*0.5 - 2, 33),
        addwidth = -5,
    })
    vipBg:addChild(vipNum)
    -- name
    local nicheng = zc.createLabel({
        text = zc.lang("game_name") .. ":",
        fontSize = 18,
        ap = cc.p(0 , 0),
        pos = cc.p(vipBg:getPositionX() + 70, vipBg:getPositionY()),
        color = self._fontColor,
    })
    rightBg:addChild(nicheng)
    local selfName = zc.createLabel({
        text = self._playerInfo.userName,
        fontSize = 18,
        ap = cc.p(0, 0),
        pos = cc.p(nicheng:getPositionX()+nicheng:getContentSize().width+20, nicheng:getPositionY()),
    })
    rightBg:addChild(selfName)
    self._selfName = selfName

    local country = zc.createLabel({
        text = zc.lang("country") ..":",
        fontSize = 18,
        ap = cc.p(0, 1),
        pos = cc.p(nicheng:getPositionX(), nicheng:getPositionY() - 5),
        color = self._fontColor,
    })
    rightBg:addChild(country)
    local countryNameStr = self._playerInfo.countryName
    if not countryNameStr or countryNameStr == "" then
        countryNameStr = zc.lang("no_have_for_now")
    end
    local countryName = zc.createLabel({
        text = countryNameStr,
        fontSize = 18,
        ap = cc.p(0, 1),
        pos = cc.p(country:getPositionX()+country:getContentSize().width+20, country:getPositionY()),
    })
    rightBg:addChild(countryName)

    if not APP_REVIEW then
        local changeName = zc.createPushButton({
            normalFile = "res/Main/setting/changeName_btn.png",
            anchor = cc.p(0, 0),
            pos = cc.p(nicheng:getPositionX() + 210, nicheng:getPositionY()),
            name = "changeName",
            endCallback = function()
                zc.createSourcePop("ChangeName", "src/layer/setting/ChangeNamePop.lua")
                -- zc.tipInfo(zc.lang("tips_unopen_fornow") .. "!")
            end,
        })
        rightBg:addChild(changeName)
    end

    --战斗力 
    local _fightBg = zc.getSprite({
        src = "res/Main/role/common/fight_bg.png",
        ap = cc.p(0, 0.5),
        pos = cc.p(25, vipBg:getPositionY() - 62),
        scale = 0.9,
    })
    rightBg:addChild( _fightBg)
    local _fightNum = zc.createFntLabel({
        fnt = "res/Main/common/fnt/fight_change.fnt",
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 90, _fightBg:getContentSize().height/2 ),
        addwidth = -15,
        text = self._playerInfo.fighting
    })
    _fightNum:setScale( 0.7 )
    _fightBg:addChild( _fightNum )
    -- level
    local levelbg = zc.createImageView({
        src = "res/Main/common/avator_levelbg.png",
        ap = cc.p(0, 0.5),
        pos = cc.p(rightSize.width*0.5 - 10, _fightBg:getPositionY()),
    })
    rightBg:addChild(levelbg)
    local level = zc.createLabel({
        text = self._playerInfo.level,
        fontSize = 20,
        shadow = true,
        ap = cc.p(0.5, 0.5),
        pos = cc.p(levelbg:getContentSize().width/2, levelbg:getContentSize().height/2),
    })
    levelbg:addChild(level)

    --经验条
    local expbarbg = zc.createImageView({
        src = "res/Main/home/avator_expbarbg.png",
        ap = cc.p(0, 0.5),
        pos = cc.p(levelbg:getPositionX() + 5, levelbg:getPositionY()-2),
    })
    rightBg:addChild(expbarbg, -1)
    expbarbg:setScaleX(1.2)
    local expbar = cc.ProgressTimer:create(zc.createSprite("res/Main/home/avator_expbar.png"))
    expbar:setType(cc.PROGRESS_TIMER_TYPE_BAR)
    expbar:setMidpoint(cc.p(0, 0.5))
    expbar:setBarChangeRate(cc.p(1, 0))
    expbar:setPosition(cc.p(expbarbg:getContentSize().width/2, expbarbg:getContentSize().height/2))
    expbarbg:addChild(expbar)
    expbar:setPercentage(tonumber(self._playerInfo.exp)/tonumber(self._playerInfo.expLimit)*100)
    local expNum = zc.createLabel({
        text = self._playerInfo.exp.."/"..self._playerInfo.expLimit,
        fontSize = 18,
        outLine = true,
        ap = cc.p(0.5, 0.5),
        pos = cc.p(expbarbg:getPositionX()+expbarbg:getBoundingBox().width/2, expbarbg:getPositionY()),
    })
    rightBg:addChild(expNum)
    --
    local resBg = zc.createImageView({
        src = "res/Main/setting/wordBack.png",
        ap = cc.p(0, 0.5),
        pos = cc.p(9, rightSize.height-144),
    })
    rightBg:addChild(resBg)
    local myLab = zc.createLabel({
        text = zc.lang("items_preview"),
        fontSize = 18,
        ap = cc.p(0, 0.5),
        pos = cc.p(32, resBg:getContentSize().height/2),
    })
    resBg:addChild(myLab)

    local resBg2 = zc.createImageView({
        src = "res/Main/setting/wordBack.png",
        ap = cc.p(0, 0.5),
        pos = cc.p(9, rightSize.height - 305),
    })
    rightBg:addChild(resBg2)
    local myLab2 = zc.createLabel({
        text = zc.lang("resource_preview"),
        fontSize = 18,
        ap = cc.p(0, 0.5),
        pos = cc.p(32, resBg:getContentSize().height/2),
    })
    resBg2:addChild(myLab2)

    local scaleInfo = zc.getScaleNode("res/Main/role/common/name_bg.png", cc.size(480, 120))
    scaleInfo:setAnchorPoint(0.5, 1)
    scaleInfo:setPosition(rightBg:getContentSize().width/2, resBg:getPositionY()-18)
    rightBg:addChild(scaleInfo)
    
    local scaleInfo2 = zc.getScaleNode("res/Main/role/common/name_bg.png", cc.size(480, 120))
    scaleInfo2:setAnchorPoint(0.5, 1)
    scaleInfo2:setPosition(rightBg:getContentSize().width/2, resBg2:getPositionY()-20)
    rightBg:addChild(scaleInfo2)

    self:inintInfoFirst(scaleInfo)
    self:inintInfoSecond(scaleInfo2)

end
--第一个info信息
function SelfInfoPop:inintInfoFirst(_infoNode)

    local line = zc.getSprite({
        src = "res/Main/setting/separate_line.png",
        ap = cc.p(0.5, 0.5),
        pos = cc.p(_infoNode:getContentSize().width/2, _infoNode:getContentSize().height/2),
    })
    _infoNode:addChild(line)

    local perHeight = _infoNode:getContentSize().height/3
    local leftPosX = 34
    local middlePosX = _infoNode:getContentSize().width/2 + 30
    local upgrade_card_num = 0
    if PlayerData:getPropByMainTypeAndSubType(1,1001) then
        upgrade_card_num = PlayerData:getPropByMainTypeAndSubType(1,1001).num
    end
    local _tmpDb = gameData.getDataFromCacheWithPrimaryKey("item", zc.cardExtractPopId)
    local cardExtractPopName = _tmpDb.name or zc.lang("zhaomujuan")
    local cardExtractPopNum = zc.getNumByTypeId(zc.mainTypeProp, zc.cardExtractPopId)

    local itemPos = {
        [1] = cc.p(leftPosX, perHeight*2 + perHeight/2),
        [2] = cc.p(leftPosX, perHeight + perHeight/2),
        [3] = cc.p(leftPosX, perHeight/2),
        [4] = cc.p(middlePosX, perHeight*2 + perHeight/2),
        [5] = cc.p(middlePosX, perHeight + perHeight/2),
        [6] = cc.p(middlePosX, perHeight/2),
    }
    local itemInfo = {
        [1] = {
            itemType = zc.dataBreakthroughStone,
            img = zc.getImgByType(zc.dataBreakthroughStone),
            name = zc.getNameById(zc.dataBreakthroughStone),
            count = self._resourceInfo[zc.getTypeByType(zc.dataBreakthroughStone)] or 0, 
        },
        [2] = {
            itemType = zc.dataTypeTax ,
            img = zc.getImgByType(zc.dataTypeTax ),
            name = zc.getNameById(zc.dataTypeTax ),
            count = self._resourceInfo[zc.getTypeByType(zc.dataTypeTax)] or 0, 
        },        
        [3] = {
            itemType = zc.dataServicePoints,
            img = zc.getImgByType(zc.dataServicePoints),
            name = zc.getNameById(zc.dataServicePoints),
            count = self._resourceInfo[zc.getTypeByType(zc.dataServicePoints)] or 0, 
        },        
        [4] = {
            img = "res/Main/bag/gem/1.png",
            name = LANGUAGE_SHOP.shop_expCard,
            count = upgrade_card_num or 0,
            scale = 40/110, 
        },       
        [5] = {
            img = zc.cardExtractPopSmallImg,
            name = cardExtractPopName,
            count = cardExtractPopNum,
        },  
    } 
    for i = 1, #itemInfo do
        local itemimg = zc.getSprite({
            src = itemInfo[i].img,
            ap = cc.p(0.5, 0.5),
            pos = cc.p(itemPos[i].x, itemPos[i].y)
        })
        _infoNode:addChild(itemimg)
        itemimg:setScale(0.9)
        if itemInfo[i].scale then
            itemimg:setScale(itemInfo[i].scale)
        end 
        local name = zc.createLabel({
            text = itemInfo[i].name,
            fontSize = 20,
            ap = cc.p(0.5, 0.5),
            color = self._fontColor,
            pos = cc.p(itemimg:getPositionX() + 55, itemimg:getPositionY()),
        })
        _infoNode:addChild(name)
        local count = zc.createLabel({
            text = itemInfo[i].count,
            fontSize = 20,
            ap = cc.p(0, 0.5),
            pos = cc.p(itemimg:getPositionX() + 115, itemimg:getPositionY()),
        })
        _infoNode:addChild(count)
    end
end
--第二个info信息
function SelfInfoPop:inintInfoSecond(_infoNode)

    local perHeight = _infoNode:getContentSize().height/3
    local leftPosX = 38
    local itemPos = {
        [1] = cc.p(leftPosX, perHeight*2+perHeight/2),
        [2] = cc.p(leftPosX, perHeight*1+perHeight/2), 
        [3] = cc.p(leftPosX, perHeight/2), 
    }
    local itemInfo = {
        [1] = {
            itemType = zc.dataMineral,
            img = zc.getImgByType(zc.dataMineral),
            name = zc.lang("kuangshi_chang"),
            count = zc.lang("nums") ..self._ironInfo.num,
            perHour = self._ironInfo.yield,
            max = self._ironInfo.numLimit,
        },
        [2] = {
            itemType = zc.dataTypeWood,
            img = zc.getImgByType(zc.dataTypeWood),
            name = zc.lang("mucai_chang"),
            count = zc.lang("nums") ..self._woodInfo.num,
            perHour = self._woodInfo.yield,
            max = self._woodInfo.numLimit,
        },
        [3] = {
            itemType = zc.dataTypeEnergy,
            img = zc.getImgByType(zc.dataTypeEnergy),
            name = zc.getNameById(zc.dataTypeEnergy),
            count = self._resourceInfo.energy.."/"..self._resourceInfo.energyLimit,
        },
    }
    for i = 1, #itemInfo do
        local itemimg = zc.getSprite({
            src = itemInfo[i].img,
            ap = cc.p(0.5, 0.5),
            pos = cc.p(itemPos[i].x, itemPos[i].y)
        })
        _infoNode:addChild(itemimg)
        itemimg:setScale(0.9)
        local name = zc.createLabel({
            text = itemInfo[i].name,
            color = self._fontColor,
            fontSize = 20,
            ap = cc.p(0, 0.5),
            pos = cc.p(itemimg:getPositionX()+20, itemimg:getPositionY() ),
        })
        _infoNode:addChild(name)
        local count = zc.createLabel({
            text = itemInfo[i].count,
            fontSize = 20,
            ap = cc.p(0, 0.5),
            pos = cc.p(itemimg:getPositionX()+100, itemimg:getPositionY() ),
        })
        _infoNode:addChild(count)
        if i == #itemInfo then  
            self._tiliNumLab = count
        end
        if i < #itemInfo then
           local per = zc.createLabel({
                text = itemInfo[i].perHour.."/" .. zc.lang("date_hours"),
                fontSize = 20,
                ap = cc.p(0, 0.5),
                pos = cc.p(itemimg:getPositionX()+170, itemimg:getPositionY() ),
            })
            _infoNode:addChild(per) 

            local container = zc.createLabel({
                text = zc.lang("totle_capacity"),
                fontSize = 20,
                ap = cc.p(0, 0.5),
                pos = cc.p(itemimg:getPositionX()+285, itemimg:getPositionY() ),
            })
            _infoNode:addChild(container) 
            
            local containerNum = zc.createLabel({
                text = itemInfo[i].max,
                fontSize = 20,
                ap = cc.p(0, 0.5),
                pos = cc.p(itemimg:getPositionX()+350, itemimg:getPositionY() ),
            })
            _infoNode:addChild(containerNum) 
        end
    end

    local titiTime = zc.createLabel({
        text = zc.lang("tili_recover_time"),
        fontSize = 18,
        ap = cc.p(0, 0.5),
        pos = cc.p(itemPos[1].x+210, _infoNode:getContentSize().height-90),
    })
    _infoNode:addChild(titiTime)


    local timestr_h_m_s = zc.getTimeHMS(self._time)
    local timeTili = zc.createLabel({
        text = timestr_h_m_s,
        fontSize = 18,
        ap = cc.p(0, 0.5),
        pos = cc.p(itemPos[1].x+325, titiTime:getPositionY() ),
    })
    _infoNode:addChild(timeTili)
    
    
    local totaleTiliTime = zc.createLabel({
        text = zc.lang("tili_all_recover"),
        fontSize = 18,
        ap = cc.p(0, 0.5),
        pos = cc.p(titiTime:getPositionX(), _infoNode:getContentSize().height-110),
    })
    _infoNode:addChild(totaleTiliTime)

    local timestr_h_m = zc.getTimeHMS(self._totalTime)
    local timeTiliTotal = zc.createLabel({
        text = timestr_h_m,
        fontSize = 18,
        ap = cc.p(0, 0.5),
        pos = cc.p(timeTili:getPositionX(), totaleTiliTime:getPositionY() ),
    })
    _infoNode:addChild(timeTiliTotal)

    --定时器
    schedule(self, function()
        self._time = self._time - 1
        self._totalTime = self._totalTime - 1
        if self._time < 0 then
            --体力恢复1点的操作
            print(" self._totalTime  ", self._totalTime)
            self._resourceInfo.energy = tonumber(self._resourceInfo.energy) + 1
            if self._totalTime < 0 then
                if self:getActionByTag(10000) then
                    self:stopActionByTag(10000)
                end
                if self._tiliNumLab then
                    self._tiliNumLab:setString(self._resourceInfo.energy.."/"..PlayerData:getResourcesInfo().energyLimit)
                end
                timeTili:setVisible(false)
                timeTiliTotal:setVisible(false)
            else
                if self._tiliNumLab then
                    self._tiliNumLab:setString(self._resourceInfo.energy.."/"..PlayerData:getResourcesInfo().energyLimit)
                end
                --体力增加1 修改
                self._time = self._interval - 1
            end
        end

        local timeStr = zc.getTimeHMS(self._time)
        timeTili:setString(timeStr)
        local totalTimeStr = zc.getTimeHMS(self._totalTime)
        timeTiliTotal:setString(totalTimeStr)
    end, 1.0, 10000)

    if self._time == 0 and self._totalTime == 0 then
        if self:getActionByTag(10000) then
            self:stopActionByTag(10000)
        end
        timeTiliTotal:setString("")
        timeTili:setString("")
        titiTime:setVisible(false)
        totaleTiliTime:setVisible(false)
    end
end

function SelfInfoPop:onCleanup()
    local textureCache = cc.Director:getInstance():getTextureCache()
    textureCache:removeTextureForKey("res/Main/setting/setting_bg.png")
    textureCache:removeTextureForKey("res/Main/setting/btn_move.png")
    textureCache:removeTextureForKey("res/Main/setting/btn_open_d.png")
    textureCache:removeTextureForKey("res/Main/setting/btn_open_u.png")
    textureCache:removeTextureForKey("res/Main/setting/changeName_btn.png")
    textureCache:removeTextureForKey("res/Main/setting/separate_line.png")
    textureCache:removeTextureForKey("res/Main/setting/setting_btn_h.png")
    textureCache:removeTextureForKey("res/Main/setting/setting_btn_n.png")
    textureCache:removeTextureForKey("res/Main/setting/wordBack.png")
    
    if self._callback then
        self._callback()
    end
end

function SelfInfoPop:onCleanup()
    zc.removeEventListener(CUSTOM_EVENT.EVENT_REFRESH_NAME_SELFINFO)
    zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_REFRESH_NAME_MAINCIGY})
end

return SelfInfoPop