--[[
    FileName: WeaponCastLayer.lua
    Author: wangming
    Date: 2016-12-20
    Purpose: 兵器洗练
]]
local WeaponCastAttrPop = class("WeaponCastAttrPop", function()
	return ZCPopLayer:create({opacityValue = 255*0.65})
end)

function WeaponCastAttrPop:create(params)
	local _pop = WeaponCastAttrPop.new(params)
	_pop:show()
	return _pop
end

function WeaponCastAttrPop:ctor(params)
	local _weaponDatas = {}
	_weaponDatas[1] = params.beforeData or {}
	_weaponDatas[2] = params.afterData or {}
	_weaponDatas[2].color = _weaponDatas[1].color
	_isPreview = params.isPreview
	local _level = tonumber(params.level) or 0

	local _changeInfos = {}
	for i=1,6 do
		local _num1 = tonumber(_weaponDatas[1]["effectValue" .. i]) or 0
		local _num2 = tonumber(_weaponDatas[2]["effectValue" .. i]) or 0
		if _num1 ~= _num2 then
			local _effId = tonumber(_weaponDatas[1]["effectInfo" .. i].effectId) or 0
			table.insert(_changeInfos, {
				num1 = zc.getAttributeNumById(_effId, _num1), 
				num2 = zc.getAttributeNumById(_effId, _num2),
				effId = _effId,
			})
		end
	end

	local _width = 420
	local _height = 380
	local _bg = nil
	local _y_ = _height - 90
	-- 背景
	if _isPreview then
		_bg = ccui.Scale9Sprite:create(cc.rect(8, 8, 4, 4), "res/Main/role/fetter/popDi.png")
		_bg:setContentSize(_width, _height)
		_bg:setPosition(zc.width*0.5, zc.height*0.5)
		self:addContent(_bg)

		-- 标题
		local _title = zc.createSprite("res/Main/smithy/weaponCast/title.png")
		_title:setAnchorPoint(0.5, 0)
		_title:setPosition(_width*0.5, _height - 11)
		_bg:addChild(_title)
	else
		_width = 620
		_height = 428
		_bg = ccui.Scale9Sprite:create(cc.rect(200, 60, 635, 263), "res/Main/copy/copy_dibian.png")
		_bg:setContentSize(_width, _height)
		_bg:setPosition(zc.width*0.5, zc.height*0.5 - 100)
		self:addChild(_bg)
		_y_ = _height - 140

		local _spine = zc.createSpine({
			fileName = "duanzao_chenggong",
			actName = "start",
			needInit = false,
			loop = false,
		})
		_spine:setPosition(_width*0.5, _height + 50)
		_bg:addChild(_spine, 1)
		_spine:registerSpineEventHandler( function ( event )
			local name  = event.animation
			if name == "start" then
				_spine:playAction("idle", true)
			end
		end, sp.EventType.ANIMATION_COMPLETE)
	end


	local _arrow = zc.createSprite("res/Main/role/equip/arrow_2.png")
    _arrow:setPosition(_width*0.5, _y_)
    _bg:addChild(_arrow)
    self._arrow = _arrow

    self._icons = {}
	for i=1, 2 do
		local _y = _y_
		local _x = _width*0.5 - 120 + (i - 1)*240
		local _node = cc.Node:create()
		_node:setPosition(_x, _y)
		_bg:addChild(_node)
        self._icons[i] = _node
		local _di = zc.createSprite("res/Main/smithy/weaponCast/role_magic_bg.png")
		_di:setScale(0.7)
		_di:setOpacity(255*0.5)
		_node:addChild(_di)
		local _data = _weaponDatas[i]
		local _icon, _info = zc.createIconByData({
            data = {
                mainType = zc.mainTypeWeapon,
                type = 1, 
                id = _data.weaponId,
                quality = _data.color,
                grade = _data.grade,
                level = (zc.getLevelTipShow().._level),
            },
            showLevel = true,
        })
        _icon:setScale(0.65)
        _node:addChild(_icon)

        local _name = _info.name
        local _grade = tonumber(_data.grade) or 0
        if _grade > 0 then
        	_name = _name .. "+" .. _grade
        end
        local _nameLab = zc.createLabel({
        	text = _name,
        	fontSize = 20,
        	color = zc.getColorByQuality(_data.color),
        	pos = cc.p(0, -80)
        })
        _node:addChild(_nameLab)
	end

	-- for i=2, 4 do
	-- 	_changeInfos[i] = _changeInfos[1]
	-- end

	local _h = 45
	local _y = _y_ - 200 + (#_changeInfos - 1)/2*_h
	local _x = -160

	if not _isPreview then
		local _light = zc.createSprite("res/Main/smithy/weaponCast/light.png")
		_light:setPosition(_width*0.5, _y_ - 100)
		_bg:addChild(_light)
		self._light = _light
	end
	self._values = {}
	self._riches = {}
    for i=1, #_changeInfos do
		local _node = cc.Node:create()
		_node:setPosition(_width*0.5, _y)
		_bg:addChild(_node)
		self._values[i] = _node

    	if i%2 == 1 then
	    	local _di = zc.createSprite("res/Main/smithy/weaponCast/infoDi.png")
	    	_node:addChild(_di)
	    end

    	local _name = LANGUAGE_getAttributeTextById(_changeInfos[i].effId)
    	local _nameLab = zc.createRichLabel({
			text = _name .. ":",
			fontSize = 20,
			anchor = cc.p(0, 0.5),
			pos = cc.p(_x, 0),
		})
		_nameLab:setGradientColor(cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1))
		_node:addChild(_nameLab)
		self._riches[i] = _nameLab
		local _numLab1 = zc.createLabel({
			text = _changeInfos[i].num1,
			color = cc.c3b(231, 201, 85),
			fontSize = 20,
			pos = cc.p(_x + 140, 0),
		})
		_node:addChild(_numLab1)
        local arrowImg = zc.getSprite({
            src = "res/Main/common/arrow_green.png",
			pos = cc.p(_x + 206, 0),
        })
        _node:addChild(arrowImg)

		local _numLab2 = zc.createLabel({
			text = _changeInfos[i].num2,
			color = cc.c3b(124, 255, 96),
			fontSize = 20,
			pos = cc.p(_x + 300, 0),
		})
		_node:addChild(_numLab2)

    	_y = _y - _h
    end

	if not _isPreview then
		self:getContainerLayer():setTouchEndedCallback(function() end)
		self._arrow:setOpacity(0)
		self._light:setOpacity(0)
		for k,v in pairs(self._values) do
			v:setVisible(false)
		end
		self._icons[2]:setVisible(false)

		zc.doFuncForAllChild(self._icons[1], function( child )
			local _nowOpacity = child:getOpacity()
			child:setOpacity(0)
			local _array = {}
			_array[#_array + 1] = cc.DelayTime:create(0.3)
			_array[#_array + 1] = cc.FadeTo:create(0.3, _nowOpacity)
			if child == self._icons[1] then
				_array[#_array + 1] = cc.CallFunc:create(function()
					self._arrow:setScale(10)
					self._arrow:runAction(cc.Sequence:create(
						cc.Spawn:create(
			                cc.EaseOut:create(cc.ScaleTo:create(0.2, 0.9), 3),
			                cc.FadeIn:create(0.2)
			            ),
			            cc.ScaleTo:create(0.05, 1.2),
			            cc.ScaleTo:create(0.02, 0.95),
			            cc.ScaleTo:create(0.01, 1),
						cc.CallFunc:create(function()
							self._icons[2]:setVisible(true)
							zc.doFuncForAllChild(self._icons[2], function( child2 )
								local _nowOpacity2 = child2:getOpacity()
								child2:setOpacity(0)
								local _array2 = {}
								_array2[#_array2 + 1] = cc.FadeTo:create(0.3, _nowOpacity2)
								if child2 == self._icons[2] then
									_array2[#_array2 + 1] = cc.CallFunc:create(function()
										local armature = zc.createAnimationArmature({
			                                fileName = "ef_mingge",
			                                autoRemove = true,
			                            })
			                            armature:getAnimation():playWithIndex(0)
			                            armature:setScale(0.85)
			                            self._icons[2]:addChild(armature)
			                            self._light:runAction(cc.Sequence:create(
			                            	cc.FadeTo:create(0.2, 255),
			                            	cc.DelayTime:create(#self._values*0.2),
			                            	cc.CallFunc:create(function()
			                            		self:getContainerLayer():setTouchEndedCallback(function()
			                            			self:hide()
			                            		end)
												local _currentGID = getGuideManager():getCurrentGID()
												if _currentGID == GuideKey.KEY_14208 then
										            getGuideManager():setHeroFuncCallBack(function()
										            	getGuideManager():setFuncGuideOver()
										            end)
										            getGuideManager():startGuide()
												end
			                            	end)
										))
			                            for k,v in pairs(self._values) do
			                            	v:setVisible(true)
											zc.doFuncForAllChild(v, function( child3 )
												if child3 == self._riches[k] then
													child3:setVisible(false)
													performWithDelay(child3, function()
														child3:setVisible(true)
													end, 0.2 + 0.1*tonumber(k))
												else
													local _nowOpacity3 = child3:getOpacity()
													child3:setOpacity(0)
													performWithDelay(child3, function()
														child3:runAction(cc.FadeTo:create(0.3, _nowOpacity3))
													end, 0.2 + 0.1*tonumber(k))
												end
											end)
			                            end
									end)
								end
								child2:runAction(cc.Sequence:create(_array2))
							end)
						end)
					))
				end)
			end
			child:runAction(cc.Sequence:create(_array))
		end)
	end
end

return WeaponCastAttrPop