--[[
	名将副本章节界面
	唐实聪
	2016.4.5
]]
local TollgateHeroMainLayer = class("TollgateHeroMainLayer", function()
	return ZCDialog:create()
end)

function TollgateHeroMainLayer:create( params )
	return TollgateHeroMainLayer.new( params )
end

function TollgateHeroMainLayer:ctor( params )
	self:initData( params )
	self:initUI()

    self._tollgateTableview:scrollToCell( self._curPage )
    self:scorllviewScriptScroll( self._tollgateTableview )
end

function TollgateHeroMainLayer:onEnter()
    if not self:isVisible() then
        return
    end
    if self._refresh then
        if self._refresh == 1 then
            local _lastSceneData = PlayerData:getLastSceneData()
            -- dump(_lastSceneData,"_lastSceneData")
            PlayerData:setLastSceneData({})
            if _lastSceneData.lastScene and _lastSceneData.lastScene == "TollgateHero" then
                if _lastSceneData.battleStatus then
                    -- 已挑战次数
                    self._backData.challengeNum = _lastSceneData.challengeNum
                    -- 赢了刷新
                    local chapterId = tonumber( _lastSceneData.chapterId ) or 0
                    local tollgateId = tonumber( _lastSceneData.tollgateId ) or 0

                    local _chapterFlag = false
                    for i, v in ipairs( self._backData.chapters ) do
                        local _chapterId = tonumber( v.chapterId ) or 0
                        if chapterId == _chapterId then
                            _chapterFlag = true
                            local _tollgateFlag = false
                            for j, u in ipairs( v.tollgates or {} ) do
                                local _tollgateId = tonumber( u.tollgateId ) or 0
                                if _tollgateId == tollgateId then
                                    _tollgateFlag = true
                                    u.num = 1
                                end
                            end
                            if not _tollgateFlag then
                                v.tollgates = v.tollgates or {}
                                v.tollgates[#v.tollgates + 1] = {
                                    tollgateId = tollgateId,
                                    num = 1,
                                }
                            end
                            break
                        end
                    end
                    if not _chapterFlag then
                        self._backData.chapters[#self._backData.chapters + 1] = {
                            chapterId = chapterId,
                            drawPass = 0,
                            tollgates = {
                                {
                                    tollgateId = tollgateId,
                                    num = 1,
                                }
                            },
                        }
                    end
                    self:buildTollgateData( self._backData )
                    self:refresh()
                end
            end
        elseif self._refresh == 2 then
            self:buildTollgateData( self._backData )
            self:refresh()
        end
        self._refresh = nil
    end
    --功能引导
    self:addGuide()
end

function TollgateHeroMainLayer:onCleanup()
    ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo( "res/Main/battle/battleresult/win/win.csb", true )
    ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo( "res/animation/zhangjietongguan/zhangjietongguan.csb", true )
    local _forms = PlayerData:getNowFormationFiles()
    for k, v in pairs( self._record ) do
        -- 如果已上阵，不删除
        local _path, _resId = zc.battle.getHeroArmaturePathByResId( k )
        if not _forms[_resId] then
            ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo( _path )
        end
    end
    self:removeAllChildren()
    helper.collectMemory()
end

function TollgateHeroMainLayer:initData( params )
    -- 关卡数据，静态数据，只需要取一次
    self._tollgateList = {}
    -- 章节数据
    self:buildTollgateData( params or {} )
    
	self._size = self:getContentSize()

    self._record = {}
end
-- 构造数据
function TollgateHeroMainLayer:buildTollgateData( data )
    -- dump(data,"buildTollgateData")
    -- 备份数据
    self._backData = data
    -- 已挑战次数
    self._challengeNum = tonumber( data.challengeNum ) or 0
    -- 挑战次数上限
    self._challengeNumLimit = tonumber( data.challengeNumLimit ) or 0
    -- 查询当前正在打的章节
    local chapterData = {}
    local curChapter = 0
    for i, v in ipairs( data.chapters or {} ) do
        local _chapter = tonumber( v.chapterId ) or 0
        if v.tollgates and #v.tollgates > 0 then
            chapterData[_chapter] = v
            if _chapter > curChapter then
                curChapter = _chapter
            end
        end
    end
    -- 如果已打通，当前正在打下个副本
    do
        local _chapterData = chapterData[tonumber( curChapter )]
        if _chapterData and _chapterData.tollgates then
            local _tollgateList = self:getTollgateListByChapterId( curChapter )
            if #_chapterData.tollgates == #_tollgateList then
                -- 打通当前副本
                curChapter = curChapter + 1
            end
        end
    end
    -- 记录当前打的关卡，第一次进入时滑到对应位置
    curChapter = curChapter == 0 and 1 or curChapter
    -- print("curChaptercurChaptercurChapter========",curChapter)
    self._curPage = curChapter - 1

    -- 判断总显示章节
    -- 总章节
    local maxChapter = curChapter + 1
    -- 置灰章节数
    local grayChapter = 1
    do
        local _chapterList = gameData.getDataFromCacheWithPrimaryKey("hero_chapter", curChapter)
        if _chapterList and tonumber( _chapterList.awardsId ) == 1 then
            -- 当前正在打奖励关卡
            maxChapter = curChapter + 2
            grayChapter = 1
        else
            local _pIdx = (tonumber(curChapter) or 0) + 1
            _chapterList = gameData.getDataFromCacheWithPrimaryKey("hero_chapter", _pIdx)
            if _chapterList and tonumber( _chapterList.awardsId ) == 1 then
                -- 下一关是奖励关卡
                maxChapter = curChapter + 2
                grayChapter = 2
            end
        end
    end
    -- print("maxChapter",maxChapter)
    -- print("grayChapter",grayChapter)

    -- 组织数据
    self._chapterData = {}
    local function _do( i, isGray )
        local _data = {}
        -- 后端数据
        for k, v in pairs( chapterData[tonumber( i )] or {} ) do
            _data[k] = v
        end
        -- 静态表
        local _chapterList = gameData.getDataFromCacheWithPrimaryKey("hero_chapter", i)
        for k, v in pairs( _chapterList ) do
            _data[k] = v
        end
        _data.isGray = isGray
        self._chapterData[i] = _data
    end
    for i = 1, maxChapter - grayChapter do
        _do(i, false)
    end
    for i = maxChapter - grayChapter + 1, maxChapter do
        _do(i, true)
    end
end
-- 根据章节id取关卡数据
function TollgateHeroMainLayer:getTollgateListByChapterId( _chapterId )
    local _tollgateList = self._tollgateList[tonumber( _chapterId )]
    -- 已经取过数据，直接返回
    if _tollgateList then
        return _tollgateList
    end
    -- 没有取过数据
    _tollgateList = gameData.getDataFromCache("hero_tollgate", {chapterId = _chapterId})
    if #_tollgateList == 0 and next( _tollgateList ) then
        _tollgateList = { _tollgateList }
    end
    table.sort(_tollgateList, function( a, b )
        return tonumber( a.id ) < tonumber( b.id )
    end)
    self._tollgateList[tonumber( _chapterId )] = _tollgateList
    return _tollgateList
end

function TollgateHeroMainLayer:initUI()
	-- 背景
    local background = cc.Sprite:create( "res/Main/tollgateHero/main_bg.jpg" )
    background:setPosition( self._size.width/2, self._size.height/2 )
    self:addChild( background )
    zc.setBgSize( background )
    -- 云
    zc.getYunDongTai(nil,3,function(armUp, armDown)
        if not tolua.isnull(self) then
            self:addChild( armUp )
            self:addChild( armDown )
        end
    end)

	-- 标题
	local title = cc.Sprite:create( "res/Main/tollgateHero/title.png" )
	title:setAnchorPoint( 0, 1 )
	title:setPosition( 0, self._size.height )
	self:addChild( title, 1 )
	-- 关闭按钮
	local close = zc.createTopBackButton({
		listener = function ()
			LayerManager.removeLayout()
		end
	})
	self:addChild( close, 1 )

	-- tableview
	local tableviewSize = cc.size( self._size.width*1.5, self._size.height )
	local tollgateTableview = zc.ZCTableView:create( tableviewSize )
	tollgateTableview:setAutoAlign( true )
    tollgateTableview:setBounceable( true )
    tollgateTableview:setInertia( true )
    tollgateTableview:setDirection( cc.SCROLLVIEW_DIRECTION_HORIZONTAL )
    tollgateTableview:setPosition( -self._size.width/4, 0 )
    tollgateTableview:setDelegate()
    tollgateTableview:setVerticalFillOrder( cc.TABLEVIEW_FILL_TOPDOWN )
    -- 配置
    local cellWidth = self._size.width/2
    local cellHeight = self._size.height

    local function scorllviewScriptScroll( table )
        self:scorllviewScriptScroll( table )
    end

    local function cellSizeForTable( table, index )
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex( table, index )
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
        end

        self:createCell( cell, index, cellWidth, cellHeight )
        

        return cell
    end
    local function numberOfCellsInTableView()
    	return #self._chapterData + 2
    end
    local function tablePageAfterSliding( table, cell )
    end
    tollgateTableview:registerScriptHandler( cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX )
    tollgateTableview:registerScriptHandler( tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX )
    tollgateTableview:registerScriptHandler( numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW )
    tollgateTableview:registerScriptHandler( tablePageAfterSliding, zctech.TABLECELL_AFTER_SLIDING )
    tollgateTableview:registerScriptHandler( scorllviewScriptScroll, zctech.SCROLLVIEW_SCRIPT_SCROLL )
    self:addChild( tollgateTableview )
    tollgateTableview:reloadData()
    self._tollgateTableview = tollgateTableview

    -- 今日可挑战背景
    local challengeBg = cc.Sprite:create( "res/Main/copy/leftTimesBg.png" )
    challengeBg:setPosition( 100, 27 )
    self:addChild( challengeBg, 2 )
    -- 今日可挑战文字
    local challengeText = zc.createLabel({
        text = LANGUAGE_TOLLGATEHERO.challenge_times_today..":",
        fontSize = 20,
        stroke = true,
        pixel = 2,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 30, challengeBg:getContentSize().height/2 ),
    })
    challengeBg:addChild( challengeText )
    -- 今日可挑战数字
    local leftTimes = tonumber( self._challengeNumLimit ) - tonumber( self._challengeNum )
    leftTimes = leftTimes > 0 and leftTimes or 0
    local challengeNum = zc.createLabel({
        text = leftTimes.."/"..self._challengeNumLimit,
        fontSize = 23,
        color = display.COLOR_GREEN,
        outLine = true,
        pixel = 1.5,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( challengeText:getPositionX() + challengeText:getContentSize().width + 7, challengeText:getPositionY() - 2 ),
    })
    challengeBg:addChild( challengeNum )
    self._challenge = challengeNum
end
-- 创建cell
function TollgateHeroMainLayer:createCell( cell, index, cellWidth, cellHeight )
    -- 清理
    cell:removeAllChildren()
    cell._cellBg = nil
    cell._grayTable = {}
    cell._icon = {}

    local data = self._chapterData[index]
    if data then
        local _tollgateList = self:getTollgateListByChapterId( data.id )
        if tonumber( data.awardsId or 0 ) == 1 then
            -- 特殊关卡
            local cellBg = cc.Sprite:create( "res/Main/tollgateHero/tollgateBg_2.png" )
            cellBg:setPosition( cellWidth/2, cellHeight/2 - 50 )
            cellBg:setCascadeOpacityEnabled( true )
            cell:addChild( cellBg )
            cell._cellBg = cellBg
            cell._grayTable[#cell._grayTable + 1] = cellBg
            -- 标题
            local title = cc.Sprite:create( "res/Main/tollgateHero/tollgateTitle_2.png" )
            title:setPosition( cellBg:getContentSize().width/2, cellBg:getContentSize().height - 20 )
            cellBg:addChild( title )
            cell._grayTable[#cell._grayTable + 1] = title
            local _first = 1
            if data.tollgates and #_tollgateList == #data.tollgates then
                -- 通关
                local passBg = cc.Sprite:create( "res/Main/tollgateHero/passBg.png" )
                passBg:setPosition( 40, 110 )
                cellBg:addChild( passBg, 1 )
                passBg:runAction(
                    cc.RepeatForever:create(
                        cc.RotateBy:create(
                            5, 360
                        )
                    )
                )
                cell._grayTable[#cell._grayTable + 1] = passBg
                local pass = cc.Sprite:create( "res/Main/tollgateHero/pass.png" )
                pass:setPosition( passBg:getPosition() )
                cellBg:addChild( pass, 1 )
                cell._grayTable[#cell._grayTable + 1] = pass
                _first = 0
            end
            -- 名字
            local nameBg = cc.Sprite:create( "res/Main/tollgateHero/nameBg.png" )
            nameBg:setAnchorPoint( 0, 1 )
            nameBg:setPosition( -10, cellBg:getContentSize().height + 30 )
            cellBg:addChild( nameBg )
            cell._grayTable[#cell._grayTable + 1] = nameBg
            local namePath = string.format( "res/Main/tollgateHero/name/%s.png", data.icon )
            if not zc.checkFileExist( namePath )  then
                namePath = "res/Main/tollgateHero/name/1.png"
            end
            local nameSp = zc.createSprite( namePath )
            nameSp:setAnchorPoint( 0.5, 1 )
            nameSp:setPosition( nameBg:getContentSize().width/2, nameBg:getContentSize().height - 20 )
            nameBg:addChild( nameSp )
            cell._grayTable[#cell._grayTable + 1] = nameSp
            -- 奖励
            local awardList = string.split( _tollgateList[1].firstDrop or "", "," )
            local awardData = {}
            for i, v in ipairs( awardList ) do
                awardData[i] = gameData.getDataFromCacheWithPrimaryKey("drop_award", v)
            end
            for i, v in ipairs( awardData ) do
                local icon = zc.createIconByData({
                    data = {
                        mainType = v.itemType,
                        type = v.itemId, 
                        id = v.itemId,
                        -- quality = 1,
                        num = v.itemNum,
                    }, 
                    pos = cc.p( cellBg:getContentSize().width/2 + 100*( i - 0.5 - #awardData/2 ), 65 ), 
                })
                icon:setScale( 0.6 )
                icon:setCascadeOpacityEnabled( true )
                cellBg:addChild( icon )
                cell._icon[#cell._icon + 1] = icon
            end
            -- 按钮
            local btn = zc.createPushButton({
                touchSize = cell._cellBg:getContentSize(),
                pos = cc.p( cell._cellBg:getContentSize().width/2, cell._cellBg:getContentSize().height/2 ),
                needEnableWhenMoving = true,
                needSwallow = false,
                endCallback = function()
                    if self._tollgateTableview:getCurrentPage() + 1 == index then
                        if not data.isGray then
                            if _first == 0 then
                                -- 只可以打一次
                                zc.tipInfo( zc.lang("tips_reward_fight_can_only_attack_once") )
                                return
                            end
                            do
                                local _energy = tonumber( _tollgateList[1].expend ) or 0
                                local _myEnergy = tonumber( PlayerData.resourcesInfo.energy ) or 0
                                if _myEnergy < _energy then
                                    zc.tipInfo( zc.lang("tips_tili_unenough") )
                                    return
                                end
                            end
                            self._refresh = 1
                            local _tollgateId = _tollgateList[1].id
                            -- 布阵界面显示第几波怪物
                            local key = 1
                            local enemy = gameData.getDataFromCache("hero_npcGroup", {tollgateId = _tollgateId})
                            if enemy and enemy.id then
                            else
                                if enemy[key] then
                                    enemy = enemy[key]
                                end
                            end
                            local enemys = {}
                            for i = 1, 9 do
                                if enemy["location"..i] and tonumber(enemy["location"..i]) and tonumber( enemy["location"..i] ) > 0 then
                                    enemys[i] = gameData.getDataFromCacheWithPrimaryKey( "hero_npc", enemy["location"..i])
                                else
                                    enemys[i] = {}
                                end
                            end
                            local _dataInfo = {
                                battleType= BattleType.TollgateHero,
                                enemys = {enemys = enemys},
                                topinfo = false,
                                tollgateId = _tollgateId,
                                isFirstFight = _first,
                                chapterId = data.id,
                            }
                            zc.createMatrixBattle( _dataInfo )
                        end
                    else
                        self._tollgateTableview:scrollToCell( index - 1, true )
                    end
                end
            })
            cell._cellBg:addChild( btn )
            cell._btn = btn
        else
            -- 普通关卡
            local cellBg = cc.Sprite:create( "res/Main/tollgateHero/tollgateBg_1.png" )
            cellBg:setPosition( cellWidth/2, cellHeight/2 - 80 )
            cellBg:setCascadeOpacityEnabled( true )
            cell:addChild( cellBg )
            cell._cellBg = cellBg
            cell._grayTable[#cell._grayTable + 1] = cellBg
            if data.tollgates and #_tollgateList == #data.tollgates then
                -- 通关
                local passBg = cc.Sprite:create( "res/Main/tollgateHero/passBg.png" )
                passBg:setPosition( 90, 235 )
                cellBg:addChild( passBg, 1 )
                passBg:runAction(
                    cc.RepeatForever:create(
                        cc.RotateBy:create(
                            5, 360
                        )
                    )
                )
                cell._grayTable[#cell._grayTable + 1] = passBg
                local pass = cc.Sprite:create( "res/Main/tollgateHero/pass.png" )
                pass:setPosition( passBg:getPosition() )
                cellBg:addChild( pass, 1 )
                cell._grayTable[#cell._grayTable + 1] = pass
            end
            -- 英雄
            local heroPath = string.format( "res/Main/hero/body/%s.png", data.icon )   
            if not zc.checkFileExist( heroPath )  then
                heroPath = "res/Main/hero/body/1.png"
            end
            local heroSp = zc.createSprite( heroPath )
            heroSp:setAnchorPoint( 1, 0 )
            heroSp:setPosition( cellBg:getContentSize().width, 15 )
            cellBg:addChild( heroSp )
            cell._grayTable[#cell._grayTable + 1] = heroSp
            -- 关卡标志
            local _passData = {}
            for i, v in ipairs( data.tollgates or {} ) do
                _passData[tonumber( v.tollgateId )] = true
            end
            for i, v in ipairs( _tollgateList ) do
                if _passData[tonumber( v.id )] then
                    local iconBg = cc.Sprite:create( "res/Main/tollgateHero/iconBg_light.png" )
                    iconBg:setPosition( iconBg:getContentSize().width/2*i, i%2 == 1 and 90 or 122 )
                    iconBg:setCascadeOpacityEnabled( true )
                    cellBg:addChild( iconBg )
                    cell._grayTable[#cell._grayTable + 1] = iconBg
                    local icon = cc.Sprite:create( "res/Main/tollgateHero/icon.png" )
                    icon:setPosition( iconBg:getContentSize().width/2 + 4, iconBg:getContentSize().height/2 + 7 )
                    iconBg:addChild( icon )
                    cell._grayTable[#cell._grayTable + 1] = icon
                else
                    local iconBg = cc.Sprite:create( "res/Main/tollgateHero/iconBg_dark.png" )
                    iconBg:setPosition( iconBg:getContentSize().width/2*i, i%2 == 1 and 90 or 122 )
                    cellBg:addChild( iconBg )
                    cell._grayTable[#cell._grayTable + 1] = iconBg
                end
            end
            -- 名字
            local nameBg = cc.Sprite:create( "res/Main/tollgateHero/nameBg.png" )
            nameBg:setPosition( 90, cellBg:getContentSize().height )
            cellBg:addChild( nameBg )
            cell._grayTable[#cell._grayTable + 1] = nameBg
            local namePath = string.format( "res/Main/tollgateHero/name/%s.png", data.icon )   
            if not zc.checkFileExist( namePath )  then
                namePath = "res/Main/tollgateHero/name/1.png"
            end
            local nameSp = zc.createSprite( namePath )
            nameSp:setAnchorPoint( 0.5, 1 )
            nameSp:setPosition( nameBg:getContentSize().width/2, nameBg:getContentSize().height - 20 )
            nameBg:addChild( nameSp )
            cell._grayTable[#cell._grayTable + 1] = nameSp
            -- 按钮
            local btn = zc.createPushButton({
                touchSize = cell._cellBg:getContentSize(),
                pos = cc.p( cell._cellBg:getContentSize().width/2, cell._cellBg:getContentSize().height/2 ),
                needEnableWhenMoving = true,
                needSwallow = false,
                endCallback = function()
                    if self._tollgateTableview:getCurrentPage() + 1 == index then
                        if not data.isGray then
                            --功能引导
                            local _currentGID = getGuideManager():getCurrentGID()
                            if _currentGID == GuideKey.KEY_11607 then
                                zc.setGuideFuncNextStep(_currentGID)
                                self._tollgateTableview:setTouchEnabled(true)
                            end
                            self._refresh = 2
                            zc.createSourceScene("TollgateHeroChapter", "src/layer/tollgateHero/TollgateHeroChapterLayer.lua", {
                                list = _tollgateList,
                                data = data,
                                challengeNum = self._challengeNum,
                                challengeNumLimit = self._challengeNumLimit,
                                record = self._record,
                                callback = function( backData )
                                    -- dump(backData,"backData")
                                    -- 已挑战次数
                                    self._backData.challengeNum = backData.challengeNum
                                    -- 赢了刷新
                                    local chapterId = tonumber( backData.data.chapterId ) or 0
                                    local _chapterFlag = false
                                    for i, v in ipairs( self._backData.chapters ) do
                                        local _chapterId = tonumber( v.chapterId ) or 0
                                        if chapterId == _chapterId then
                                            _chapterFlag = true
                                            v.tollgates = backData.data.tollgates
                                            v.drawPass = backData.data.drawPass
                                            break
                                        end
                                    end
                                    if not _chapterFlag then
                                        self._backData.chapters[#self._backData.chapters + 1] = {
                                            chapterId = chapterId,
                                            drawPass = backData.data.drawPass,
                                            tollgates = backData.data.tollgates,
                                        }
                                    end
                                end
                            })
                        end
                    else
                        self._tollgateTableview:scrollToCell( index - 1, true )
                    end
                end
            })
            btn:setContentSize(cell._cellBg:getContentSize())
            cell._cellBg:addChild( btn )
            cell._btn = btn
        end
        if data.isGray then
            for i, v in ipairs( cell._grayTable ) do
                zc.setGray( v )
            end
            for i, v in ipairs( cell._icon ) do
                zc.setGray( v )
                zc.setGray( v:getChildByName( "pic" ) )
            end
        else
            --功能引导
            if index == 1 then
                self._guide_btn = cell._btn
            end
        end
    end
end
function TollgateHeroMainLayer:scorllviewScriptScroll( table )
    local cellWidth = self._size.width/2
    local scaleMax = 0.85
    local scaleMin = 0.55
    local opacityMax = 255
    local opacityMin = 100
    local offsetTotal = 100

    local curX = table:getContentOffset().x
    local midPage = table:getCurrentPage() + 1
    for i = -2, 2 do
        local cell = table:cellAtIndex( midPage + i )
        if cell and cell._cellBg then
            -- 计算相对于中间位置的cell的偏移量
            local part = math.abs( cell:getPositionX() - cellWidth + curX )/cellWidth
            -- 缩放
            local scale = ( 1 - part )*( scaleMax - scaleMin ) + scaleMin
            cell._cellBg:setScale( scale )
            -- 透明度
            local opacity = ( 1 - part )*( opacityMax - opacityMin ) + opacityMin
            cell._cellBg:setOpacity( opacity )
            -- 位置偏移
            part = part - 0.5
            local offsetX = 0
            if part > 0 then
                offsetX = 2*part*offsetTotal*( -i )
            end
            cell._cellBg:setPositionX( cellWidth/2 + offsetX )
            -- -2和2位置的cell只设置缩放和透明度，不显示，这样显示的时候不会突然设置缩放和透明度
            if i == -2 or i == 2 then
                cell._cellBg:setVisible( false )
            else
                cell._cellBg:setVisible( true )
            end
        end
    end
end
-- 刷新
function TollgateHeroMainLayer:refresh()
    self._tollgateTableview:reloadDataAndScrollToCurrentCell()
    self._tollgateTableview:scrollToCell( self._curPage, true )
    self:scorllviewScriptScroll( self._tollgateTableview )
    local leftTimes = tonumber( self._challengeNumLimit ) - tonumber( self._challengeNum )
    leftTimes = leftTimes > 0 and leftTimes or 0
    self._challenge:setText( leftTimes.."/"..self._challengeNumLimit )
end

--新手引导--功能引导
function TollgateHeroMainLayer:addGuide()
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_11605 then
        zc.setGuideFuncNextStep(_currentGID)
        self:addGuide()
    elseif inArray(_currentGID, GuideGroup.DUI_HUA) then
        local function _callBack()
            self._tollgateTableview:setTouchEnabled(false)
            getGuideManager():setObjTarget(self._guide_btn)
            getGuideManager():startGuide()
        end
        getGuideManager():setHeroFuncCallBack(_callBack)
        getGuideManager():startGuide()
    end 
end

return TollgateHeroMainLayer