--[[
    TollgateHeroRewardPreviewPop.lua
    唐实聪
    2016.6.1儿童节
]]
local TollgateHeroRewardPreviewPop = class( "TollgateHeroRewardPreviewPop", function ()
    return ZCPopLayer:create()
end)
function TollgateHeroRewardPreviewPop:create(params)
    return TollgateHeroRewardPreviewPop.new(params)
end
function TollgateHeroRewardPreviewPop:ctor(params)
    -- dump(params, "params=====")
    self:initData(params)
    self:initUI()
    self:show()
end
function TollgateHeroRewardPreviewPop:initData(params)
    self._rewardData = {}
    local chapterId = params.chapterId
    local awards = gameData.getDataFromCacheWithPrimaryKey("hero_chapter", chapterId).awards
    self._rewardData[#self._rewardData + 1] = gameData.getDataFromCacheWithPrimaryKey("drop_award", awards)
    -- 是否可以领取
    self._fetchable = params.fetchable or false
    -- 是否已经领取
    if tonumber( params.drawPass ) == 1 then
        self._fetched = true
    else
        self._fetched = false
    end
end
function TollgateHeroRewardPreviewPop:initUI()
    local popBg = cc.Sprite:create("res/Main/wushenTower/pop_reward.png")
    popBg:setPosition(zc.width/2, zc.height/2)
    self:addContent(popBg)
    self._popBg = popBg
    local close = zc.createBackButton({
        pos = cc.p(popBg:getContentSize().width-20, popBg:getContentSize().height-10),
        listener = function () 
            self:hide()
        end,
    })
    popBg:addChild(close)

    self:createTableView()

    local getBtn = zc.createCommomBtn({
        tag = 1,
        text = "",
        needSwallow = false,
        endCallback = function()
            print("clicked ... ======================")
        end,
        anchor = cc.p(0.5, 0.5),
        pos = cc.p(popBg:getContentSize().width/2, 65)
    })
    popBg:addChild(getBtn)

    local btnText = ""
    local btnColor = display.COLOR_DEEPGRAY
    if self._fetched then
        getBtn:setEnable(false)
        getBtn:setText(LANGUAGE_TOLLGATEHERO.fetched)
    elseif self._fetchable then
        getBtn:setText(LANGUAGE_TOLLGATEHERO.can_fetch)
    else
        getBtn:setEnable(false)
        getBtn:setText(zc.lang("un_finish"))
    end
end
function TollgateHeroRewardPreviewPop:createTableView()
    local tableSize = cc.size(435, 200)
    local tableNode = self._popBg
    local myTableBg = ccui.Scale9Sprite:create(cc.rect(26,25,1,1), "res/Main/wushenTower/pop_tableBg.png")
    myTableBg:setContentSize(tableSize)
    myTableBg:setAnchorPoint(cc.p(0.5, 0.5))
    myTableBg:setPosition(cc.p(tableNode:getContentSize().width/2, tableNode:getContentSize().height/2 + 20))
    tableNode:addChild(myTableBg)
    local myTable = zc.ZCTableView:create(cc.size(myTableBg:getContentSize().width-4,myTableBg:getContentSize().height-4))
    myTable:setPosition(2,2)
    myTable:setInertia(true) --设置惯性
    myTable:setBounceable(true)
    myTable:setDirection(zctech.SCROLLVIEW_DIRECTION_VERTICAL) --设置横向纵向
    myTable:setDelegate()
    myTable:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    
    myTableBg:addChild(myTable)
    
    local function cellSizeForTable(table,idx)
        return 92, tableSize.width-4
    end
    local function numberOfCellsInTableView(table)
        return #self._rewardData
    end
    local function tableCellAtIndex(table,idx)
        local nowIdx = idx + 1
        local cell = table:dequeueCell()
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            cell:setContentSize(cc.size(tableSize.width-4, 92))
        else
            cell:removeAllChildren()
        end
        self:updateCell(cell, nowIdx)
        return cell
    end
    myTable:registerScriptHandler(numberOfCellsInTableView,cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    myTable:registerScriptHandler(cellSizeForTable,cc.TABLECELL_SIZE_FOR_INDEX)
    myTable:registerScriptHandler(tableCellAtIndex,cc.TABLECELL_SIZE_AT_INDEX)
    
    myTable:reloadData()
end

function TollgateHeroRewardPreviewPop:updateCell(cell, indx)
    local cellData = self._rewardData[indx]
    -- dump(cellData, "cellData =================== ")
    local cellImg = cc.Sprite:create("res/Main/wushenTower/pop_cell_img.png")
    cellImg:setAnchorPoint(0.5, 0.5)
    cellImg:setPosition(cell:getContentSize().width/2, cell:getContentSize().height/2)
    cell:addChild(cellImg)
    local cellSize = cellImg:getContentSize()
    
    local name = zc.createLabel({
        text = zc.lang("reward") .. " "..indx,
        outLine = true,
        fontSize = 30,
        pos = cc.p(100, cellSize.height/2 + 20),
        pixel = 1.3
    })
    cellImg:addChild(name)
    local desc = zc.createLabel({
        text = zc.lang("through_now_chapter_can_get"),
        outLine = true,
        fontSize = 20,
        pos = cc.p(100, cellSize.height/2 - 20),
        color = display.COLOR_BTNFONTYELLOW,
        pixel = 1.3
    })
    cellImg:addChild(desc)

    local item, info = zc.createIconByData({
        data = {
            mainType = cellData.itemType,
            type = cellData.itemId, 
            id = cellData.itemId,
            num = cellData.itemNum,
        }, 
        showDetailPop = true,
        pos = cc.p(280, cellSize.height/2), 
        color = iconcolor
    })
    item:setScale(0.65)
    cellImg:addChild(item)

end

return TollgateHeroRewardPreviewPop