--[[
    FileName: WushenshilianRewardPop.lua
    Author: andong
    Date: 2016-05-19
    Purpose: 宝箱奖励弹窗
]]
local WushenshilianRewardPop = class( "WushenshilianRewardPop", function ()
    return ZCPopLayer:create()
end)
function WushenshilianRewardPop:create(params)
    return WushenshilianRewardPop.new(params)
end
function WushenshilianRewardPop:ctor(params)
    self:initData(params)
    self:initUI()
    self:show()
end
function WushenshilianRewardPop:initData(params)
    self._params = params or {}
    self._callback = params.callback
    self._info = params["info"] or {}
    self._rewardRecord = params["rewardRecord"] or {}

    local boxRewardStr = self._info["boxReward"] or ""
    -- local gemRewardStr = self._info["gemReward"] or ""
    local randomGemId = self._info["gemRewardUi"] or ""

    local boxReward = string.split(boxRewardStr, ",")
    -- local gemReward = string.split(gemRewardStr, ",")
    self._status = self._info["status"]
    --处理奖励，普通奖励和宝石奖励
    self._rewardData = {}
    if boxReward and next(boxReward) then
        for i = 1, #boxReward do
            local v = string.split(boxReward[i], "|")
            if v and v[2] then
                local idx = #self._rewardData + 1
                self._rewardData[idx] = {}
                self._rewardData[idx].mainType = v[1]
                self._rewardData[idx].type = v[2]
                self._rewardData[idx].itemId = v[2]
                self._rewardData[idx].num = v[3]
                self._rewardData[idx].random = false
                if v[4] and tonumber(v[4]) < 1 then
                    self._rewardData[idx].random = true
                end
            end
        end
    end

    --随机掉落宝石
    if randomGemId ~= "" and type(randomGemId) == "number" then
        local idx = #self._rewardData + 1
        self._rewardData[idx] = {}
        self._rewardData[idx].mainType = 12
        self._rewardData[idx].type = randomGemId
        self._rewardData[idx].itemId = randomGemId
        self._rewardData[idx].num = 1
        self._rewardData[idx].random = false
    end
end
function WushenshilianRewardPop:initUI()
    local popBg = cc.Sprite:create("res/Main/wushenTower/pop_reward.png")
    popBg:setPosition(zc.width/2, zc.height/2)
    self:addContent(popBg)
    self._popBg = popBg
    local close = zc.createBackButton({
        pos = cc.p(popBg:getContentSize().width-10, popBg:getContentSize().height-10),
        listener = function () 
            self:hide()
        end,
    })
    popBg:addChild(close)

    self:createReward()

    local getBtn = zc.createCommomBtn({
        tag =1,
        btnText = "",
        needSwallow = false,
        anchor = cc.p(0.5, 0.5),
        pos = cc.p(popBg:getContentSize().width/2, 65),
        endCallback = function() end,
    })
    popBg:addChild(getBtn)

    local btnText = ""
    local btnColor = display.COLOR_DEEPGRAY
    local btnState = 1
    if self._status == 2 then
        getBtn:setEnable(false)
        getBtn:setText(LANGUAGE_TOLLGATEHERO.fetched)
    elseif self._status == 0 then
        getBtn:setEnable(false)
        getBtn:setText(zc.lang("un_finish"))
    else
        getBtn:setText(LANGUAGE_TOLLGATEHERO.can_fetch)
        btnState = 2
    end

    local btnLab = zc.createLabel({
        text = btnText,
        stroke = true,
        fontSize = 24,
        pos = cc.p(getBtn:getContentSize().width/2, getBtn:getContentSize().height/2),
        color = btnColor,
        pixel = 1.3
    })
    getBtn:addChild(btnLab, 1)
    

    getBtn:setTouchEndedCall(function()
        if self._status == 2 then
            zc.tipInfo(zc.lang("reward") .. LANGUAGE_TOLLGATEHERO.fetched .. "！")
        elseif self._status == 0 then
            zc.tipInfo(zc.lang("reward") .. zc.lang("un_finish") .. "！")
        else
            zc.sendHttp({
                apiName = "Wushenshilian_treasure",
                params = {awardId=self._info.id}, 
                successCallback = function(net_data)
                    self._status = 2 
                    getBtn:setEnable(false)
                    getBtn:setText(LANGUAGE_TOLLGATEHERO.fetched)
                    net_data.noAnimation = true
                    PlayerData:updataData(net_data)
                    zc.createShowReward(net_data)
                    if self._callback then
                        self._callback(self._info.id)
                    end
                end,
            })
        end

    end)

end

function WushenshilianRewardPop:createReward()
    local tableHeight = 132
    if table.nums(self._rewardData) > 4 then
        tableHeight = 132 + 100
    end
    local tableSize = cc.size(435, tableHeight)
    local tableNode = self._popBg
    local myTableBg = ccui.Scale9Sprite:create(cc.rect(26,25,1,1), "res/Main/wushenTower/pop_tableBg.png")
    myTableBg:setContentSize(tableSize)
    myTableBg:setAnchorPoint(cc.p(0.5, 0.5))
    myTableBg:setPosition(cc.p(tableNode:getContentSize().width/2, tableNode:getContentSize().height/2 + 20))
    tableNode:addChild(myTableBg)    

    local innerHeight = tableHeight
    for i = 1, table.nums(self._rewardData) do
        local modI = i <= 4 and (i-1) or ((i-1)%4)
        local posX = 72 + modI*98
        local posY = innerHeight-66-100*( math.floor((i-1)/4) )

        local reward = self._rewardData[i]
        local item, info = zc.createIconByData({
            data = {
                mainType = reward.mainType,
                type = reward.type, 
                id = reward.itemId,
                num = reward.num,
            }, 
            showDetailPop = true,
            pos = cc.p(posX, posY), 
        })
        local _scale = 87/125
        item:setScale(_scale)
        myTableBg:addChild(item)

        if reward.random then
            local random = zc.createSprite("res/Main/wushenTower/drop_random.png")
            random:setAnchorPoint(cc.p(0,1))
            local itemSize = item:getBoundingBox()
            random:setPosition( posX-itemSize.width/2-1, posY + itemSize.height/2+1)
            random:setScale(0.9)
            myTableBg:addChild(random)
        end
    end
end


function WushenshilianRewardPop:createTableView()
    local tableSize = cc.size(435, 200)
    local tableNode = self._popBg
    local myTableBg = ccui.Scale9Sprite:create(cc.rect(26,25,1,1), "res/Main/wushenTower/pop_tableBg.png")
    myTableBg:setContentSize(tableSize)
    myTableBg:setAnchorPoint(cc.p(0.5, 0.5))
    myTableBg:setPosition(cc.p(tableNode:getContentSize().width/2, tableNode:getContentSize().height/2 + 20))
    tableNode:addChild(myTableBg)
    local myTable = zc.ZCTableView:create(cc.size(myTableBg:getContentSize().width-4,myTableBg:getContentSize().height-4))
    myTable:setPosition(2,2)
    myTable:setInertia(true) --设置惯性
    myTable:setBounceable(true)
    myTable:setDirection(zctech.SCROLLVIEW_DIRECTION_VERTICAL) --设置横向纵向
    myTable:setDelegate()
    myTable:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    
    myTableBg:addChild(myTable)
    
    local function cellSizeForTable(table,idx)
        return 92, tableSize.width-4
    end
    local function numberOfCellsInTableView(table)
        return #self._rewardData
    end
    local function tableCellAtIndex(table,idx)
        local nowIdx = idx + 1
        local cell = table:dequeueCell()
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            cell:setContentSize(cc.size(tableSize.width-4, 92))
        else
            cell:removeAllChildren()
        end
        self:updateCell(cell, nowIdx)
        return cell
    end
    myTable:registerScriptHandler(numberOfCellsInTableView,cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    myTable:registerScriptHandler(cellSizeForTable,cc.TABLECELL_SIZE_FOR_INDEX)
    myTable:registerScriptHandler(tableCellAtIndex,cc.TABLECELL_SIZE_AT_INDEX)
    
    myTable:reloadData()
end

function WushenshilianRewardPop:updateCell(cell, indx)
    local cellData = self._rewardData[indx]
    local cellImg = cc.Sprite:create("res/Main/wushenTower/pop_cell_img.png")
    cellImg:setAnchorPoint(0.5, 0.5)
    cellImg:setPosition(cell:getContentSize().width/2, cell:getContentSize().height/2)
    cell:addChild(cellImg)
    local cellSize = cellImg:getContentSize()
    
    local name = zc.createLabel({
        text = zc.lang("reward") .. " "..indx,
        stroke = true,
        fontSize = 30,
        pos = cc.p(100, cellSize.height/2 + 20),
        pixel = 1.3
    })
    cellImg:addChild(name)

    local reward = cellData
    local descStr = ""
    if reward and reward.random then
        descStr = zc.lang("may_reward")
    end
    local desc = zc.createLabel({
        text = descStr,
        stroke = true,
        fontSize = 20,
        pos = cc.p(100, cellSize.height/2 - 20),
        color = display.COLOR_BTNFONTYELLOW,
        pixel = 1.3
    })
    cellImg:addChild(desc)

    local item, info = zc.createIconByData({
        data = {
            mainType = reward.mainType,
            type = reward.type, 
            id = reward.itemId,
            num = reward.num,
        }, 
        showDetailPop = true,
        pos = cc.p(280, cellSize.height/2), 
        color = iconcolor
    })
    item:setScale(0.65)
    cellImg:addChild(item)

end
return WushenshilianRewardPop